/* include/hash.h
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: hash.h,v 1.14 2005/05/05 21:49:17 romexzf Exp $
 */

#ifndef HAVEINC_hash
#define HAVEINC_hash

/* 	doivent tre des multiples de 2 (1024, 512, 256)
	pour pouvoir (&)and-er au lieu de (%)moduler*/
#define CHANHASHSIZE 512 /* taille de la hash des chan avec une hash de 200 et 200 chan regs
						131 offsets utilise, temps rduit par 10 pour getchaninfo()*/
#define USERHASHSIZE 2048
#define NICKHASHSIZE 1024
#define CMDHASHSIZE 128

#define SUSPEND_REMOVE(x) do { 					\
	if(x) {										\
		if((x)->timer) timer_remove((x)->timer); \
		free(x);								\
	}											\
} while(0)

/* chan */
extern unsigned int do_hashc(const char *);
extern int hash_delchan(aChan *);
extern aChan *getchaninfo(const char *);
extern aChan *add_chan(const char *, const char *);
extern void del_chan(aChan *, int, const char *);

/* nick */
extern unsigned int base64toint(const char *);
extern aNick *num2nickinfo(const char *);
extern aNick *getnickbynick(const char *);
extern aNick *add_nickinfo(const char *, const char *, const char *, const char *,
			const char *, aServer *, const char *, time_t, const char *);
extern void del_nickinfo(const char *, const char *);
extern int switch_nick(aNick *, const char *);
extern int purge_nickandserv(void);

/* user */
extern int switch_user(anUser *, const char *);
extern anUser *getuserinfo(const char *);
extern anUser *add_regnick(const char *, const char *, time_t, int, int, const char *, int);
extern void del_regnick(anUser *, int, const char *);

/* cmd */
extern int RegisterCmd(const char *, int, int, int, int (*) (aNick *, aChan *, int, char **));
extern int HashCmd_switch(aHashCmd *, const char *);
extern aHashCmd *FindCommand(const char *);
extern aHashCmd *FindCoreCommand(const char *);

/* misc */
extern anUser *GetUserIbyMail(const char *);
extern int ChanLevelbyUserI(anUser *, aChan *);
extern aJoin *getjoininfo(aNick *, const char *);
extern aJoin *GetJoinIbyC(aNick *, aChan *);
aDNR *find_dnr(const char *, int);
extern char *IsAnOwner(anUser *);
extern anAccess *GetAccessIbyUserI(anUser *, aChan *);
extern aNick *GetMemberIbyNum(aChan *, const char *);
extern aNick *GetMemberIbyNick(aChan *, const char *);
extern aServer *num2servinfo(const char *);
extern aServer *GetLinkIbyServ(const char *);

extern int handle_suspend(struct suspendinfo **, const char *, const char *, time_t);
extern void do_suspend(struct suspendinfo **, const char *, const char *, time_t, time_t);

#endif /*HAVEINC_hash*/
