/* src/admin_chan.c - commandes admin pour gerer les salons
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_chan.c,v 1.82 2005/05/06 22:56:10 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "admin_chan.h"
#include "add_info.h"
#include "del_info.h"
#include "hash.h"
#ifdef SQLLOG
#include "sql_log.h"
#include <mysql.h>
#endif

#define MAXMATCHES 40

int admin_chan(aNick *nick, aChan *chan, int parc, char **parv)
{
	char *c = NULL;
	const char *arg = parv[1];
	int i = 0, m = 0;

	if(parc > 1) chan = getchaninfo((c = parv[2]));

	if(!strcasecmp(arg, "suspend"))
	{
		time_t timeout = 0;
		char *ptr = NULL;

		if(parc < 2) return csntc(nick, "Syntaxe: %s SUSPEND <salon> [arg]", parv[0]);

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		if(parc >= 3) /* do more parsing */
		{
			if(*parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
				return csntc(nick, GetReply(nick, L_INCORRECTDURATION));
			ptr = parv2msg(parc, parv, timeout ? 4 : 3, 250);
		}

		switch(handle_suspend(&chan->suspend, nick->user->nick, ptr, timeout))
		{
			case -1:
				return csntc(nick, "Veuillez prciser une raison pour suspendre ce salon.");
			case 1:
				cspart(chan, chan->suspend->raison);
				putlog(LOG_CHANS, "SUSPEND ON %s par %s@%s (%s)", c, nick->nick,
					nick->user->nick, chan->suspend->raison);
		}
		/* report.. */
		if(IsSuspend(chan)) show_suspend(nick, chan);
		else if(!CJoined(chan))
		{
			csntc(nick, "Le salon \2%s\2 n'est plus suspendu.", c);
			csjoin(chan, 0);
			putlog(LOG_CHANS, "SUSPEND OFF %s par %s@%s", c, nick->nick, nick->user->nick);
		}
	}
	else if(!strcasecmp(arg, "del"))
	{
		char tmp[300];
		if(parc < 2) return csntc(nick, "Syntaxe: %s DEL <salon> <raison>", parv[0]);

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		snprintf(tmp, sizeof tmp, "Delchan par %s@%s (%s)", nick->nick,
			nick->user->nick, (parc > 2) ? parv2msg(parc, parv, 3, 200) : "Aucune raison");

		csntc(nick, "Le salon %s a bien t supprim.", c);
		del_chan(chan, HF_SQLLOG, tmp);
	}
	else if(!strcasecmp(arg, "reg"))
	{
		anUser *u;
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s REG <salon> <Username|%%nick> [thme]", parv[0]);

		if(chan) return csntc(nick, GetReply(nick, L_ALREADYREG), c);

		if(!chan_check(c, nick)) return 0;

		if(!(u = ParseNickOrUser(nick, parv[3]))) return 0;

		chan = add_chan(c, (parc > 3) ? parv2msg(parc, parv, 4, 300) : "Adminreg");
		add_access(u, c, OWNERLEVEL, A_OP | A_PROTECT, CurrentTS);
		csjoin(chan, 1);
		csntc(nick, "%s a t enregistr pour %s.", c, u->nick);
	}
	else if(!strcasecmp(arg, "list"))
	{
		int description = getoption("-description", parv, parc, 2, 0);
		int modes = getoption("-modes", parv, parc, 2, 0);
		int users = getoption("-users", parv, parc, 2, 1);
		int defmodes = getoption("-defmodes", parv, parc, 2, 0);
		int url = getoption("-url", parv, parc, 2, 0);
		int max = getoption("-count", parv, parc, 2, 1);
		int topic = getoption("-topic", parv, parc, 2, 0), j = 0;

		if(parc < 2 || *parv[2] == '-') m = 1;
		if(modes) modes = cmodetoflag(0, parv[modes]);
		if(defmodes) defmodes = cmodetoflag(0, parv[defmodes]);
		if(!max) max = strcmp("-all", parv[parc]) ? MAXMATCHES : -1;

		for(;j < CHANHASHSIZE;++j) for(chan = chan_tab[j];chan;chan = chan->next)
			if((m || !match(parv[2], chan->chan))
			&& (!description || !match(parv[description], chan->description))
			&& (!modes || HasMode(chan, modes) == modes)
			&& (!defmodes || HasDMode(chan, defmodes) == defmodes)
			&& (!url || !match(parv[url], chan->url))
			&& (!topic || !match(parv[topic], chan->topic))
			&& chan->users >= users && (++i <= max || max < 0))
			{
				if(i == 1) csntc(nick, "\2Liste des salons enregistrs :");
				if(!IsSuspend(chan)) csntc(nick, "  %s (%s) [%d]", chan->chan, GetCModes(chan->mode), chan->users);
				else csntc(nick, "  %s (%s) (\2Suspendu\2)", chan->chan, GetCModes(chan->defmodes));
			}

		if(i > MAXMATCHES && max == MAXMATCHES)
			csntc(nick, GetReply(nick, L_EXCESSMATCHES), i, MAXMATCHES);
		if(max > 0 && max < i)
			csntc(nick, "Un Total de %d entres trouves (%d listes)", i, max);
		else if(i) csntc(nick, "Un Total de %d entres trouves", i);
		else csntc(nick, "Aucun salon enregistr correspondant trouv.");
	}
	else if(!strcasecmp(arg, "suspendlist"))
	{
		int from = getoption("-from", parv, parc, 2, 0), j = 0;
		if(parc < 2 || *parv[2] == '-') m = 1;

		for(;j < CHANHASHSIZE;++j) for(chan = chan_tab[j];chan;chan = chan->next)
			if(IsSuspend(chan) && (m || !match(parv[2], chan->chan))
				&& (!from || !strcasecmp(parv[from], chan->suspend->from)))
			{
				if(!i++) csntc(nick, "\2Liste des salons suspendus :");
				csntc(nick, "  %s Par %s Expire %s Raison: %s",
						chan->chan, chan->suspend->from,
						chan->suspend->expire ? get_time(nick, chan->suspend->expire) : "Jamais",
						chan->suspend->raison);
			}

		csntc(nick, "Il y a un total de\2 %d\2 salons suspendus correspondant  vos critres.", i);
	}
	else if(!strcasecmp(arg, "join"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s JOIN <salon> [-force]", parv[0]);

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		if(IsSuspend(chan))
			return csntc(nick, "%s est actuellement suspendu pour %s, utilisez '%s SUSPEND %s' pour lever le suspend.",
				c, chan->suspend->raison, parv[0], c);

		if(CJoined(chan) && !getoption("-force", parv, parc, 3, -1))
			return csntc(nick, "Je suis dj sur %s (En cas de desynch, utilisez l'argument '-force')", c);

		csjoin(chan, 1); /* nicklist restaure dans le csjoin */
	}
	else if(!strcasecmp(arg, "part"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s PART <salon> [raison]", parv[0]);

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		cspart(chan, (parc > 2) ? parv2msg(parc, parv, 3, 300) : "Aucune Raison");
	}
	else if(!strcasecmp(arg, "setowner"))
	{/* chan setowner # gna [leve]*/
		anUser *newu;
		anAccess *newo;
		int newlevel = 0;

		if(parc < 3) return csntc(nick, "Syntaxe: %s SETOWNER <chan> <nouvel owner> [nouveau level de l'ancien owner]", parv[0]);
		if(parc > 3 && ((newlevel = strtol(parv[4], NULL, 10)) <= 0 || newlevel >= OWNERLEVEL))
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);
		if(!(newu = getuserinfo(parv[3]))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[3]);

		if(chan->owner->user == newu)/* le futur owner l'est dj */
			return csntc(nick, "%s est dj l'owner de %s.", newu->nick, c);
		else if(newlevel) chan->owner->level = newlevel;
		else del_access(chan->owner->user, chan);

		if((newo = GetAccessIbyUserI(newu, chan))) newo->level = OWNERLEVEL, chan->owner = newo;
		else chan->owner = add_access(newu, c, OWNERLEVEL, (A_OP | A_PROTECT), CurrentTS);

		csntc(nick, "%s est le nouvel owner de %s.", newu->nick, c);
	}
#ifdef SQLLOG
	else if(!strcasecmp(arg, "historique"))
	{
		time_t date = 0;
		MYSQL_RES *result;
		MYSQL_ROW row;
		int cmd, user;
		char tmp[250] = "SELECT * FROM "SQLCHAN" WHERE chan='%s' AND TS>=%T";

		if(parc < 2) return csntc(nick, "Syntaxe: %s HISTORIQUE <salon> [%%XjXh] [-cmd <cmd>] [-user <user>]", parv[0]);
		if(parc >= 3 && *parv[3] == '%' && (date = convert_duration(++parv[3])) <= 0)
			return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

		if((cmd = getoption("-cmd", parv, parc, 3, 0))) strcat(tmp, " AND cmd='%s'");
		if((user = getoption("-user", parv, parc, 3, 0))) strcat(tmp, " AND user='%s'");
		strcat(tmp, " order by TS desc LIMIT 0,100");

		if(sql_query(SQL_QRAW, tmp, c, date ? CurrentTS - date : 0,
			cmd ? parv[cmd] : (user ? parv[user] : ""), user ? parv[user] : "") < 0)
				return csntc(nick, "La requte SQL a chou.");

		if(!(result = mysql_store_result(bot.sql_id)) || !(row = mysql_fetch_row(result)))
			return csntc(nick, "Aucune entre trouve.");

		if(date) csntc(nick, "Commandes effectues sur \2%s\2 (Par %s) depuis %s.",
					c, user ? parv[user] : "tout le monde", duration(date));
		else csntc(nick, "Commandes effectues sur \2%s\2 (Par %s).",
				c, user ? parv[user] : "tout le monde");

		for(;row;row = mysql_fetch_row(result))
			csntc(nick, "%s: \2%s\2 [%s] par \2%s\2",
				get_time(nick, strtol(row[2], NULL, 10)), row[3], NONE(row[4]), row[0]);

		mysql_free_result(result);
	}
#endif
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), arg);
	return 1;
}

int whoison(aNick *nick, aChan *chan, int parc, char **parv)
{
	int first = 1, size = 0;
	char buf[451] = {0}, tmp[NICKLEN + 16];
	aLink *lp = chan->members;

	if(!lp) return csntc(nick, "Aucun User sur %s.", chan->chan);
	for(;lp;lp = lp->next)
	{
		int tmps = size, p = fastfmt(tmp, " $$", GetChanPrefix(lp->value.j->nick, lp->value.j), lp->value.j->nick->nick);
		size += p;
		if(size >= 450)
		{
			if(first) csntc(nick, "Un total de %d Users sur %s:%s...", chan->users, chan->chan, buf);
			else csntc(nick, "              :%s...", buf);
			first = 0;
			strcpy(buf, tmp);
			size = p;
		}
		else strcpy(&buf[tmps], tmp);
	}
	if(first && *buf) csntc(nick, "Un total de %d Users sur %s:\2%s", chan->users, chan->chan, buf);
	else if(*buf) csntc(nick, "              :%s", buf);

	return 1;
}

void show_suspend(aNick *nick, aChan *chan)
{
	if(!chan->suspend) return; /* on aurait jamais du l'envoyer l*/
	if(IsSuspend(chan))
	{
		char buf[200];
		Strncpy(buf, get_time(nick, chan->suspend->debut), sizeof buf -1);
		csntc(nick, "Le salon %s \2est\2 suspendu par %s (le %s). Expire %s",
			chan->chan,	chan->suspend->from, buf,
			chan->suspend->expire ? get_time(nick, chan->suspend->expire) : "Jamais");
	}
	else csntc(nick, "Le salon tait suspend par %s", chan->suspend->from);
	csntc(nick, "Raison: %s", chan->suspend->raison);
}
