/* src/admin_cmds.c - Diverses commandes pour admins
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_cmds.c,v 1.85 2005/04/26 23:01:48 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "add_info.h"
#include "del_info.h"
#include "hash.h"
#include "config.h"
#include "divers.h"
#include "showcommands.h"
#include "aide.h"
#include "timers.h"
#include "template.h"
#include "socket.h"
#ifdef SQLLOG
#include <mysql.h>
#include "sql_log.h"
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif

int inviteme(aNick *nick, aChan *chaninfo, int parc, char **parv)
{ 	/* invite des admins sur le salon d'infos */
    putserv("%s " TOKEN_INVITE " %s :%s", cs.num, nick->nick, bot.pchan);
    return 1;
}

void CleanUp(void)
{
	aChan *c, *ct;
	aBan *b, *bt;
	anUser *u, *ut;
	anAccess *a, *at;
	aDNR *dnr = dnrhead, *dt;
#ifdef USE_MEMOSERV
	aMemo *m, *mt;
#endif
	struct ignore *ii = ignorehead, *it;
	int i = 0;

	free(quit_msg);
	free(mailprog);
	free(pasdeperm);
	free(network);

#ifdef SQLLOG
	sql_flush(SQL_QINSERTU);
	sql_flush(SQL_QINSERTC);
	if(bot.sql_id) mysql_close(bot.sql_id);
	free(bot.sql_host);
#endif

	sockets_close();
	purge_nickandserv();

	for(i = 0;i < CHANHASHSIZE;++i) for(c = chan_tab[i];c;c = ct)
	{
		ct = c->next;
		for(b = c->banhead;b;free(b->raison), free(b->mask), free(b), b = bt) bt = b->next;
		if(c->suspend) free(c->suspend);
		if(c->motd) free(c->motd);
		free(c);
	}
	for(i = 0;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = ut)
	{
		ut = u->next;
		for(a = u->accesshead;a;free(a), a = at)
		{
			at = a->next;
			if(a->info) free(a->info);
		}
#ifdef USE_MEMOSERV
		for(m = u->memohead;m;free(m), m = mt) mt = m->next;
#endif
		if(u->suspend) free(u->suspend);
		if(u->lastlogin) free(u->lastlogin);
		free(u);
	}
	for(;dnr;free(dnr->mask), free(dnr->raison), free(dnr), dnr = dt) dt = dnr->next;
	for(;ii;free(ii), ii = it) it = ii->next;
	return;
}

int die(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *r = parc ? parv2msg(parc, parv, 1, 300) : quit_msg;

	running = 0;

	putserv("%s " TOKEN_QUIT " :%s", cs.num, r);
#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s", ws.num, r);
#endif
	putserv("%s " TOKEN_SQUIT " %s 0 :%s", bot.servnum, bot.server, r);
	return 1;
}

/*
 * restart_bot parv[1->parc-1] = reason
 */
int restart_bot(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	FILE *fuptime;
	char *r = parc ? parv2msg(parc, parv, 1, 300) : quit_msg;

	if((fuptime = fopen("uptime.tmp", "w"))) /* permet de maintenir l'uptime (/me tricheur) */
	{
		fprintf(fuptime, "%lu", bot.uptime);
		fclose(fuptime);
	}

	running = 0;
	ConfFlag |= CF_RESTART;

	putserv("%s " TOKEN_QUIT " :%s [\2Restarting\2]", cs.num, r);
#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s [\2Restarting\2]", ws.num, r);
#endif
	putserv("%s "TOKEN_SQUIT" %s 0 :%s [\2RESTART\2]", bot.servnum, bot.server, r);
	return 1;
}

int chcomname(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aHashCmd *cmd;
	const char *newcmd = parv[2], *lastcmd = parv[1];

	if(strlen(newcmd) > CMDLEN)
		return csntc(nick, "Un nom de commande ne doit pas dpasser 14 caractres.");

	if(!(cmd = FindCommand(lastcmd)) || *cmd->name == '\1')
		return csntc(nick, GetReply(nick, L_NOSUCHCMD), lastcmd);

	if(FindCommand(newcmd)) return csntc(nick, "La commande %s existe dj.", newcmd);

	HashCmd_switch(cmd, newcmd);
	csntc(nick, "La commande\2 %s\2 s'appelle maintenant\2 %s\2.", lastcmd, cmd->name);
	BuildCommandsTable(1);
	write_cmds();
	return 1;
}

int chlevel(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int level, adm = getoption("-admin", parv, parc, 2, -1);
	aHashCmd *cmdp;

	if(!(cmdp = FindCommand(cmd))) return csntc(nick, GetReply(nick, L_NOSUCHCMD), cmd);

	if(!is_num(parv[2]) || (level = atoi(parv[2])) > OWNERLEVEL || level < 0)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if(level == cmdp->level && !adm)
		return csntc(nick, "%s est dj au niveau %d.", cmd, level);

	if(!NeedNoAuthCmd(cmdp) && !level)
		return csntc(nick, "La commande %s ne peut tre utilise que par des users"
			" identifis sur les services. Le niveau minimal est donc de 1.", cmd);

	if(!ChanCmd(cmdp))
	{
		if(level > MAXADMLVL) return csntc(nick, "Le niveau maximum Administrateur est %d.", MAXADMLVL);
		else if(level > nick->user->level)
			return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));
		else
		{ /* dtermination du niveau admin ou pas car level 0-4 */
			if(level > 1) cmdp->flag |= CMD_ADMIN;
			else cmdp->flag &= ~CMD_ADMIN;
		}
	}
	if(ChanCmd(cmdp) && adm)
	{	 /* argument de switch admin pour les cmds chans (0-500 <= donc ncessaire) */
		if(AdmCmd(cmdp)) cmdp->flag &= ~CMD_ADMIN;
		else cmdp->flag |= CMD_ADMIN;
	}

	cmdp->level = level;

	csntc(nick, "Le niveau de la commande %s est maintenant\2 %d\2 (%s).",
				cmdp->name, cmdp->level, AdmCmd(cmdp) ? "Admin" : "User");
	write_cmds();
	BuildCommandsTable(1);
	return 1;
}

int disable_cmd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int disable = 0;
	aHashCmd *cmdp = FindCommand(cmd);

	if(!cmdp) return csntc(nick, GetReply(nick, L_NOSUCHCMD), cmd);

	if(!strcasecmp(parv[2], "ON")) disable = 1;
	else if(strcasecmp(parv[2], "OFF")) return csntc(nick, GetReply(nick, L_SHOULDBEONOFF));

	if(!strcasecmp(cmdp->corename, "DISABLE")
		|| !strcasecmp(cmdp->corename, "CHCOMNAME")
		|| !strcasecmp(cmdp->corename, "CHLEVEL")
		|| !strcasecmp(cmdp->corename, "LOGIN")) /* a nuirait  tout.. */
			return csntc(nick, "Veuillez ne pas desactiver \2%s\2.", cmd);

	if(disable) cmdp->flag |= CMD_DISABLE;
	else cmdp->flag &= ~CMD_DISABLE;

	csntc(nick, "La commande %s est maintenant \2%sactive\2", cmd, disable ? "des" : "");
	BuildCommandsTable(1);
	write_cmds();
	return 1;
}

#ifdef USE_WELCOMESERV
int set_motds(aNick *nick, aChan *c, int parc, char **parv)
{
	char *tmp;
	if(!strcasecmp(parv[1], "LIST"))
	{
		csntc(nick, "Motd User : \2%s\2", user_motd);
		csntc(nick, "Motd Admin: \2%s\2", admin_motd);
		return 1;
	}
	else if(parc < 2) return syntax_cmd(nick, FindCoreCommand("adminmotd"));
	else if(!strcasecmp(parv[1], "USER")) tmp = user_motd;
	else if(!strcasecmp(parv[1], "ADMIN")) tmp = admin_motd;
	else return csntc(nick, "Il n'y a pas de motd pour %s", parv[1]);

	if(strcasecmp("none", parv[2]))
	{
		parv2msgn(parc, parv, 2, tmp, 390);
		csntc(nick, "Le motd pour les %ss est maintenant:\2 %s", parv[1], tmp);
	}
	else *tmp = 0, csntc(nick, "Le motd pour les %ss a bien t supprim", parv[1]);

	write_welcome();
	return 1;
}
#endif

/* cmds (c) keeo (le 3 en 1 par progs) */
int globals_cmds(aNick *nick, aChan *chan, int parc, char **parv)
{
	const char *cmd = parv[1];
	int i = 0;

	if(!strcasecmp(cmd, "MSG"))
	{
		char *msg = parv2msg(parc, parv, 2, 400);

		if(msg && *msg == ':') msg++;
		for(;i < CHANHASHSIZE;++i) for(chan = chan_tab[i];chan;chan = chan->next)
			if(chan->users > 1)	putserv("%s "TOKEN_PRIVMSG" %s :\2%s>\2 %s", cs.num,
										chan->chan, nick->nick, msg);
	}

#ifdef USE_MEMOSERV
	else if(!strcasecmp(cmd, "MEMO"))
	{
		char memo[MEMOLEN + 1];
		anUser *user;
		aNick *who;
		int min_level = getoption("-min", parv, parc, 2, 1);
		int max_level = getoption("-max", parv, parc, 2, 1);
		int mask = getoption("-user", parv, parc, 2, 0);
		int owner = getoption("-owner", parv, parc, 2, -1);

		for(i = 2;i < parc && *parv[i] == '-';i += 2);

		parv2msgn(parc, parv, i, memo, MEMOLEN);

		if(max_level > 4 || min_level > 4)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		for(i = 0;i < USERHASHSIZE;++i) for(user = user_tab[i];user;user = user->next)
			if(user != nick->user && (!min_level || user->level >= min_level)
			   && (!max_level || user->level <= max_level)
			   && (!mask || !match(parv[mask], user->nick))
			   && (!owner || IsAnOwner(user)))
			{
				who = user->n;
				if(who && !IsAway(who))
					csntc(who, "\2MEMO DE %s :\2 %s", nick->nick, memo);
				add_memo(user, nick->user->nick, CurrentTS, memo, (who && !IsAway(who)) ? MEMO_READ : 0);
			}

		csntc(nick, "Un memo a bien t envoy avec les critres spcifis avec le texte suivant :");
		csntc(nick, " \2 %s\2", memo);
	}
#endif
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), cmd);
	return 1;
}

int dnr_manage(aNick *nick, int type, int parc, char **parv)
{
	if(!strcasecmp(parv[1], "list"))
	{
		int from = getoption("-from", parv, parc, 2, 0);
		int mask = getoption("-match", parv, parc, 2, 0), count = 0;
		aDNR *dnr = dnrhead;

		for(;dnr;dnr = dnr->next)
			if(dnr->flag & type && (!from || !strcasecmp(parv[from], dnr->from))
				&& (!mask || !match(parv[mask], dnr->mask)))
			{
				csntc(nick, "DNRMask: \2%s\2 Auteur: \2%s\2 Pos le:\2 %s\2 Raison: %s",
					dnr->mask, dnr->from, get_time(nick, dnr->date), dnr->raison);
				++count;
			}
		csntc(nick, "%d matches correspondantes.", count);
	}
	else if(!strcasecmp(parv[1], "add") && parc >= 2)
	{
		aDNR *dnr = find_dnr(parv[2], type | DNR_MASK);
		int both = getoption("-both", parv, parc, 3, -1) ? DNR_TYPECHAN|DNR_TYPEUSER : type;

		if(dnr)
			return csntc(nick, "%s est dj un DNR mask (Ajout par %s, le %s [%s])",
				dnr->mask, dnr->from, get_time(nick, dnr->date), dnr->raison);

		add_dnr(parv[2], nick->user->nick,
			parc > (both != type ? 3 : 2)
			? parv2msg(parc, parv, both != type ? 4 : 3, 250) : "Service",
			CurrentTS, HasWildCard(parv[2]) ? both | DNR_MASK : both);
		csntc(nick, "%s a t ajout aux DNR masks.", parv[2]);
		write_dnr();
	}
	else if(!strcasecmp(parv[1], "del") && parc >= 2)
	{
		aDNR *dnr = find_dnr(parv[2], type);

		if(!dnr) return csntc(nick, "%s n'est pas un DNR mask.", parv[2]);
		del_dnr(dnr);
		csntc(nick, "DNR mask supprim.");
		write_dnr();
	}
	else csntc(nick, "Syntaxe: %s (ADD|DEL) <dnr-mask> [raison]", parv[0]);
	return 0;
}

int dnrchan_manage(aNick *nick, aChan *c, int parc, char **parv)
{
	return dnr_manage(nick, DNR_TYPECHAN, parc, parv);
}

int dnruser_manage(aNick *nick, aChan *c, int parc, char **parv)
{
	return dnr_manage(nick, DNR_TYPEUSER, parc, parv);
}

int rehash_conf(aNick *nick, aChan *chan, int parc, char **parv)
{
	int langs;
	Lang *l;

#ifdef SQLLOG
	if(getoption("-sql", parv, parc, 1, -1))
	{
		sql_flush(SQL_QINSERTU);
		sql_flush(SQL_QINSERTC);
		return csntc(nick, "SQL buffers flushed.");
	}
#endif
	if(getoption("-purge", parv, parc, 1, -1))
	{
		check_accounts();
		check_chans();
		return csntc(nick, "Purge effectue.");
	}
	if(getoption("-template", parv, parc, 1, -1))
	{
		tmpl_load();
		return csntc(nick, "Templates reloaded from disk.");
	}
	if((langs = getoption("-aide", parv, parc, 1, 0)))
	{
		if((l = lang_isloaded(parv[langs])))
		{
			help_load(l);
			csntc(nick, "Help should have been reload for language %s.", l->langue);
		}
		else csntc(nick, GetReply(nick, L_NOSUCHLANG), parv[langs]);
		return 0;
	}
	langs = LangCount; /* save old count in case of load */

	if(load_config(FICHIER_CONF) == -1)
		return csntc(nick, "Une erreur s'est produite lors de l'actualisation...");

	if(LangCount != langs)
	{
		help_cmds_realloc(langs);
		for(l = DefaultLang;l;l = l->next) if(l->id > langs) help_load(l);
		csntc(nick, "%d langs and help loaded.", LangCount - langs);
	}

	return csntc(nick, "Configuration actualise avec succs");
}

int showconfig(aNick *nick, aChan *c, int parc, char **parv)
{
	csntc(nick, "Etat des variables de configuration");
	csntc(nick, "Nom du rseau : %s Uplink: %s:%d", network, bot.ip, bot.port);
	csntc(nick, "Nom du serveur des Services : %s (%s) [Num: %s]", bot.server, bot.name, bot.servnum);
	csntc(nick, "Salon de controle : %s Salon d'aide : %s", bot.pchan, bot.chan);
	csntc(nick, "Message de quit : %s", quit_msg);
	csntc(nick, "Temps avant un kill : %s Change nick : %s", duration(kill_interval),
		GetConf(CF_CHANGENICK) ? "Oui" : "Non");
	csntc(nick, "Max LastSeen : %s", duration(maxlastseen));
	csntc(nick, "Admin exempt de flood : %s Kill pour flood : %s",
		GetConf(CF_ADMINEXEMPT) ? "Oui" : "Non", GetConf(CF_KILLFORFLOOD) ? "Oui" : "Non");
	csntc(nick, "Temps d'ignore : %s", duration(ignoretime));
	csntc(nick, "Programme de mail : %s", mailprog ? mailprog : "aucun");
	csntc(nick, "Host hidding: %s (Suffixe: %s)", GetConf(CF_HOSTHIDING) ? "Oui" : "Non", hidden_host);
#ifdef WEB2CS
	csntc(nick, "Web2Cs/Port : %d", bot.w2c_port);
#endif
	return 1;
}
