/* src/admin_manage.c - commandes pr gerer les admins
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_manage.c,v 1.19 2005/04/20 23:19:18 romexzf Exp $
 */

#include "main.h"
#include "add_info.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "hash.h"
#ifdef HAVE_TRACK
#include "track.h"
#endif

aNick **adminlist = NULL;
int adminmax = 0;

/* Try to insert a new admin in list, otherwise append it. */
int adm_active_add(aNick *nick)
{
	int i = 0;

	while(i < adminmax && adminlist[i]) ++i; /* look for a free slot */
	/* can't find one, make the list grow up */
	if(i >= adminmax) adminlist = realloc(adminlist, sizeof *adminlist * ++adminmax);
	adminlist[i] = nick; /* insert it into the free slot */
	return adminmax;
}

/* Try to find admin in list and empties the slot */
int adm_active_del(aNick *nick)
{
	int i = 0;
#ifdef HAVE_TRACK
	struct track *track = trackhead, *tt = NULL;
	for(;track;track = tt)
	{
		tt = track->next;
		if(nick == track->tracker) del_track(track->tracked);
	}
#endif
	while(i < adminmax && adminlist[i] != nick) ++i; /* searching in list.. */
	if(i < adminmax) adminlist[i] = NULL; /* free the slot */
	return 0;
}

/*
 * admin_level <username> <niveau>
 */

int admin_level(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *user;
	int level;
#ifdef USE_MEMOSERV
	char memo[MEMOLEN + 1];
#endif

	if(!is_num(parv[2]) || (level = atoi(parv[2])) > MAXADMLVL || level < 1)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if(!(user = getuserinfo(parv[1])))
		return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[1]);

	if(level >= nick->user->level && nick->user->level != MAXADMLVL)
		return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

	if(level == user->level)
		return csntc(nick, "%s est dj au niveau %d.", user->nick, level);

	if(level < ADMINLEVEL)
	{
		csntc(nick, "%s n'est plus Administrateur.", user->nick);
#ifdef USE_MEMOSERV
		if(!user->n) snprintf(memo, MEMOLEN, "Votre accs Administrateur a t supprim par %s.",
						nick->user->nick);
#endif
		if(user->n) adm_active_del(user->n);
	}
	else if(user->level < ADMINLEVEL)
	{
		csntc(nick, "%s est maintenant Administrateur au niveau\2 %d\2.", user->nick, level);
#ifdef USE_MEMOSERV
		if(!user->n) snprintf(memo, MEMOLEN, "%s vous a nomm Administrateur.", nick->user->nick);
#endif
		if(user->n) adm_active_add(user->n);
	}
	else csntc(nick, "Vous avez modifi le niveau Administrateur de \2%s\2 en\2 %d\2.",
			user->nick, level);

#ifdef USE_MEMOSERV
	if(!user->n) add_memo(user, "Services", CurrentTS, memo, MEMO_AUTOEXPIRE);
#endif
	user->level = level;
	return 1;
}
