/* src/admin_user.c - commandes admins pour gerer les users
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_user.c,v 1.136 2005/05/05 21:49:17 romexzf Exp $
 */

#include <sys/time.h>
#include "main.h"
#include "outils.h"
#include "hash.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "divers.h"
#include "crypt.h"
#ifdef SQLLOG
#include "sql_log.h"
#include <mysql.h>
#endif
#ifdef HAVE_VOTE
#include "vote.h"
#endif
#include "admin_user.h"

int cs_whois(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char chans[451] = {0}, b[CHANLEN + 4] = {0};
	int size = 0, count = 0;
	anUser *uptr;
	aJoin *join;
	aNick *nptr = getnickbynick(parv[1]);

	if(!nptr) return csntc(nick, GetReply(nick, L_NOSUCHNICK), parv[1]);

	csntc(nick, "Infos sur \2%s", nptr->nick);
	csntc(nick, " Nom: %s", nptr->name);
#ifdef HAVE_CRYPTHOST
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(nptr), nptr->nick, nptr->ident, nptr->crypt);
	csntc(nick, " Realhost: \002%s\2", nptr->host);
#else
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(nptr), nptr->nick, nptr->ident, nptr->host);
#endif
	csntc(nick, " IP:\2 %s", GetIP(nptr->base64));
	if(IsOper(nptr)) csntc(nick, " %s est \2IRCop.", nptr->nick);
	csntc(nick, " UserModes: \2%s\2", (nptr->flag & N_UMODES) ? GetModes(nptr->flag) : "Aucun");

	for(join = nptr->joinhead;join; join = join->next)
	{
		int tmps = size, p = fastfmt(b, "$$$ ", IsVoice(join) ? "+" : "", IsOp(join) ? "@" : "", join->channel);
		size += p;
		if(size >= 450) /* le dernier fait depasser le buffer */
		{
			csntc(nick, count++ ? "%s...": " Prsent sur %s...", chans);
			strcpy(chans, b); /* ajoute le suivant pour la prochaine */
			size = p;
		}
		else strcpy(&chans[tmps], b);
	}

	csntc(nick, !count ? " Prsent sur %s" : "%s", *chans ? chans : "aucun salon.");

	if(nptr->user)
	{
		anAccess *acces = nptr->user->accesshead;
		*chans = 0;
		size = count = 0;

		csntc(nick, " %s est logu sous l'username \2%s\2%s", nptr->nick, nptr->user->nick,
			IsAdmin(nptr->user) ? " (\2Admin\2)" : "");

		for(;acces;acces = acces->next)
		{
			int tmps = size, p;
			if(AWait(acces)) continue;

			size += (p = strlen(acces->c->chan));
			if(size >= 450)
			{
				csntc(nick, count++ ? "%s ..." : GetReply(nick, L_ACCESSON) , chans);
				size = p;
				strcpy(chans, acces->c->chan); /* ajoute le suivant pour la prochaine */
			}
			else strcpy(&chans[tmps], acces->c->chan);
			chans[size++] = ' ';
			chans[size] = 0;
		}

		if(*chans) csntc(nick, !count ? GetReply(nick, L_ACCESSON) : "%s", chans);
		else if(!count) csntc(nick, GetReply(nick, L_NOACCESS));
	}
	else if((uptr = getuserinfo(parv[1]))) csntc(nick, " %s est un username enregistr. %s",
				uptr->nick, IsAdmin(uptr) ? " (\2Admin\2)" : "");

	csntc(nick, " Connect depuis %s sur \2%s", duration(CurrentTS - nptr->ttmco), nptr->serveur->serv);
	return 1;
}

int admin_user(aNick *nick, aChan *chan, int parc, char **parv)
{
	char *user = parv[2];
	const char *arg = parv[1];
	anUser *u = NULL;
	aNick *n;

	if(!strcasecmp(arg, "info"))
	{
		int mail = getoption("-mail", parv, parc, 2, 0);/* critres de recherche */
		int seen = getoption("-seen", parv, parc, 2, 0);
		int date = getoption("-date", parv, parc, 2, 1);
		int max = getoption("-count", parv, parc, 2, 1);
		int auth = getoption("-auth", parv, parc, 2, -1);
		int suspend = getoption("-suspend", parv, parc, 2, -1);
		int oubli = getoption("-oubli", parv, parc, 2, -1);
		int first = getoption("-first", parv, parc, 2, -1);
		int nopurge = getoption("-nopurge", parv, parc, 2, -1), found = 0, i = 0;
		time_t seent = (seen > 0) ? convert_duration(parv[seen]) : 0;
		if(seent < 0) seent = 0;

		if(*parv[2] == '-') user = NULL;
		else if(!HasWildCard(user))
			return (u = ParseNickOrUser(nick, user)) ? show_userinfo(nick, u, 1) : 0;
		if(!max) max = strcmp("-all", parv[parc]) ? MAXMATCHES : -1;

		for(;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = u->next)
		{
			if((!mail || !match(parv[mail], u->mail))
			&& (!seent || (CurrentTS - u->lastseen) <= seent)
			&& (!date || u->lastseen <= date)
			&& (!auth || (auth && u->n) || (!auth && !u->n))
			&& (!suspend || IsSuspend(u))
			&& (!oubli || UOubli(u))
			&& (!first || UFirst(u))
			&& (!nopurge || UNopurge(u))
			&& (!user || !match(user, u->nick))
			&& (++found <= max || max < 0))
			{
				show_userinfo(nick, u, 1);
				csntc(nick, "-");
			}
		}

		if(found > MAXMATCHES && max == MAXMATCHES)
			csntc(nick, GetReply(nick, L_EXCESSMATCHES), found, MAXMATCHES);
		if(max > 0 && max < found)
			csntc(nick, "Un Total de %d entres trouves (%d listes)",	found, max);
		else if(found) csntc(nick, "Un Total de %d entres trouvs",	found);
		else csntc(nick, "Aucun Username correspondant \2 %s\2 trouv.", user ? user : "*");
	}

	else if(!strcasecmp(arg, "match"))
	{
		char *p = parv[2];
		int w = 1, host = 0, nicki = 0, ident = 0, real = 0, join = 0, par = 2;
		int mode = 0, ac = 0, count = 0, crypt = 0, serv = 0, nolimit = 0, i = 0;
		struct timeval tv;
		long ticks;
		aServer *link = NULL;

		if(*p != '+' && *p != '-')
			return csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", parv[0]);

		gettimeofday(&tv, NULL);
		ticks = tv.tv_usec;

#define ADMUSR_PARSE_MATCH(x) 	if(((x) = ++par) <= parc) { if(!w) (x) = -(x); } \
								else err = 1; \
								break; \

		for(;*p;p++)
		{
			char err = 0;
			switch(*p)
			{
				case '+': w = 1; break;
				case '-': w = 0; break;
				case 'h': ADMUSR_PARSE_MATCH(host);
				case 'n': ADMUSR_PARSE_MATCH(nicki);
				case 'u': ADMUSR_PARSE_MATCH(ident);
				case 'g': ADMUSR_PARSE_MATCH(real);
				case 'c': ADMUSR_PARSE_MATCH(join);
				case 'a': ADMUSR_PARSE_MATCH(ac);
#ifdef HAVE_CRYPTHOST
				case 'C': ADMUSR_PARSE_MATCH(crypt);
#endif
				case 'm':
						if(++par <= parc)
						{
							mode = parse_umode(0, parv[par]);
							if(!w) mode = -mode;
						}
						else err = 1;
						break;

				case 's':
						if((serv = ++par) <= parc)
						{
							if(!(link = GetLinkIbyServ(parv[serv])))
								return csntc(nick, "Serveur %s inconnu.", parv[serv]);
							if(!w) serv = -serv;
						}
						else err = 1;
						break;
				default:
						err = 1;
			}
			if(err) return csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", parv[0]);
		}

		if(!strcasecmp("-all", parv[parc])) nolimit = 1;

		for(;i < NICKHASHSIZE;i++) for(n = nick_tab[i];n;n = n->next)
			if((!nicki || (nicki > 0 && !match(parv[nicki], n->nick)) || (nicki < 0 && match(parv[-nicki], n->nick)))
				&& (!ident || (ident > 0 && !match(parv[ident], n->ident)) || (ident < 0 && match(parv[-ident], n->ident)))
				&& (!real || (real > 0 && !match(parv[real], n->name)) || (real < 0 && match(parv[-real], n->name)))
				&& (!serv || (serv > 0 && link == n->serveur) || (serv < 0 && link != n->serveur))
				&& (!host || (host > 0 && !match(parv[host], n->host)) || (host < 0 && match(parv[-host], n->host)))
				&& (!mode || (mode > 0 && (n->flag & mode) == mode) || (mode < 0 && !(n->flag & -mode)))
				&& (!ac || (n->user && ((ac > 0 && !match(parv[ac], n->user->nick)) || (ac < 0 && match(parv[-ac], n->user->nick)))))
#ifdef HAVE_CRYPTHOST
				&& (!crypt || (crypt > 0 && !match(parv[crypt], n->crypt)) || (crypt < 0 && match(parv[-crypt], n->crypt)))
#endif
				&& (!join || (join > 0 && getjoininfo(n, parv[join])) || (join < 0 && !getjoininfo(n, parv[-join])))
				&& (++count <= MAXMATCHES || nolimit))
					csntc(nick, "Matching Nick: [%d] \2%s%s\2 -> %s@%s", count, GetPrefix(n), n->nick, n->ident, n->host);

		gettimeofday(&tv, NULL);
		if(count > MAXMATCHES && !nolimit)
			csntc(nick, "[%d] Trop de nicks trouvs, restreignez votre recherche ou forcez avec '-all' en DERNIER argument.", MAXMATCHES);

		csntc(nick, "Fin de la Recherche: %d matche%s trouve%s. (Effectue en %6.3f ms)",
			count, PLUR(count), PLUR(count), (float) (tv.tv_usec - ticks)/1000);
	}

	else if(!strcasecmp(arg, "suspend"))
	{
		time_t timeout = 0;
		char *ptr = NULL;

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(nick->user == u) return csntc(nick, "Vous ne pouvez pas vous suspendre.");

		if(IsAdmin(u) && u->level >= nick->user->level)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

		if(parc >= 3) /* do more parsing */
		{
			if(*parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
				return csntc(nick, GetReply(nick, L_INCORRECTDURATION));
			ptr = parv2msg(parc, parv, timeout ? 4 : 3, 250);
		}

		switch(handle_suspend(&u->suspend, nick->user->nick, ptr, timeout))
		{
			case -1:
				return csntc(nick, "Veuillez prciser une raison pour suspendre cet UserName.");
			case 1:
				if((n = u->n))
				{
					csntc(n, "Votre Username vient d'tre suspendu par l'Admin %s.", nick->nick);
					cs_account(n, NULL);
				}
		}
		/* report.. */
		if(IsSuspend(u)) show_ususpend(nick, u);
		else csntc(nick, "L'Username \2%s\2 n'est plus suspendu.", u->nick);
	}

	else if(!strcasecmp(arg, "cantregchan"))
	{
		time_t timeout = 0;

		if(!strcasecmp(nick->user->nick, user))
			return csntc(nick, "Vous ne voulez plus pouvoir enregistrer de chan ?");

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAdmin(u))
			return csntc(nick, "Vous ne pouvez pas empecher un Admin d'enregistrer des salons.");

		if(parc >= 3 && *parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
			return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

		if(!CantRegChan(u) || timeout)
		{
			if((u->cantregchan = timeout ? CurrentTS + timeout : 0))
				csntc(nick, "%s ne peut pas enregistrer de salon jusqu'au %s.",
					u->nick, get_time(nick,u->cantregchan));
			else csntc(nick, "%s ne peut pas enregistrer de salon.", u->nick);
		}
		else
		{
			u->cantregchan = -1;
			csntc(nick, "L'Username \2%s\2 peut maintenant enregistrer des salons.", u->nick);
		}
	}

	else if(!strcasecmp(arg, "mail"))
	{
		if(parc < 3 || !IsValidMail(parv[3]))
			return csntc(nick, GetReply(nick, L_MAIL_INVALID));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		Strncpy(u->mail, parv[3], MAILLEN);
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}

	else if(!strcasecmp(arg, "newpass"))
	{
		if(parc < 3 || strlen(parv[3]) < 6) return csntc(nick, GetReply(nick, L_PASS_LIMIT));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAdmin(u) && u->level >= nick->user->level && u != nick->user)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

		MD5pass(parv[3], u->passwd);
#ifdef MD5TRANSITION
		SetUMD5(u); /* mark it as md5 */
#endif
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}

	else if(!strcasecmp(arg, "newnick"))
	{
		if(parc < 3 ) return csntc(nick, "Syntaxe: %s NEWNICK <username> <new>", parv[0]);

		if(strlen(parv[3]) > NICKLEN || !IsValidNick(parv[3]))
			return csntc(nick, GetReply(nick, L_USER_INVALID), parv[3]);

		if(getuserinfo(parv[3])) return csntc(nick, GetReply(nick, L_ALREADYREG), parv[3]);
		if(!(u = getuserinfo(user))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), user);

		if(IsAdmin(u) && u->level >= nick->user->level && u != nick->user)
			return csntc(nick, "Vous ne pouvez pas changer l'username d'un Admin de niveau suprieur ou gal.");

		switch_user(u, parv[3]);
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}

	else if(!strcasecmp(arg, "del"))
	{
		if(!strcasecmp(nick->user->nick, user))
			return csntc(nick, "Vous ne devriez pas vous deluser.. (Utilisez la commande \2%s\2)", RealCmd("drop"));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAdmin(u) && u->level >= nick->user->level)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

		if((n = u->n))
		{
			cs_account(n, NULL);
			csntc(n, "Votre Username vient d'tre supprim par l'Administrateur %s (%s)",
				nick->nick, (parc > 2) ? parv2msg(parc, parv, 3, 200) : "Aucune raison");
		}

		csntc(nick, "L'username %s a bien t supprim.", u->nick);

		if(IsAnOwner(u))
		{
			char tmp[200];
			snprintf(tmp, sizeof tmp, "Suppression de l'username de l'owner (%s) par %s",
				u->nick, nick->user->nick);
			del_regnick(u, HF_SQLLOG, tmp);
		}
		else del_regnick(u, HF_SQLLOG, NULL);
	}
#ifdef SQLLOG
	else if(!strcasecmp(arg, "historique"))
	{
		time_t date = 0;
		MYSQL_RES *result;
		MYSQL_ROW row;
		int cmd, c;
		char tmp[250] = "SELECT * FROM "SQLUSER" WHERE user='%s' AND TS>=%T";

		if(!(u = ParseNickOrUser(nick, user))) return 0;
		if(parc < 2) return csntc(nick, "Syntaxe: %s HISTORIQUE <username> [%%XjXh] [-cmd <cmd> -chan]", parv[0]);
		if(parc >= 3 && *parv[3] == '%' && (date = convert_duration(++parv[3])) <= 0)
			return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

		if((c = getoption("-chan", parv, parc, 3, -1))) strcpy(tmp, "SELECT * FROM "SQLCHAN" WHERE user='%s' AND TS>=%T");
		if((cmd = getoption("-cmd", parv, parc, 3, 0))) strcat(tmp, " AND cmd='%s'");
		strcat(tmp, " order by TS desc LIMIT 0,100");

		if(sql_query(SQL_QRAW, tmp, u->nick, date ? CurrentTS - date : 0, cmd ? parv[cmd] : "") < 0)
			return csntc(nick, "La requte SQL a chou.");

		if(!(result = mysql_store_result(bot.sql_id)) || !(row = mysql_fetch_row(result)))
			return csntc(nick, "Aucune entre trouve.");

		if(date) csntc(nick, "Commandes effectues par %s depuis %s.", u->nick, duration(date));
		else csntc(nick, "Commandes effectues par %s.", u->nick);
		for(;row;row = mysql_fetch_row(result))
		{
			if(c) csntc(nick, "%s: \2%s\2 [%s] sur \2%s", get_time(nick, strtol(row[2], NULL, 10)), row[3], NONE(row[4]), row[1]);
			else csntc(nick, "%s: commande \2%s\2 [%s]", get_time(nick, strtol(row[1], NULL, 10)), row[2], NONE(row[3]));
		}
		mysql_free_result(result);
	}
#endif
	else if(!strcasecmp(arg, "stats"))
	{
		int memn = getoption("-mem", parv, parc, 2, -1);
		int cmds = getoption("-cmds", parv, parc, 2, 0);
		int traffic = getoption("-traffic", parv, parc, 2, -1);
		int all = getoption("-all", parv, parc, 2, -1);

		if(memn || all)
		{ /*memm: mmoire totale, usr: nb d'usernames, chs: nb de chans reg, usra: nb de nicks logus*/
			int memm = 0, usr = 0, chs = 0, usra = 0, ni = 0, maxbans, maxaccess, ma, mb, i = 0;
			int maxb = 0, tmpb = 0, useb = 0, bb = 0;/* Hash stats */
			aBan *ban;
			aJoin *j;
			struct ignore *ig = ignorehead;
			aLink *lp;

			if(!(maxbans = getoption("-maxbans", parv, parc, 2, 1))) maxbans = WARNACCESS;
			if(!(maxaccess = getoption("-maxaccess", parv, parc, 2, 1))) maxaccess = WARNACCESS;

			uptime(nick, chan, parc, parv);

			for(bb = 0, maxb = 0, useb = 0;i < NICKHASHSIZE;i++)
			{
				for(tmpb = 0, n = nick_tab[i];n;n = n->next, ni++)
				{
					tmpb++;/* item dans la petite linkedlist*/
					for(ma = 0, j = n->joinhead;j;j = j->next, ma++);
					memm += ma * sizeof(aJoin);
				}
				if(tmpb > maxb) maxb = tmpb;/* linkedlist locale la + longue?*/
				useb += tmpb;
				if(tmpb) bb++;/* une linked liste utilise en +*/
			}
			csntc(nick, "Hash Nick: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %.3f)", bb, NICKHASHSIZE, maxb, (float) useb / bb);

			for(i = 0, bb = 0, maxb = 0, useb = 0;i < USERHASHSIZE;i++)
			{
				for(tmpb = 0, u = user_tab[i];u;u = u->next, usr++, tmpb++)
				{
					if(u->suspend) memm += sizeof(struct suspendinfo);
					if(u->lastlogin) memm += strlen(u->lastlogin);
					if(u->n) usra++;
				}
				if(tmpb > maxb) maxb = tmpb;
				useb += tmpb;
				if(tmpb) bb++;
			}
			csntc(nick, "Hash User: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %.3f)", bb, USERHASHSIZE, maxb, (float) useb / bb);

			for(i = 0, bb = 0, maxb = 0, useb = 0;i< CHANHASHSIZE;i++)
			{
				for(tmpb = 0, chan = chan_tab[i];chan;chan = chan->next, chs++, tmpb++)
				{
					if(chan->suspend) memm += sizeof(struct suspendinfo);
					if(chan->motd) memm += strlen(chan->motd);

					for(mb = 0, ban = chan->banhead;ban;ban = ban->next, mb++) if(ban->raison) memm += strlen(ban->raison);
					if(mb >= maxbans) csntc(nick, "Warning Bans Limite: \2\00304%d\2\3 Bans sur %s", mb, chan->chan);

					for(ma = 0, lp = chan->access;lp;lp = lp->next, ma++) if(lp->value.a->info) memm += strlen(lp->value.a->info);
					if(ma >= maxaccess) csntc(nick, "Warning Acces Limite: \2\00304%d\2\3 Accs sur %s", ma, chan->chan);
					memm += ma * (sizeof(anAccess) + sizeof(aLink)) + mb * sizeof(aBan) + chan->users * sizeof(aLink);
				}
				if(tmpb > maxb) maxb = tmpb;
				useb += tmpb;
				if(tmpb) bb++;
			}
			csntc(nick, "Hash Chan: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %.3f)", bb, CHANHASHSIZE, maxb, (float) useb / bb);

			for(ma = 0;ig;ig = ig->next, ma++); memm += ma * sizeof *ig;
			memm += ni * sizeof(aNick) + usr * sizeof(anUser) + chs * sizeof(aChan) +
				(USERHASHSIZE + CHANHASHSIZE + NICKHASHSIZE + MAXNUM + CMDHASHSIZE) * sizeof(void *);

			csntc(nick, "Memoire Utilise pour les Users/Chans: %.3f KB", memm / 1024.0);
			csntc(nick, "Il y a %d Usernames et %d Chans enregistrs.", usr, chs);
			csntc(nick, "%d Chatteurs connects, dont %.1f %% logus.", ni, (usra / (float) ni) * 100);
		}
		if(cmds || all)
		{
			int x = 0, adm = 0, usr = 0;
			aHashCmd *cmd;

			for(;x < CMDHASHSIZE;x++) for(cmd = cmd_hash[x];cmd;cmd = cmd->next)
			{
				if(AdmCmd(cmd)) adm += cmd->used;
				else usr += cmd->used;
				if(cmds && *cmd->name != '\1' && !match(parv[cmds], cmd->name))
					csntc(nick, "- %s (\002%d\2)", cmd->name, cmd->used);
			}
			csntc(nick, "Un total de %d commandes utilises, dont %d Admins (Ratio: %.2f%)",
				(adm + usr), adm, (adm / (float) (usr + adm)) * 100);
		}
		if(traffic || all)
		{
			double up, down;
			int mbu = 0, mbd = 0;

			up = bot.dataS/1024.0;
			down = bot.dataQ/1024.0;
			if(up >= 1024)
			{
				up /= 1024.0;
				mbu = 1;
			}
			if(down >= 1024)
			{
				down /= 1024.0;
				mbd = 1;
			}
			csntc(nick, "Traffic: Up:\2 %.3f %s\2 Down:\2 %.3f %s\2", up, mbu ? "MB":"KB", down, mbd ? "MB":"KB");
			csntc(nick, "Traffic Actuel: %.3f Ko/s", (bot.dataQ - bot.lastbytes) / ((CurrentTS - bot.lasttime) * 1024.0));
#	ifdef WEB2CS
			csntc(nick, "Web2CS: Traffic Up:\2 %.3f Ko\2 Down:\2 %.3f Ko\2 in %d connections",
				bot.WEBtrafficUP/1024.0, bot.WEBtrafficDL/1024.0, bot.CONtotal);
#	endif
		}
		if(!all && !memn && !traffic && !cmds)
			csntc(nick, GetReply(nick, L_UNKNOWNOPTION), parc < 2 ? "<NULL>" : parv[2]);
	}
	else if(!strcasecmp(arg, "nopurge"))
	{
		if(!(u = ParseNickOrUser(nick, user))) return 0;

		switch_option(nick, parc < 3 ? NULL : parv[3], arg, u->nick, &u->flag, U_NOPURGE);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), arg);
	return 1;
}

void show_ususpend(aNick *nick, anUser *user)
{
	if(!user->suspend) return; /* on aurait jamais du l'envoyer l*/
	if(IsSuspend(user))
	{
		char buf[200] = {0};
		Strncpy(buf, get_time(nick, user->suspend->debut), sizeof buf - 1);
		csntc(nick, "L'UserName %s \2EST\2 suspendu par %s (le %s). Expire %s", user->nick, user->suspend->from, buf,
			user->suspend->expire ? get_time(nick, user->suspend->expire) : "Jamais");
	}
	else csntc(nick, "L'UserName %s \2ETAIT\2 suspendu par %s", user->nick, user->suspend->from);
	csntc(nick, "Raison: %s", user->suspend->raison);
}

int show_userinfo(aNick *nick, anUser *user, int flag)
{
	anAccess *a;
	char chans[451] = {0}, chansw[451] = {0}, b[REGCHANLEN + 20];
	int size = 0, counta = 0, sizew = 0, countw = 0;

	csntc(nick, GetReply(nick, L_INFO_ABOUT), user->nick);
	csntc(nick, GetReply(nick, L_MAILIS), user->mail);

	if(UFirst(user)) csntc(nick, " En cours de validation, enregistrement il y a %s", duration(CurrentTS - user->lastseen));
	else csntc(nick, GetReply(nick, L_LASTLOGINTIME), duration(CurrentTS - user->lastseen));
	if(IsAdmin(user)) csntc(nick, GetReply(nick, L_USERISADMIN), user->level);

	for(a = user->accesshead;a;a = a->next)
	{
		int p = snprintf(b, sizeof b, "%s (%d) ", a->c->chan, a->level);
		if(AWait(a))
		{
			int tmps = sizew;
			sizew += p;
			if(sizew >= sizeof chansw -1) /* le dernier fait depasser le buffer */
			{
				csntc(nick, countw++ ? "%s ..." : GetReply(nick, L_PROPACCESS) , chansw);
				strcpy(chansw, b); /* ajoute le suivant pour la prochaine*/
				sizew = p;
			}
			else strcpy(&chansw[tmps], b);
		}
		else
		{
			int tmps = size;
			size += p;
			if(size >= sizeof chans -1) /* le dernier fait depasser le buffer */
			{
				csntc(nick, counta++ ? "%s ..." : GetReply(nick, L_ACCESSON), chans);
				strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
				size = p;
			}
			else strcpy(&chans[tmps], b);
		}
	}

	if(*chans) csntc(nick, !counta ? GetReply(nick, L_ACCESSON) : "%s", chans);
	else if(!counta) csntc(nick, GetReply(nick, L_NOACCESS));
	if(*chansw) csntc(nick, !countw ? GetReply(nick, L_PROPACCESS) : "%s", chansw);
	else if(!countw) csntc(nick, GetReply(nick, L_NOPROPACCESS));

	if(flag && user->n) csntc(nick, " %s%s est actuellement logu sous cet Username (%s@%s)",
		GetPrefix(user->n), user->n->nick, user->n->ident, user->n->host);
	else if(flag) csntc(nick, " Personne n'est actuellement logu sous cet username");

	if(user->suspend) show_ususpend(nick, user);
	csntc(nick, GetReply(nick, L_OPTIONS), GetUserOptions(user));
	csntc(nick, " Last Login depuis %s", user->lastlogin ? user->lastlogin : "<Non Disponible>");

#ifdef HAVE_VOTE
	if(!flag && Vote.actif && user->level >= Vote.level)
	{
		if(user->vote) csntc(nick, " Vous avez vot pour: \2%s", vote[user->vote -1].prop);
		else csntc(nick, " Vous n'avez pas encore vot");
	}
#endif
	return 0;
}
