/* src/aide.c - Aide
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: aide.c,v 1.32 2005/04/02 10:21:34 romexzf Exp $
 */

#include "main.h"
#include "cs_cmds.h"
#include "outils.h"
#include "debug.h"
#include "hash.h"
#include "showcommands.h"

int syntax_cmd(aNick *nick, aHashCmd *cmd)
{
	static char helpcmd[CMDLEN + 1] = {0};

	if(!*cmd->syntax) return csntc(nick, "\2Erreur de syntaxe !");
	if(helpcmd[0] == 0) strcpy(helpcmd, RealCmd("aide"));

	csntc(nick, cmd->syntax, cmd->name);
	csntc(nick, GetReply(nick, L_MOREHELP), cs.nick, helpcmd, cmd->name);
	return 1;
}

int aide(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char more[16], *list;
	int morelen = 0, cur = 0, i = 0;
	aHashCmd *cmdp;
	HelpBuf *help;

	if(!parc) return showcommands(nick, chaninfo, parc, parv);

	if(!(cmdp = FindCommand(parv[1]))) return csntc(nick, GetReply(nick, L_NOSUCHCMD), parv[1]);

	help = cmdp->help[nick->user ? nick->user->lang->id : DefaultLang->id];
	if(!help) return csntc(nick, "\2Aide non trouve, consultez un Administrateur");

	if(parc > 1) {
		Strncpy(more, parv[2], sizeof more - 1);
		morelen = strlen(more);
		csntc(nick, GetReply(nick, L_HELP1), parv[1], more,
			AdmCmd(cmdp) ? "4ADMIN" : ChanCmd(cmdp) ? "12CHAN" : "3USER", cmdp->level);
	}
	else {
		csntc(nick, GetReply(nick, L_HELP2), parv[1],
			AdmCmd(cmdp) ? "4ADMIN" : ChanCmd(cmdp) ? "12CHAN" : "3USER", cmdp->level);
		if(*cmdp->syntax) csntc(nick, cmdp->syntax, cmdp->name);
	}

	list = help->buf[help->count-1];

	for(;i < help->count;++i)
	{
		char *p = help->buf[i];
		if(*p == '|')
		{
			if(morelen && !cur)
			{
				if(strncasecmp(++p, more, morelen)) /* still not found, go on */
					continue;
				else cur = 1; /* found.. */
			}
			else break; /* found a pipe and was not looking for option or already found */
		}
		else if(morelen && !cur) continue;
		csntc(nick, "%s", p);
	}

	/* looking for option, but not found! */
	if(morelen && !cur)	csntc(nick, "Aucune aide sur \2%s\2 pour la commande %s", more, parv[1]);
	/* there are options but none found/searched */
	if(*list == '|' && !cur) csntc(nick, GetReply(nick, L_OPTIONS), list + 1);
	/* option found or none looked for (search success!) */
	if(cur || !morelen) csntc(nick, GetReply(nick, L_SHOWCMDHELP2), bot.chan);
	if(cmdp->flag & CMD_DISABLE) csntc(nick, "\2\0034Note:\2\3 %s", GetReply(nick, L_CMDDISABLE));

	return 1;
}

static HelpBuf *help_newbuf(void)
{
	HelpBuf *ptr = calloc(1, sizeof *ptr);
	if(!ptr) Debug(W_MAX, "help::load, newbuf failed!");

	return ptr;
}

static int help_addbuf(HelpBuf *help, const char *buf)
{
	if(!(help->buf = realloc(help->buf, sizeof *help->buf * ++help->count)))
		return Debug(W_MAX, "help::load, addbuf OOM! (%s)", buf);
	help->buf[help->count -1] = NULL;
	str_dup(&help->buf[help->count -1], buf);
	return 0;
}

static int help_read_file(Lang *lang, FILE *fp)
{
	char buf[300], list[200];
	aHashCmd *cmdp = NULL;
	int cmds = 0, drop = 1;

	while(fgets(buf, sizeof buf, fp))
	{
		strip_newline(buf);

		if(*buf == '#') /* got a new command */
		{	/* first, end with previsous command options list, if any */
			if(cmdp && *list) help_addbuf(cmdp->help[lang->id], list);

			if(!(cmdp = FindCoreCommand(buf + 1)))
			{
				Debug(0, "help::loading lang %s, unknown command %s", lang->langue, buf + 1);
				drop = 1;
				continue;
			} /* help** has already been realloc'ed. or must be. */
			else drop = 0;

			if(cmdp->help[lang->id])
			{
				int i = 0;
				for(;i < cmdp->help[lang->id]->count;++i) free(cmdp->help[lang->id]->buf[i]);
				cmdp->help[lang->id]->count = 0;
			}
			else cmdp->help[lang->id] = help_newbuf();
			*list = 0; /* reinit list */
			++cmds;
		}
		else if(drop) continue;
		else if(*buf == '!')
		{
			if(lang == DefaultLang) Strncpy(cmdp->syntax, buf + 1, SYNTAXLEN);
		}
		else if(*buf == '|')
		{
			char *p = strchr(buf + 1, ' ');

			if(p) help_addbuf(cmdp->help[lang->id], buf), *p = 0;

			strcat(list, *list ? " " : "|");
			strcat(list, buf + 1);
		}
		else if(*buf) help_addbuf(cmdp->help[lang->id], buf);
	}
	return cmds;
}

int help_load(Lang *lang)
{
	FILE *fp;
	char buf[100];
	int i;

	if(!lang)
	{
		for(lang = DefaultLang;lang;lang = lang->next) help_load(lang);
		return 0;
	}
	snprintf(buf, sizeof buf, "aide/%s.help", lang->langue);

	if(!(fp = fopen(buf, "r")))
	{
		Debug(W_MAX, "help: unable to open helpfile for lang %s", lang->langue);
		return -1; /* error */
	}

	i = help_read_file(lang, fp);
	fclose(fp);

	if(i != CmdsCount) Debug(W_MAX, "help: Missing %d help for lang %s", CmdsCount - i, lang->langue);
	return i;
}

int help_cmds_realloc(int oldcount)
{
	register aHashCmd *tmp;
	int i = 0, j;
	void *p;

	for(;i < CMDHASHSIZE;++i)
		for(tmp = cmd_hash[i];tmp;tmp = tmp->next)
			if(!(p = realloc(tmp->help, sizeof *tmp->help * (LangCount+1))))
				Debug(W_MAX, "help::realloc, OOM!");
			else for(j = oldcount, tmp->help = p;j <= LangCount;++j) tmp->help[j] = NULL;
	return 0;
}
