/* src/ban.c - Commandes en rapport avec le ban
 * Copyright (C) 2002-2005
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: ban.c,v 1.75 2005/04/22 22:29:42 romexzf Exp $
 */

#include "main.h"
#include "ban.h"
#include "outils.h"
#include "hash.h"
#include "add_info.h"
#include "del_info.h"
#include "fichiers.h"
#include "cs_cmds.h"
#include "config.h"
#include <ctype.h>

/* sous traitement des commandes ban,
 * extrait/formate les arguments du ban (+erreurs) - Cesar */
static int do_ban(aNick *nick, aChan *chan, int parc, char **parv,
	char **mask, char *raison, time_t *timeout, int *level, char **duree)
{
	int i = 2, l;

	if(parc >= 3 && is_num(parv[2])) *level = atoi(parv[i++]);
	*mask = parv[i++];
	if(parc >= i) /* raison ou dure */
	{
		if(*parv[i] == '%') /* dure */ *duree = ++parv[i++];
		if(parc >= i) parv2msgn(parc, parv, i, raison, 300);
	}

	l = ChanLevelbyUserI(nick->user, chan);
	if(*level < 1 || (l < *level && !IsAdmin(nick->user)))
		return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

	if(*duree && (*timeout = convert_duration(*duree)) <= 0 && (**duree != '0' || !chan->bantime))
		return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

	if(CNoBans(chan))
		return csntc(nick, GetReply(nick, L_NOBANSON), chan->chan);

	if(chan->banlevel > l && !IsAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_BANLVLON), chan->banlevel, chan->chan);

	return 1;
}

/* check if 'mask' is overlapped by current banlist
 * or/and try to remove overlapping bans.
 * Double match :/ -Cesar */
static int check_redundant_bans(aChan *chan, const char *mask, aNick *nick)
{
	register aBan *b, *b2;
	int n = IsAdmin(nick->user) ? OWNERLEVEL : ChanLevelbyUserI(nick->user, chan);

	for(b = chan->banhead;b;b = b2)
	{
		b2 = b->next;

		if(!strcasecmp(b->mask, mask) || !mmatch(b->mask, mask))
			return csntc(nick, GetReply(nick, L_ALREADYMATCHED), mask, b->mask, b->level);
		if(n >= b->level && !mmatch(mask, b->mask)) /* b is overlapped */
			del_ban(chan, b); /* clear it if enough level*/
	}
	return 1;
}

/* Repris en majorit d'ircu (Carlo Wood (Run)) */
char *pretty_mask(char *mask)
{
	static char star[2] = "*", retmask[NUHLEN];
	char *last_dot = NULL, *ptr = mask;
	/* Case 1: default */
	char *nick = mask, *user = star, *host = star;

	/* Do a _single_ pass through the characters of the mask: */
	for(; *ptr; ++ptr)
	{
		if(*ptr == '!')
		{	/* Case 3 or 5: Found first '!' (without finding a '@' yet) */
			user = ++ptr;
			host = star;
		}
		else if(*ptr == '@')
		{	/* Case 4: Found last '@' (without finding a '!' yet) */
			nick = star;
			user = mask;
			host = ++ptr;
		}
		else if(*ptr == '.')
		{	/* Case 2: Found last '.' (without finding a '!' or '@' yet) */
			last_dot = ptr;
			continue;
		}
		else continue;

		for(; *ptr; ++ptr) if(*ptr == '@') host = ptr + 1;
		break;
	}
	if(user == star && last_dot)
	{	/* Case 2: */
		nick = star;
		host = mask;
	}
	/* Check lengths */
	if(nick != star)
	{
		char *nick_end = (user != star) ? user - 1 : ptr;
		if(nick_end - nick > NICKLEN) nick[NICKLEN] = 0;
		*nick_end = 0;
	}
	if(user != star)
	{
		char *user_end = (host != star) ? host - 1 : ptr;
		if(user_end - user > USERLEN)
		{
			user = user_end - USERLEN;
			*user = '*';
		}
		*user_end = 0;
	}
	if(host != star && ptr - host > HOSTLEN)
	{
		host = ptr - HOSTLEN;
		*host = '*';
	}
	if(!*host) host = star; /* prevent '*!@*' or 'a!b@' */
	if(!*user) user = star;
	if(!*nick) nick = star; /* ... and '!b@c' */
	for(ptr = retmask;*nick;++nick)
		if(*nick != '*' || ptr == retmask || nick[-1] != '*') *ptr++ = *nick;
	for(*ptr++ = '!';*user;++user) if(*user != '*' || ptr[-1] != '*') *ptr++ = *user;
	for(*ptr++ = '@';*host;++host) if(*host != '*' || ptr[-1] != '*') *ptr++ = *host;
	*ptr = 0;
	return retmask;
}

/*
 * ban_cmd - Progs 11/09/02 (c'est l'anniversaire des attentats qui m'y a fait penser)
 * Syntaxe: BAN <chan> [level] <mask|%nick> [%dure] [raison]
 */
int ban_cmd(aNick *nick, aChan *chan, int parc, char **parv)
{
	aLink *join;
	char raison[300] = {0}, *duree = NULL, *mask = NULL;
	int level = 1;
	time_t timeout = 0;

	if(!do_ban(nick, chan, parc, parv, &mask, raison, &timeout, &level, &duree))
		return 0;

	if(*mask == '%') /* old nickban */
	{
		aNick *cible = GetMemberIbyNick(chan, ++mask);
		if(!cible) return csntc(nick, GetReply(nick, L_NOTONCHAN), mask, parv[1]);
		if(!check_protect(nick, cible, chan)) return 0;
		mask = getbanmask(cible, chan->bantype);
	}
	else
	{
		mask = pretty_mask(mask);

		for(join = chan->members;join;join = join->next)
		{
			aNick *cible = join->value.j->nick;
			if(cible != nick && !match(mask, GetNUHbyNick(cible, 1))
				&& !check_protect(nick, cible, chan)) return 0;
		}
	}

	if(!check_redundant_bans(chan, mask, nick)) return 0;

	csmode(chan, MODE_OBV, "+b $", mask);
	add_ban(chan, mask,	*raison ? raison : DEFRAISON, nick->user->nick, CurrentTS,
			chan->bantime ? ((timeout || (duree && *duree == '0')) ? timeout : chan->bantime) : timeout,
			level);
	return 1;
}

int kickban(aNick *nick, aChan *chan, int parc, char **parv)
{
	aNick *cible;
	char raison[300] = {0}, *mask = NULL, *duree = NULL;
	int level = 1;
	time_t timeout = 0;

	if(!do_ban(nick, chan, parc, parv, &mask, raison, &timeout, &level, &duree))
		return 0;

	if(!strcasecmp(cs.nick, mask)) return csntc(nick, "Non!");

	if(!(cible = GetMemberIbyNick(chan, mask)))
		return csntc(nick, GetReply(nick, L_NOTONCHAN), parv[2], chan->chan);

	if(!check_protect(nick, cible, chan)) return 0;

	mask = getbanmask(cible, chan->bantype);

	csmode(chan, MODE_OBV, "+b $", mask);
	cskick(chan->chan, cible->numeric, "(\2$\2) $", nick->user->nick, *raison ? raison : DEFRAISON);

	if(!check_redundant_bans(chan, mask, nick)) return 0;/* kick permis mais ban non ajout*/

	add_ban(chan, mask,	*raison ? raison : DEFRAISON, nick->user->nick, CurrentTS,
			chan->bantime ? ((timeout || (duree && *duree == '0')) ? timeout : chan->bantime) : timeout,
			level);
	return 1;
}

/*
 * banlist #chan
 */
int banlist(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban;
	int i = 0, m = 0, id = 0;
	char buf[200];

	if(parc < 2) m = 1;

	for(ban = chaninfo->banhead;ban;ban = ban->next)
	{
		++id;
		if(m || !mmatch(parv[2], ban->mask))
		{
			i++;
			csntc(nick, GetReply(nick, L_BLMASK), ban->mask, ban->level, id);
			csntc(nick, GetReply(nick, L_BLFROM), ban->de, ban->raison);

			Strncpy(buf, get_time(nick, ban->debut), sizeof buf - 1);
			csntc(nick, GetReply(nick, L_BLEXPIRE), buf,
				ban->fin ? get_time(nick, ban->fin) : "Jamais");
			csntc(nick, "-");
		}
	}

	if(i) csntc(nick, GetReply(nick, L_TOTALBAN), i, PLUR(i), chaninfo->chan);
	else csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chaninfo->chan);
	return 1;
}

/*
 * clear_bans #chan -from -minlevel -maxlevel
 */
int clear_bans(aNick *nick, aChan *chan, int parc, char **parv)
{
	aBan *ban = chan->banhead, *tmp = NULL;
	char modes[7] = {0}, masks[451] = {0};
	size_t size = 0, temp;
	int n = IsAdmin(nick->user) ? OWNERLEVEL : ChanLevelbyUserI(nick->user, chan);
	int minlevel = getoption("-minlevel", parv, parc, 1, 1);
	int maxlevel = getoption("-maxlevel", parv, parc, 1, 1);
	int from = getoption("-from", parv, parc, 1, 0), i = 0, nb = 0, s;

	for(;ban;ban = tmp)
	{
		tmp = ban->next;
		if((from && strcasecmp(parv[from], ban->de)) || (minlevel && ban->level < minlevel)
			|| (maxlevel && ban->level > maxlevel)) continue;

		if(n < ban->level)
		{
			csntc(nick, GetReply(nick, L_CANTREMOVEBAN), ban->mask, ban->level);
			continue;
		}/* format la chaine de modes pour l'unban 6 -b max dans 450 bytes max */

		temp = size;/* taille actuelle */
		size += (s = strlen(ban->mask)); /* future taille */
		if(size > 450 || nb > 5) /* buf plein ou 6 modes queued */
		{
			modes[nb] = 0;
			csmode(chan, MODE_OBV, "-$ $", modes, masks);
			size = s, temp = 0,	nb = 0;
		}
		strcpy(&masks[temp], ban->mask);
		modes[nb++] = 'b';
		masks[size++] = ' ';
		masks[size] = 0;
		del_ban(chan, ban);
		i++;
	}
	modes[nb] = 0;
	if(nb) csmode(chan, MODE_OBV, "-$ $", modes, masks);

	if(i) csntc(nick, GetReply(nick, L_TOTALBANDEL), i, PLUR(i), chan->chan);
	else csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chan->chan);
	return 1;
}

static int unban_nick(aNick *nick, aNick *bn, aChan *chan)
{
	aBan *ban, *bt = NULL;
	int i = 0, n = IsAdmin(nick->user) ? OWNERLEVEL : ChanLevelbyUserI(nick->user, chan);

	while((ban = is_ban(bn, chan, bt)))
	{
		if(n < ban->level)
			return csntc(nick, GetReply(nick, L_CANTREMOVEBAN), ban->mask, ban->level);
		i++;
		bt = ban->next;
		csmode(chan, MODE_OBV, "-b $", ban->mask);
		csntc(nick, GetReply(nick, L_BANDELETED), ban->mask, chan->chan);
		del_ban(chan, ban);
	}

	csntc(nick, GetReply(nick, L_TOTALBANDEL), i, PLUR(i), chan->chan);

	/* invite aprs l'unban (si complet) pour passer les +b locaux */
	if(!GetJoinIbyC(bn, chan))
		putserv("%s " TOKEN_INVITE " %s :%s", cs.num, bn->nick, chan->chan);

	return 1;
}

int unban(aNick *nick, aChan *chan, int parc, char **parv)
{
	aBan *ban, *bant = NULL;
	char *b = parv[2];
	int i = 1, n, items[10], count = 0, removed = 0;

	if(*b == '%')
	{
		aNick *nptr = getnickbynick(++b);
		if(!nptr) return csntc(nick, GetReply(nick, L_NOSUCHNICK), b);
		return unban_nick(nick, nptr, chan);
	}
	else if(isdigit(*b)) count = item_parselist(b, items, ASIZE(items));

	for(ban = chan->banhead, n = ChanLevelbyUserI(nick->user, chan);ban;ban = bant, ++i)
	{
		bant = ban->next;
		if((count && item_isinlist(items, count, i) != -1) || (!count && !mmatch(b, ban->mask)))
		{
			if(!IsAdmin(nick->user) && n < ban->level)
			{
				csntc(nick, GetReply(nick, L_CANTREMOVEBAN), ban->mask, ban->level);
				continue;
			}
			csntc(nick, GetReply(nick, L_BANDELETED), ban->mask, chan->chan);
			csmode(chan, MODE_OBV, "-b $", ban->mask);
			del_ban(chan, ban);
			if(count == ++removed) break; /* gonna write the banlist at the end */
		}
	}

	if(!removed) csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chan->chan);
	return 1;
}

int unbanme(aNick *nick, aChan *chan, int parc, char **parv)
{
	return unban_nick(nick, nick, chan);
}

char *getbanmask(aNick *nick, int bantype)
{
	static char mask[HOSTLEN + USERLEN + 5];
	char *host = NULL;

	if(nick->user && nick->flag & N_CRYPT && (host = malloc(NICKLEN + HOSTLEN + 3)))
		fastfmt(host, "$.$", nick->user->nick, hidden_host);
	else
#ifdef HAVE_CRYPTHOST
			host = nick->crypt;	/* choix de l'host sur laquelle */
#else							/* le ban va porter: relle, crypte, +x */
			host = nick->host;	/* selon les 'protections' actuelles de nick */
#endif

	if(bantype < 2) /* ban du type *!*ident@*.host */
	{
		if(is_ip(nick->host))
		{
#ifdef HAVE_CRYPTHOST
			unsigned int a = strtol(host, NULL, 10); /* it will stop at the first dot */
			sprintf(mask, "*!*%s@%u.*", nick->ident, a);
#else
			unsigned int a, b, c;
			sscanf(host, "%u.%u.%u.", &a, &b, &c);
			sprintf(mask, "*!*%s@%u.%u.%u.*", nick->ident, a, b, c);
#endif
		}
        else fastfmt(mask, "*!*$@*$", nick->ident, strchr(host, '.'));
    }
    else if(bantype == 2) fastfmt(mask, "*!*$@*", nick->ident); /* type : *!*ident@* */
	else if(bantype == 3) fastfmt(mask, "*$*!*@*", nick->nick); /* type : *nick*!*@* */
	else if(bantype == 4) fastfmt(mask, "*!*@$", host); /* type : *!*@host */
	else fastfmt(mask, "*!*$@$", nick->ident, host); /* type *!*ident@host */

	if(nick->user && nick->flag & N_CRYPT) free(host);
	return mask;
}

aBan *is_ban(aNick *nick, aChan *chan, aBan *from) /* retourne un pointeur vers le ban */
{													/* vrifie sur toutes les hosts (sauf IP) */
	aBan *b = from ? from : chan->banhead;
	char host[HOSTLEN] = "";

	for(;b;b = b->next)
	{
		if((b->flag & BAN_ANICKS || !match(b->nick, nick->nick))
			&& (b->flag & BAN_AUSERS || !match(b->user, nick->ident))
			&& (b->flag & BAN_AHOSTS || (
#ifdef HAVE_CRYPTHOST
			!match(b->host, nick->crypt) || /* le plus probable */
#endif
			(nick->user	&& (*host || fastfmt(host, "$.$", nick->user->nick, hidden_host))
			&& !match(b->host, host)) || !match(b->host, nick->host))))
				return b;
		}

	return NULL;
}

char *GetBanType(aChan *chan)
{
	static char *mask[6] = {
		"<invalide>", 	"*!*ident@*.host",
		"*!*ident@*", 	"*nick*!*@*",
		"*!*@host",		"*!*ident@host"
	};
	return mask[chan->bantype >= ASIZE(mask) ? 0 : chan->bantype];
}
