/* src/convertdb.c - conversion des DBs
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: convertdb.c,v 1.5 2005/05/01 23:06:22 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "fichiers.h"
#include <ctype.h>
#include <errno.h>
#include <unistd.h>

#define USERS_FILE DBDIR"/usr.db"
#define CHANS_FILE DBDIR"/chans.db"

aChan *chan_tab[CHANHASHSIZE] = {0}; 	/* hash chan */
anUser *user_tab[USERHASHSIZE] = {0}; 	/* hash username */
Lang *DefaultLang = NULL;

static int ver_u, ver_c; /* version des fichiers prsents */

/* String Tools */
static void strip_newline(char *string)
{
   register char *p = string;
   while(*p && *p != '\n' && *p != '\r') ++p;
   *p = '\0';
}

static void conv_char(const char *chan, char *string)
{
	int i = 0;

	for(;*chan;++chan)
	{
		if(!isalnum(*chan))
		{
			string[i++] = '%';
			i += sprintf(string + i, "%03o", *chan);
		}
		else string[i++] = tolower(*chan);
	}
	string[i] = '\0';
}

static char *str_dup(char **to, const char *from)
{
	if(from && *from)
	{
		*to = realloc(*to, strlen(from) + 1);
		if(!*to) printf("error: strdup: no memory for 'to'(%p)=%s 'from'(%p)=%s\n",
			(void *) *to, *to, (const void *) from, from);
		else strcpy(*to, from);
	}
	return *to;
}

static char *Strncpy(char *to, const char *from, size_t n)/* copie from dans to. ne copie que n char */
{							/* MAIS AJOUTE LE CHAR \0  la fin, from DOIT donc faire n+1 chars. */
	const char *end = to + n;
	char *save = to;
	while(to < end && (*to++ = *from++));
	*to = 0;
	return save;
}

int split_buf(char *buf, char **parv, int size)
{
	int parc = 0, i;

	while(*buf && parc < size)
	{
		while(*buf == ' ') *buf++ = 0; /* on supprime les espaces pour le dcoupage */
		if(*buf == ':') /* last param */
		{
			parv[parc++] = buf + 1;
			break;
		}
		if(!*buf) break;
		parv[parc++] = buf;
		while(*buf && *buf != ' ') ++buf; /* on laisse passer l'arg.. */
	}
	for(i = parc;i < size;++i) parv[i] = NULL; /* NULLify following items */
	return parc;
}

/* Internal DATA functions */

static const struct Modes {
	char mode;
	int value;
} CMode[] = {
	{ 'n', C_MMSG },
	{ 't', C_MTOPIC },
	{ 'i', C_MINV },
	{ 'l', C_MLIMIT },
	{ 'k', C_MKEY },
	{ 's', C_MSECRET },
	{ 'p', C_MPRIVATE },
	{ 'm', C_MMODERATE },
	{ 'c', C_MNOCTRL },
	{ 'C', C_MNOCTCP },
	{ 'O', C_MOPERONLY },
	{ 'r', C_MUSERONLY },
};

int cmodetoflag(int flag, const char *mode)
{
	unsigned int w = 1, i;

	if(!mode) return flag;
	for(;*mode;mode++)
	{
		if(*mode == '+') w = 1;
		else if(*mode == '-') w = 0;
		else for (i = 0;i < ASIZE(CMode);i++)
				if(*mode == CMode[i].mode)
				{
					if(w) flag |= CMode[i].value;
					else flag &= ~CMode[i].value;
					break;
				}
	}
	return flag;
}

int string2scmode(struct cmode *mode, const char *modes, const char *key, const char *limit)
{
    int newmode = cmodetoflag(mode->modes, modes);

	if(newmode & C_MLIMIT)
	{
		int lim = 0;
		if(limit && (lim = strtol(limit, NULL, 10))) mode->limit = lim;
		else if(!(mode->modes & C_MLIMIT)) newmode &= ~C_MLIMIT;/* +l mis mais aucune limite valide..*/
	}
	else mode->limit = 0;

	if(newmode & C_MKEY)
	{
		if(key && *key) Strncpy(mode->key, key, KEYLEN);
		else if (!*mode->key) newmode &= ~C_MKEY;/* +k mis pourtant aucun cl valide.. */
	} /* -k autoris seulement si key fournie */
	else if(mode->modes & C_MKEY && (!key || !*key)) newmode |= C_MKEY;
	else *mode->key = 0;
	return (mode->modes = newmode);
}

unsigned int do_hash(const char *str, int hashsize)
{
	unsigned int checksum = 0;
	while(*str) checksum += (checksum << 3) + tolower((unsigned char) *str++);
	return checksum & (hashsize-1);
}

aChan *getchaninfo(const char *chan)
{
	unsigned int hash = do_hash(chan, CHANHASHSIZE);
	register aChan *tmp = chan_tab[hash];
	for(;tmp && strcasecmp(chan, tmp->chan);tmp = tmp->next);
	return tmp;
}

aChan *add_chan(const char *chan, const char *description)
{
	aChan *c = calloc(1, sizeof *c);
	unsigned int hash = do_hash(chan, CHANHASHSIZE);

	if(!c)
	{
		printf("error: add_chan, malloc a chou pour aChan %s", chan);
		return NULL;
	}

	Strncpy(c->chan, chan, REGCHANLEN);
	Strncpy(c->description, description, DESCRIPTIONLEN);
	c->suspend = NULL;
	c->motd = NULL;
	c->banlevel = DEFAUT_BANLEVEL;
	c->cml = DEFAUT_CMODELEVEL;
	c->bantype = C_DEFAULT_BANTYPE;
	c->flag = C_DEFAULT;
	c->creation_time = time(0);
	c->defmodes.modes = C_DEFMODES;
	c->banhead = NULL;

	c->next = chan_tab[hash];/* swap le vieux*/
	return (chan_tab[hash] = c);		/* et le nouveau (mis en tte)*/
}

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, const char *de, time_t date, const char *message, int flag)
{
	aMemo *m = calloc(1, sizeof *m), *tmp = nick->memohead;

	if(!m)
	{
		printf("error: add_memo, malloc a chou pour aMemo %s (%s)\n", nick->nick, message);
		return;
	}

	m->next = NULL;
	Strncpy(m->de, de, NICKLEN);
	m->date = date;
	m->flag = flag;
	Strncpy(m->message, message, MEMOLEN);

	if(!tmp) nick->memohead = m;
	else
	{
		for(;tmp->next;tmp = tmp->next);
		tmp->next = m;
	}
}
#endif

anAccess *add_access(anUser *user, const char *chan, int level, int flag, time_t lastseen)
{
	anAccess *ptr = calloc(1, sizeof *ptr);

	if(!ptr)
	{
		printf("error: add_access, malloc a chou pour l'accs %s (%s)\n", user->nick, chan);
		return NULL;
	}

	ptr->level = level;
	ptr->flag = flag;
	ptr->user = user;
	ptr->info = NULL;
	ptr->lastseen = lastseen ? lastseen : user->lastseen;

	if(!(ptr->c = getchaninfo(chan)))
	{
		printf("convertdb: add_access, accs de %s sur %s, non reg ?\n", user->nick, chan);
		free(ptr);
		return NULL;
	}

	ptr->next = user->accesshead;
	return (user->accesshead = ptr);
}

void add_ban(aChan *chan, const char *mask, const char *raison, const char *de,
	time_t debut, time_t fin, int level)
{
	aBan *ban = calloc(1, sizeof *ban);

	if(!ban)
	{
		printf("error: add_ban, malloc a chou pour aBan %s sur %s de %s (%s)\n",
				mask, chan->chan, de, raison);
		return;
	}

	str_dup(&ban->mask, mask);
	str_dup(&ban->raison, raison);
	Strncpy(ban->de, de, NICKLEN);
	ban->debut = debut;
	ban->fin = fin ? debut + fin : 0;
	ban->level = level;

	if(chan->banhead) chan->banhead->last = ban;
	ban->next = chan->banhead;
	ban->last = NULL;
	chan->banhead = ban;
}

anUser *add_regnick(const char *user, const char *pass, time_t lastseen,
					int level, int flag, const char *mail, int vote)
{
	anUser *u = calloc(1, sizeof *u);
	unsigned int hash = do_hash(user, USERHASHSIZE);

	if(!u)
	{
		printf("error: add_regnick, malloc a chou pour anUser %s\n", user);
		return NULL;
	}

	Strncpy(u->nick, user, NICKLEN);
	strcpy(u->passwd, pass);
	u->lastseen = lastseen;
	u->level = level;
	Strncpy(u->mail, mail, MAILLEN);
#ifdef HAVE_VOTE
	u->vote = vote;
#endif
	u->flag = flag;
	u->suspend = NULL;
	u->lastlogin = NULL;
	u->cantregchan = -1;

	u->next = user_tab[hash];
	return (user_tab[hash] = u);
}

void do_suspend(struct suspendinfo **suspend, const char *raison, const char *nick,
	time_t expire, time_t debut)
{
	if(!*suspend) *suspend = calloc(1, sizeof **suspend);
	Strncpy((*suspend)->raison, raison, sizeof (*suspend)->raison -1);
	Strncpy((*suspend)->from, nick, NICKLEN);
	(*suspend)->debut = debut;
	(*suspend)->expire = expire > 0 ? expire + debut : expire;
}

/* We accept ALL language : that's Z' job to check it */
Lang *lang_isloaded(const char *name)
{
	Lang *lang = DefaultLang;
	for(;lang && strcasecmp(lang->langue, name);lang = lang->next);
	if(!lang)
	{
		Lang *temp = DefaultLang;

		if(!(lang = calloc(1, sizeof *lang)))
		{
			printf("error: lang_add, malloc a chou pour Lang %s\n", name);
			return NULL;
		}
		lang->next = NULL;
		Strncpy(lang->langue, name, sizeof lang->langue -1);

		if(!DefaultLang) DefaultLang = lang;
		else
		{
			for(;temp->next;temp = temp->next); /* ajout  la fin */
			if(temp) temp->next = lang;
		}
	}
	return lang;
}

/* FILE */

static int old_load_users(void)
{
	FILE *fp, *fp1;
	char buff[100];
	int stats = 0;
	anUser *u = NULL;
	anAccess *a;

	if(!(fp = fopen(USERS_FILE, "r"))) return 0;

	while(fgets(buff, sizeof buff, fp))
	{
		char buff1[500] = DBDIR "/usr/", *ar[20];
		++stats;
		strip_newline(buff);
		conv_char(buff, buff1 + sizeof(DBDIR "/usr/") - 1);
		if(!(fp1 = fopen(buff1, "r")))
		{
			printf("convertdb: Impossible d'ouvrir le fichier '%s' pour '%s': %s\n",
				buff1, buff, strerror(errno));
			continue;
		}

		while(fgets(buff1, sizeof buff1, fp1))
		{
			int items = split_buf(buff1, ar, ASIZE(ar));
			if(items < 2) continue;
			strip_newline(ar[items-1]);

			if(!strcmp(buff1, "NICK"))
			{ /* NICK <user> <pass> <lvl> <TS> <flag> <vote> <mail> <lastlogin> <cantregchan> <lang> */
				if(items < 10)
				{
					printf("converdb: Erreur lors de la lecture"
							"de la DB pour %s, item incomplet\n", ar[1]);
					break;
				}

				u = add_regnick(ar[1], ar[2], strtol(ar[5], 0, 10),	strtol(ar[3], 0, 10),
						strtol(ar[6], 0, 10), ar[8], strtol(ar[7], 0, 10));

				if(*ar[9] != '0') str_dup(&u->lastlogin, ar[9]);
				/* if no lang was saved, or not available anymore, set to default */
				if(!ar[11] || !(u->lang = lang_isloaded(ar[11]))) u->lang = DefaultLang;
				u->cantregchan = strtol(ar[10], NULL, 10);
				if(u->cantregchan > 0 && u->cantregchan < time(0)) u->cantregchan = -1;
			}

			else if(!strcmp(buff1, "SUSPEND") && items > 4)
			{ /* SUSPEND UserName deb fin :raison */
				long int debut = strtol(ar[2], NULL, 10), fin = strtol(ar[3], NULL, 10);
				do_suspend(&u->suspend, ar[4], ar[1], fin > 0 ? fin - debut : fin, debut);
			}
#ifdef USE_MEMOSERV
			else if(!strcmp(buff1, "MEMO") && items > 4)
				/* MEMO <from> <DateTS> <flag> :message */
				add_memo(u, ar[1], strtol(ar[2], NULL, 10), ar[4], strtol(ar[3], NULL, 10));
#endif
			else if(!strcmp(buff1, "ACCESS") && items > 5)
			{ /* ACCESS <#> <level> <flag> <lastseenTS> :info */
				int level = strtol(ar[2], NULL, 10), flag = strtol(ar[3], NULL, 10);

				a = add_access(u, ar[1], level, flag, strtol(ar[4], NULL, 10));
				if(a && *ar[5]) str_dup(&a->info, ar[5]);
			}
		}
		fclose(fp1);
	}
	fclose(fp);
	return stats;
}

static int old_load_chans(void)
{
	FILE *fp, *fp1;
	char buff[250], *ar[6];
	int stats = 0;

	if(!(fp = fopen(CHANS_FILE, "r"))) return 0;

	while(fgets(buff, sizeof buff, fp))
	{
		char *chan = buff, nchan[2 * REGCHANLEN] = {0}, buff1[500] = DBDIR "/chans/";
		aChan *cc;

		++stats;
		strip_newline(buff);
		conv_char(chan, nchan);
		strcpy(buff1 + sizeof(DBDIR "/chans/") - 1, nchan);
		if(!(fp1 = fopen(buff1, "r")))
		{
			printf("convertdb: Impossible d'ouvrir le fichier '%s' pour '%s': %s\n",
					buff1, buff, strerror(errno));
			continue;
		}

		if(!(cc = add_chan(chan, "")))
		{
			printf("convertdb: Erreur lors du chargement de %s\n", chan);
			break;
		}

		while(fgets(buff1, sizeof buff1, fp1))
		{
			char *field = strchr(buff1, ' ');
			strip_newline(buff1);
			if(field) *field++ = 0;
			if(!field || !*field) continue;

			if(!strcmp(buff1, "DEFMODES"))
			{
				char *lim = strchr(field, ' '), *key = NULL;
				if(lim)
				{
					*lim++ = 0;
					if((key = strchr(lim, ' '))) *key++ = 0;
				}
				cc->defmodes.modes = 0; /* otherwise, global defaults would be appended */
				string2scmode(&cc->defmodes, field, !key && strchr(field, 'k') ? lim : key, lim);
			}
			else if(!strcmp(buff1, "DEFTOPIC")) Strncpy(cc->deftopic, field, TOPICLEN);
			else if(!strcmp(buff1, "DESCRIPTION")) Strncpy(cc->description, field, DESCRIPTIONLEN);
			else if(!strcmp(buff1, "WELCOME")) Strncpy(cc->welcome, field, TOPICLEN);
			else if(!strcmp(buff1, "OPTIONS"))
			{
				int flag = 0, bantype = 0;
				unsigned long bandefault = 0;

				if(sscanf(field, "%d %d %d %d %lu %d %d", &flag, &cc->banlevel, &bantype,
					&cc->cml, &bandefault, &cc->limit_min, &cc->limit_inc) != 7)
					printf("convertdb: Options incompltes pour %s [%s]\n", chan, field);

				cc->flag = (flag & ~C_JOINED);
				cc->bantype = (bantype > 4 || bantype < 1) ? 1 : bantype;
				cc->bantime = (time_t) bandefault;
			}
			else if(!strcmp(buff1, "MOTD")) str_dup(&cc->motd, field);
			else if(!strcmp(buff1, "URL")) /* url + channel ts */
			{
				char *creation_t = strchr(field, ' ');
				if(creation_t) *creation_t++ = 0;
				Strncpy(cc->url, field, sizeof cc->url -1);
				cc->creation_time = (creation_t && *creation_t) ? strtoul(creation_t, NULL, 10) : 0;
			}
			else if(!strcmp(buff1, "SUSPEND") && split_buf(field, ar, ASIZE(ar)) > 4)
			{ /* UserName TimeStamp :raison */
				long int debut = strtol(ar[2], NULL, 10), fin = strtol(ar[3], NULL, 10);
				do_suspend(&cc->suspend, ar[4], ar[1], fin > 0 ? fin - debut : fin, debut);
			}
		}
		fclose(fp1);
		/* load bans */
		snprintf(buff1, sizeof buff1, DBDIR "/bans/%s", nchan);
		if(!(fp1 = fopen(buff1, "r"))) continue;
		while(fgets(buff1, sizeof buff1, fp1))
		{
			time_t debut, fin;
			int i = split_buf(buff1, ar, ASIZE(ar));
			if(i < 6) continue;
			strip_newline(ar[i-1]);
			debut = strtol(ar[2], NULL, 10);
			fin = strtol(ar[3], NULL, 10);
			if(!fin || time(0) < fin)
				add_ban(cc, ar[0], ar[5], ar[1], debut, fin ? fin - debut : 0, atoi(ar[4]));
		}
		fclose(fp1);
	}
	fclose(fp);
	return stats;
}

static int check_versions(int print)
{
	FILE *fp;
	char buf[500] = {0};

	if((fp = fopen(DBDIR "/" DBUSERS, "r"))) /* look for userfile's version */
	{
		fgets(buf, sizeof buf, fp);
		ver_u = !strncmp("V ", buf, 2) ? strtol(buf + 2, 0, 10) : 0;
		fclose(fp);
	}
	else if(!(fp = fopen(USERS_FILE, "r"))) return -1; /* no db at all => first run */
	else {
		if(fp) fclose(fp);
		ver_u = 0; /* before migration */
	}

	if(print && ver_u < DBVERSION_U)
	{
		puts("La DB User n'est pas  jour, lancez 'bin/convertdb -c' pour la convertir.");
		return 0;
	}

	if((fp = fopen(DBDIR "/" DBCHANS, "r")))
	{
		fgets(buf, sizeof buf, fp);
		ver_c = !strncmp("V ", buf, 2) ? strtol(buf + 2, 0, 10) : 0;
		fclose(fp);
	}
	else ver_c = 0;

	if(print && ver_c < DBVERSION_C)
	{
		puts("La DB Salon n'est pas  jour, lancez 'bin/convertdb -c' pour la convertir.");
		return 0;
	}

	if(print)
	{
		puts("Les DBs sont  jour.");
		exit(0);
	}

	return 0;
}

static void load_db(void)
{
	switch(ver_c)
	{
		case 0:
			old_load_chans();
			break;
		default:
			printf("convertdb: No handler for channel DB version %d\n", ver_c);
			exit(EXIT_FAILURE);
	}
	switch(ver_u)
	{
		case 0:
			old_load_users();
			break;
		default:
			printf("converdb: No handler for user DB version %d\n", ver_u);
			exit(EXIT_FAILURE);
	}
}

char *GetCModes(struct cmode modes)
{
	static char mode[ASIZE(CMode) + KEYLEN + 20];
	unsigned int i = 0, j = 0;
	for(;i < ASIZE(CMode);i++) if(modes.modes & CMode[i].value) mode[j++] = CMode[i].mode;

	if(modes.modes & C_MLIMIT) j += sprintf(mode + j, " %d", modes.limit);
	if(modes.modes & C_MKEY)
	{
		mode[j++] = ' ';
		strcpy(mode + j, modes.key);
	}
	else mode[j] = '\0';
	return mode;
}

int db_write_users(void)
{
	anAccess *a;
	anUser *u;
#ifdef USE_MEMOSERV
	aMemo *memo;
#endif
 	int i = 0;
 	FILE *fp;

	rename(DBDIR "/" DBUSERS, DBDIR "/" DBUSERS ".back");

	if(!(fp = fopen(DBDIR "/" DBUSERS, "w")))
		return printf("DB::Write: Impossible d'crire la DB user: %s\n",
						strerror(errno));

	fprintf(fp, "V %d\n", DBVERSION_U);

	for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next)
	{
#ifdef HAVE_VOTE
		fprintf(fp, "NICK %s %s %d %lu %d %d %s %s %ld %s\n",
			u->nick, u->passwd, u->level, u->lastseen, u->flag, u->vote, u->mail,
			u->lastlogin ? u->lastlogin : "0", u->cantregchan, u->lang->langue);
#else
		fprintf(fp, "NICK %s %s %d %lu %d 0 %s %s %ld %s\n",
			u->nick, u->passwd,	u->level, u->lastseen, u->flag, u->mail,
			u->lastlogin ? u->lastlogin : "0", u->cantregchan, u->lang->langue);
#endif
		if(u->suspend) fprintf(fp, "SUSPEND %s %ld %ld :%s\n", u->suspend->from,
			u->suspend->expire, u->suspend->debut, u->suspend->raison);

#ifdef USE_MEMOSERV
		for(memo = u->memohead;memo;memo = memo->next)
			fprintf(fp, "MEMO %s %ld %d :%s\n", memo->de, memo->date, memo->flag, memo->message);
#endif

    	for(a = u->accesshead;a;a = a->next)
    		fprintf(fp, "ACCESS %s %d %d %lu :%s\n", a->c->chan, a->level,
    			a->flag, AOnChan(a) ? time(0) : a->lastseen, NONE(a->info));
	}

    fclose(fp);
    return 1;
}

int db_write_chans(void)
{
	FILE *fp;
	int i = 0;
	aChan *c;
	aBan *ban;

	rename(DBDIR "/" DBCHANS, DBDIR "/" DBCHANS ".back");

	if(!(fp = fopen(DBDIR "/" DBCHANS, "w")))
		return printf("DB::Wwrite: Impossible d'crire la DB chans: %s\n",
					strerror(errno));

	fprintf(fp, "V %d\n", DBVERSION_C);

	for(;i < CHANHASHSIZE;++i) for(c = chan_tab[i];c; c = c->next)
	{
		fprintf(fp, "CHANNEL %s %s\nOPTIONS %d %d %d %d %lu %d %d\nURL %s %lu\n",
			c->chan, c->description, c->flag, c->banlevel, c->bantype, c->cml,
			c->bantime, c->limit_min, c->limit_inc, c->url, c->creation_time);

		if(c->defmodes.modes) fprintf(fp, "DEFMODES %s\n", GetCModes(c->defmodes));
		if(*c->deftopic) fprintf(fp, "DEFTOPIC %s\n", c->deftopic);
		if(*c->welcome) fprintf(fp, "WELCOME %s\n", c->welcome);
		if(c->motd) fprintf(fp, "MOTD %s\n", c->motd);
		if(c->suspend) fprintf(fp, "SUSPEND %s %ld %ld :%s\n", c->suspend->from,
			c->suspend->expire, c->suspend->debut, c->suspend->raison);

		for(ban = c->banhead;ban;ban = ban->next)
			fprintf(fp, "B %s %s %lu %lu %d :%s\n", ban->mask, ban->de,
				ban->debut, ban->fin, ban->level, ban->raison);
	}

	fclose(fp);
	return 1;
}

int main(int argc, char **argv)
{
	if(chdir(BINDIR) < 0)
	{
		printf("BINDIR("BINDIR") n'est pas correctement configur : %s\n", strerror(errno));
		return 0;
	}

	if(argc <= 1) check_versions(1);
	else if(!strcmp("-c", argv[1]))
	{
		if(check_versions(0) < 0)
		{
			puts("C'est le premier lancement des services, il n'y a pas de DB.");
			return 0;
		}

		load_db();
		db_write_users();
		db_write_chans();
		puts("Les DB ont t converties");
	}
	return 0;
}
