/* src/hash.c dohash()
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: hash.c,v 1.18 2005/05/03 12:00:54 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "outils.h"
#include "timers.h"
#include <ctype.h>

unsigned int do_hashc(const char *chan)
{
	unsigned int checksum = 0;
	while(*chan) checksum += (checksum << 3) + tolower((unsigned char) *chan++);
	return checksum & (CHANHASHSIZE-1);
}

anUser *GetUserIbyMail(const char *mail)
{
	anUser *u;
	int i = 0;
	for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next)
		if(!strcasecmp(mail, u->mail)) return u;
	return NULL;
}

int ChanLevelbyUserI(anUser *user, aChan *chan)
{
	anAccess *acces = GetAccessIbyUserI(user, chan);
	return acces ? acces->level : 0;
}

aJoin *getjoininfo(aNick *nick, const char *chan)
{
	register aJoin *join = nick->joinhead;
	for(;join && strcasecmp(chan, join->channel);join = join->next);
	return join;
}

aJoin *GetJoinIbyC(aNick *nick, aChan *chan)
{
	register aJoin *join = nick->joinhead;
	for(;join && chan != join->chan;join = join->next);
	return join;
}

aDNR *find_dnr(const char *pattern, int type)
{
	aDNR *dnr = dnrhead;

	for(;dnr;dnr = dnr->next)
		if((type & dnr->flag) & (DNR_TYPECHAN|DNR_TYPEUSER)
			&& ((type & DNR_MASK && !mmatch(dnr->mask, pattern))
			|| (!(type & DNR_MASK) && !strcasecmp(dnr->mask, pattern)))) break;
	return dnr;
}

char *IsAnOwner(anUser *user)
{
	anAccess *acces = user->accesshead;
	for(;acces && acces->level != OWNERLEVEL;acces = acces->next);
	return acces ? acces->c->chan : NULL;
}

anAccess *GetAccessIbyUserI(anUser *user, aChan *chan)
{
	register anAccess *a = user->accesshead;
	for(;a && a->c != chan; a = a->next);
	return (a && !AWait(a)) ? a : NULL;
}

aNick *GetMemberIbyNick(aChan *chan, const char *nick)
{
	aLink *l = chan->members;
	for(;l && strcasecmp(l->value.j->nick->nick, nick);l = l->next);
	return l ? l->value.j->nick : NULL;
}

aServer *num2servinfo(const char *num)
{
	unsigned int servindex = base64toint(num);

	if(servindex > MAXNUM-1) return NULL;
	else return serv_tab[servindex];
}

aServer *GetLinkIbyServ(const char *serv)
{
	int i = 0;
	for(;i < MAXNUM;++i) if(serv_tab[i] && !strcasecmp(serv_tab[i]->serv, serv)) return serv_tab[i];
	return NULL;
}

int handle_suspend(struct suspendinfo **suspend, const char *nick,
	const char *ptr, time_t timeout)
{
	struct suspendinfo *s_p = *suspend;

	/* purge suspend details */
	if(ptr && !strcasecmp(ptr, "-nolog"))
	{
		if(s_p && s_p->timer) timer_remove(s_p->timer);
		free(s_p), *suspend = NULL;
	}
	else if(!(s_p && (!s_p->expire || s_p->expire > CurrentTS)))
	{	/* ACT! */
		if(!ptr || !*ptr) return -1;
		do_suspend(suspend, ptr, nick, timeout, CurrentTS);
		return 1;
	}
	else /* try to update some details (expiration, reason) */
	{	/* otherwise unsuspend it */
		time_t old_expire = s_p->expire;
		if(timeout) s_p->expire = timeout + CurrentTS;
		else if(ptr && *ptr) Strncpy(s_p->raison, ptr, sizeof s_p->raison -1);
		else s_p->expire = -1;

		if(old_expire != s_p->expire)
		{
			if(s_p->timer) timer_dequeue(s_p->timer); /* dequeue old timer */
			/* Not suspended any more, free timer */
			if(s_p->expire < 0)
			{
				if(s_p->timer) timer_free(s_p->timer), s_p->timer = NULL;
			}
			else if(!old_expire) s_p->timer = timer_add(s_p->expire, TIMER_ABSOLU,
										callback_unsuspend, suspend, NULL);
			else s_p->timer->delay = s_p->expire, timer_enqueue(s_p->timer);
		}
	}
	return 0;
}

void do_suspend(struct suspendinfo **suspend, const char *raison, const char *from,
	time_t expire, time_t debut)
{
	if(!*suspend) *suspend = calloc(1, sizeof **suspend);
	Strncpy((*suspend)->raison, raison, sizeof (*suspend)->raison -1);
	Strncpy((*suspend)->from, from, NICKLEN);
	(*suspend)->data = NULL;
	(*suspend)->debut = debut;
	expire = expire > 0 ? (expire + debut > CurrentTS ? expire + debut : -1) : expire;
	(*suspend)->expire = expire;
	(*suspend)->timer = expire > 0 ? timer_add((*suspend)->expire, TIMER_ABSOLU,
									callback_unsuspend, suspend, NULL) : NULL;
}

