/* src/hash_user.c - gestion des hash user
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: hash_user.c,v 1.17 2005/04/22 22:29:44 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "debug.h"
#include "outils.h"
#include "fichiers.h"
#include "del_info.h"
#include "cs_cmds.h"
#include <ctype.h>
#include "timers.h"
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif

static inline unsigned int do_hashu(const char *user)
{
	unsigned int checksum = 0;
	while(*user) checksum += (checksum << 3) + tolower((unsigned char) *user++);
	return checksum & (USERHASHSIZE-1);
}

static int hash_deluser(anUser *user)
{
	unsigned int hash = do_hashu(user->nick);
	anUser *tmp = user_tab[hash];

	if(tmp == user) user_tab[hash] = user->next;
	else
	{
		for(;tmp && tmp->next != user;tmp = tmp->next);
		if(tmp) tmp->next = user->next;
		else Debug(W_WARN|W_MAX, "H_del_user %s non trouv  l'offset %u ?!", user->nick, hash);
	}
	return 0;
}

int switch_user(anUser *user, const char *newuser)
{
	unsigned int hash = do_hashu(newuser);

	hash_deluser(user);
	Strncpy(user->nick, newuser, NICKLEN);
	user->next = user_tab[hash];
	user_tab[hash] = user;
	if(user->n) cs_account(user->n, user);
	return 0;
}

anUser *getuserinfo(const char *nick)
{
	unsigned int hash = do_hashu(nick);
	register anUser *tmp = user_tab[hash];
	for(;tmp && strcasecmp(nick, tmp->nick);tmp=tmp->next);
	return tmp;
}

anUser *add_regnick(const char *user, const char *pass, time_t lastseen,
					int level, int flag, const char *mail, int vote)
{
	anUser *u = calloc(1, sizeof *u);
	unsigned int hash = do_hashu(user);

	if(!u)
	{
		Debug(W_MAX|W_WARN, "add_regnick, malloc a chou pour anUser %s", user);
		return NULL;
	}

	Strncpy(u->nick, user, NICKLEN);
	strcpy(u->passwd, pass);
	u->lastseen = lastseen;
	u->level = level;
	Strncpy(u->mail, mail, MAILLEN);
#ifdef HAVE_VOTE
	u->vote = vote;
#endif
	u->flag = flag;
	u->suspend = NULL;
	u->lastlogin = NULL;
	u->cantregchan = -1;

	u->next = user_tab[hash];
	user_tab[hash] = u;
	return u;
}

void del_regnick(anUser *user, int flag, const char *raison)
{
	anAccess *chan, *ct;
#ifdef USE_MEMOSERV
	aMemo *memo = user->memohead, *mem;
#endif
#ifdef HAVE_TRACK
	struct track *track = istrack(user);

	if(track)
	{
		csntc(track->tracker, "[\2Track\2] L'UserName %s que vous trackez vient d'tre supprim.",
			user->nick);
		del_track(user);
	}
#endif

	for(chan = user->accesshead;chan;chan = ct)
	{
		ct = chan->next;
		if(chan->level == OWNERLEVEL) del_chan(chan->c, flag|HF_SQLLOG, raison);
		else del_access(user, chan->c);
	}
#ifdef USE_MEMOSERV
	for(;memo;memo = mem)
	{
		mem = memo->next;
		free(memo);
	}
#endif

#ifdef SQLLOG
 	if(flag & HF_SQLLOG) sql_query(SQL_QINSERTU, "('%s', %T, 'purge', '%d')",
 							user->nick, CurrentTS, CurrentTS - user->lastseen);
#endif

	hash_deluser(user);
	SUSPEND_REMOVE(user->suspend);
	free(user);
}
