/* src/main.c - Fichier principal
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: main.c,v 1.98 2005/05/05 21:49:17 romexzf Exp $
 */

#include "main.h"
#include <unistd.h>
#include <signal.h>
#include <sys/resource.h>
#ifdef USEBSD
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <errno.h>
#include "config.h"
#include "debug.h"
#include "hash.h"
#include "outils.h"
#include "serveur.h"
#include "fichiers.h"
#include "chanserv.h"
#include "admin_user.h"
#include "nickserv.h"
#include "admin_cmds.h"
#include "admin_chan.h"
#include "admin_manage.h"
#include "divers.h"
#include "moduser.h"
#include "chanopt.h"
#include "opdeop.h"
#include "chaninfo.h"
#include "showcommands.h"
#include "aide.h"
#include "cs_register.h"
#include "cs_cmds.h"
#include "timers.h"
#include "socket.h"
#include "ban.h"
#include "flood.h"
#include "template.h"
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef HAVE_VOTE
#include "vote.h"
#endif
#ifdef USE_MEMOSERV
#include "memoserv.h"
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif
#ifdef WEB2CS
#include "web2cs.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif

int ConfFlag = 0;
int running = 1;
time_t CurrentTS = 0;

#ifdef USE_NICKSERV
aKill *killhead = NULL;
#endif
struct ignore *ignorehead = NULL;
Lang *DefaultLang = NULL;
aDNR *dnrhead = NULL;
Timer *Timers = NULL;

struct robot bot = {{0}};
#ifdef USE_WELCOMESERV
struct bots ws = {{0}};
char user_motd[400] = {0};
char admin_motd[400] = {0};
#endif
struct bots cs = {{0}};

aChan *chan_tab[CHANHASHSIZE] = {0}; 	/* hash chan */
anUser *user_tab[USERHASHSIZE] = {0}; 	/* hash username */
aNick **num_tab[MAXNUM] = {0}; 			/* table num -> struct nick */
aServer *serv_tab[MAXNUM] = {0}; 		/* table numserv -> serv info */
aNick *nick_tab[NICKHASHSIZE] = {0}; 	/* hash nick */

/*            FONCTIONS COMMANDES
 *            =-=-=-=-=-=-=-=-=-=
 *
 *    Il y a deux types de fonctions commandes.
 *    Les normales :
 *       parv[0] = pseudo du robot appel
 *       parv[1->parc] = arguments de la commandes
 *    Et les commandes salons (voir strict cs_commands) :
 *       parv[0] = nom de la commande appele
 *       parv[1] = salon appel
 *       parv[2->parv] = arguments de la commande (aprs le nom du salon)
 *    Toutes les commandes doivent respecter ce schma.
 *    Lorsqu'une commande salon est tape sur un salon, et que le premier
 *    argument n'est pas un nom de salon, le nom du salon actuel est plac
 *    automatiquement comme premier argument.
 *
 *    Chaque commande emploie une entte particulire :
 *    int commande(aNick *, aChan *, int, char **)
 *      aNick *nick = structure nickinfo de l'user qui a appel la fonction
 *      aChan *chaninfo = structure chaninfo du salon si la cmd est chan=1 (sinon chaninfo=NULL)
 *      int parc = nombre d'arguments *SANS COMPTER PARV[0]*. (si le dernier arg
 *                 est parv[5], alors parc = 5.)
 *      char **parv = tableau contenant les arguments (une ligne = un mot).
 *
 *    Outils utiles :
 *      parv2msgn(int parc, char **parv, int position, char *buf, int size)
 *          permet de convertir une fin de tableau en un message normal.
 *          il faut spcifier le tableau et le nombre d'arguments contenu
 *          ainsi que la position  laquelle on va commencer  enregistrer
 *          le contenu dans la variable envoye en paramtre (taille max size).
 *		parv2msg(int parc, char **parv, int position, int size)
 *			idem mais renvoit un pointeur vers la string forme plutot que l'crire dans buf
 *      num2nickinfo(const char *num)
 *          renvoit la struct de la num spcifie (accs direct dsormais avec la table des nums)
 *      csntc(aNick *nick, register const char *format, ...)
 *          envoyer une notice avec le CS
 *    Pour plus d'outils, voir outils.c.
 */

void sig_die(int c)
{
	Debug(W_WARN|W_MAX, "SIGTERM reu! -- abandon..");
	running = 0;
}

void sig_restart(int c)
{
	Debug(W_WARN|W_MAX, "SIGINT reu! -- Restarting..");
	running = 0;
	ConfFlag |= CF_RESTART;

	putserv("%s "TOKEN_QUIT" :Restarting", cs.num);
	putserv("%s "TOKEN_SQUIT" %s 0 :SIGINT/RESTART", bot.servnum, bot.server);
}

void sig_reload(int c)
{
	Debug(W_WARN, "Signal HUP reu!");
	signal(SIGHUP, &sig_reload);
}

int main(int argc, char **argv)
{
	int restarted = 0, silent = 0, tmp = 0, background = 1;
	FILE *fd; /* pid & uptime */
    struct rlimit rlim; /* used for core size */

	CurrentTS = time(NULL);

	while((tmp = getopt(argc, argv, "hn")) != EOF)
		switch(tmp)
		{
			case 'n':
				background = 0;
				break;
			case 'h':
				silent = 1;
				break;
			default:
				printf("Syntaxe: %s [-hn]\n", argv[0]);
				exit(EXIT_FAILURE);
		}

	chdir(BINDIR); /* on se place dans le rpertoire principal de scoderz */

	if((fd = fopen(SCODERZ_PID, "r")))
	{
		if(fscanf(fd, "%d", &tmp) == 1)
		{
			fprintf(stderr, "SCoderZ est dj lanc sur le pid %d.\n"
			"Si ce n'est pas le cas, supprimez le fichier '"SCODERZ_PID"' et recommencez.\n", tmp);
		}
		fclose(fd);
		exit(EXIT_FAILURE);
	}

	if(load_config(FICHIER_CONF) == -1)
	{
		fputs("Erreur lors de la lecture de la configuration\n", stderr);
		exit(EXIT_FAILURE);
	}

	for(tmp = 0;tmp < LANGMSGNB;++tmp)
		if(!DefaultLang->msg[tmp])
		{
			Debug(W_TTY, "lang: langage par dfaut incomplet (msgid %d manquante)", tmp);
			exit(EXIT_FAILURE);
		}

	if((fd = fopen("uptime.tmp", "r")))
	{
		fscanf(fd, "%lu", (unsigned long *) &bot.uptime);
		fclose(fd);
		remove("uptime.tmp");
		restarted = 1;
	}
	else bot.uptime = CurrentTS;

	RegisterCmd("die", 			4, CMD_ADMIN, 0, die);
	RegisterCmd("rehash",		4, CMD_ADMIN, 0, rehash_conf);
	RegisterCmd("restart", 		4, CMD_ADMIN, 0, restart_bot);
	RegisterCmd("adminlvl", 	4, CMD_ADMIN, 2, admin_level);
	RegisterCmd("chcomname", 	4, CMD_ADMIN, 2, chcomname);
	RegisterCmd("chlevel", 		4, CMD_ADMIN, 2, chlevel);
	RegisterCmd("disable", 		4, CMD_ADMIN, 2, disable_cmd);
	RegisterCmd("inviteme",  	2, CMD_ADMIN, 0, inviteme);
	RegisterCmd("write", 		2, CMD_ADMIN, 0, write_files);
	RegisterCmd("global", 		3, CMD_ADMIN, 2, globals_cmds);
	RegisterCmd("chan", 		2, CMD_ADMIN, 1, admin_chan);
#ifdef HAVE_TRACK
	RegisterCmd("track", 		2, CMD_ADMIN, 1, cmd_track);
#endif
	RegisterCmd("user", 		2, CMD_ADMIN, 2, admin_user);
	RegisterCmd("whois", 		2, CMD_NEEDNOAUTH|CMD_ADMIN, 1, cs_whois);
#ifdef HAVE_VOTE
	RegisterCmd("vote", 		3, CMD_ADMIN, 1, do_vote);
#endif
	RegisterCmd("showconfig", 	4, CMD_NEEDNOAUTH|CMD_ADMIN, 0, showconfig);
#ifdef USE_WELCOMESERV
	RegisterCmd("setglobwelcome", 4, CMD_ADMIN, 0, set_welcome);
	RegisterCmd("addwelcome", 	4, CMD_ADMIN, 2, add_welcome);
	RegisterCmd("delwelcome", 	4, CMD_ADMIN, 1, del_welcome);
	RegisterCmd("listwelcome", 	2, CMD_NEEDNOAUTH|CMD_ADMIN, 0, list_welcome);
	RegisterCmd("adminmotd", 	4, CMD_ADMIN, 1, set_motds);
#endif
	RegisterCmd("dnrchan", 		3, CMD_ADMIN, 1, dnrchan_manage);
	RegisterCmd("dnruser", 		3, CMD_ADMIN, 1, dnruser_manage);
	RegisterCmd("whoison", 		2, CMD_ADMIN | CMD_CHAN, 0, whoison);
	RegisterCmd("say", 			2, CMD_ADMIN | CMD_CHAN, 2, admin_say);
	RegisterCmd("do", 			2, CMD_ADMIN | CMD_CHAN, 2, admin_do);

	RegisterCmd("register", 	0, CMD_NEEDNOAUTH | CMD_SECURE, 3, register_user);
	RegisterCmd("login", 		0, CMD_NEEDNOAUTH | CMD_SECURE, 2, ns_login);
#ifdef USE_NICKSERV
	RegisterCmd("recover", 		0, CMD_NEEDNOAUTH | CMD_SECURE, 1, recover);
#endif
	RegisterCmd("myaccess", 	1, 0, 0, myaccess);
	RegisterCmd("drop", 		1, CMD_SECURE, 1, drop_user);
	RegisterCmd("deauth", 		1, CMD_DISABLE, 0, deauth);
	RegisterCmd("oubli", 		0, CMD_NEEDNOAUTH, 2, oubli_pass);
	RegisterCmd("set", 			1, 0, 1, user_set);
#ifdef USE_MEMOSERV
	RegisterCmd("memos", 		1, 0, 1, memos);
#endif
#ifdef HAVE_VOTE
	RegisterCmd("voter", 		1, CMD_SECURE2, 0, voter);
	RegisterCmd("results", 		1, 0, 0, vote_results);
#endif
	RegisterCmd("help", 		0, CMD_NEEDNOAUTH, 0, aide);
	RegisterCmd("aide", 		0, CMD_NEEDNOAUTH, 0, aide);
	RegisterCmd("showcommands", 0, CMD_NEEDNOAUTH, 0, showcommands);
	RegisterCmd("admin", 		0, CMD_NEEDNOAUTH, 0, show_admins);
	RegisterCmd("uptime", 		0, CMD_NEEDNOAUTH, 0, uptime);
	RegisterCmd("seen", 		1, CMD_NEEDNOAUTH, 1, lastseen);

	RegisterCmd("verify", 		0, CMD_NEEDNOAUTH, 1, verify);
	RegisterCmd("ignorelist", 	1, CMD_NEEDNOAUTH, 0, show_ignores);
	RegisterCmd("\1ping", 		0, CMD_NEEDNOAUTH, 0, ctcp_ping);
	RegisterCmd("\1version\1", 	0, CMD_NEEDNOAUTH, 0, ctcp_version);

	RegisterCmd("regchan", 		1, 0, 2, register_chan);
	RegisterCmd("unreg", 		OWNERLEVEL, CMD_CHAN, 1, unreg_chan);
	RegisterCmd("renchan", 		OWNERLEVEL, CMD_CHAN, 2, ren_chan);
	RegisterCmd("access", 		0, CMD_NEEDNOAUTH|CMD_CHAN, 2, show_access);
	RegisterCmd("banlist", 		0, CMD_NEEDNOAUTH|CMD_CHAN, 0, banlist);
	RegisterCmd("chaninfo", 	0, CMD_NEEDNOAUTH|CMD_CHAN, 1, chaninfo);
	RegisterCmd("alist", 		100, CMD_NEEDNOAUTH|CMD_CHAN, 1, see_alist);
	RegisterCmd("info", 		100, CMD_CHAN, 0, info);
	RegisterCmd("invite", 		100, CMD_CHAN, 0, invite);
	RegisterCmd("adduser", 		450, CMD_CHAN, 3, add_user);
	RegisterCmd("deluser", 		450, CMD_CHAN, 2, del_user);
	RegisterCmd("ban", 			100, CMD_CHAN, 2, ban_cmd);
	RegisterCmd("unban", 		300, CMD_CHAN, 2, unban);
	RegisterCmd("kickban", 		300, CMD_CHAN, 2, kickban);
	RegisterCmd("clearbans", 	300, CMD_CHAN, 0, clear_bans);
	RegisterCmd("unbanme", 		1, CMD_CHAN, 1, unbanme);
	RegisterCmd("kick", 		100, CMD_CHAN, 2, kick);
	RegisterCmd("mode", 		100, CMD_CHAN, 2, mode);
	RegisterCmd("topic", 		100, CMD_CHAN, 2, topic);
	RegisterCmd("opall", 		400, CMD_CHAN, 1, opall);
	RegisterCmd("deopall", 		400, CMD_CHAN, 1, deopall);
	RegisterCmd("clearmodes", 	400, CMD_CHAN, 0, clearmodes);
	RegisterCmd("voiceall", 	300, CMD_CHAN, 1, voiceall);
	RegisterCmd("devoiceall", 	300, CMD_CHAN, 1, devoiceall);
	RegisterCmd("op", 			100, CMD_CHAN, 1, op);
	RegisterCmd("deop", 		100, CMD_CHAN, 1, deop);
	RegisterCmd("devoice", 		50, CMD_CHAN, 1, devoice);
	RegisterCmd("voice", 		50, CMD_CHAN, 1, voice);
	RegisterCmd("rdefmodes", 	300, CMD_CHAN, 1, rdefmodes);
	RegisterCmd("rdeftopic", 	300, CMD_CHAN, 1, rdeftopic);
	RegisterCmd("moduser", 		400, CMD_CHAN, 3, generic_moduser);
	RegisterCmd("autoop", 		400, CMD_CHAN, 1, moduser_autoop);
	RegisterCmd("autovoice", 	400, CMD_CHAN, 1, moduser_autovoice);
	RegisterCmd("protect", 		400, CMD_CHAN, 1, moduser_protect);
	RegisterCmd("locktopic", 	400, CMD_CHAN, 0, locktopic);
	RegisterCmd("strictop", 	450, CMD_CHAN, 1, strictop);
	RegisterCmd("nobans", 		450, CMD_CHAN, 0, nobans);
	RegisterCmd("noops", 		450, CMD_CHAN, 0, noops);
	RegisterCmd("deftopic", 	400, CMD_CHAN, 1, deftopic);
	RegisterCmd("defmodes", 	400, CMD_CHAN, 1, defmodes);
	RegisterCmd("description", 	450, CMD_CHAN, 1, description);
	RegisterCmd("welcome", 		450, CMD_CHAN, 1, csetwelcome);
	RegisterCmd("setwelcome", 	450, CMD_CHAN, 1, activwelcome);
	RegisterCmd("banlevel", 	400, CMD_CHAN, 1, banlevel);
	RegisterCmd("bantype", 		400, CMD_CHAN, 1, bantype);
	RegisterCmd("chanurl", 		450, CMD_CHAN, 1, chanurl);
	RegisterCmd("motd", 		450, CMD_CHAN, 1, define_motd);
	RegisterCmd("chanopt", 		450, CMD_CHAN, 2, generic_chanopt);

	if(!restarted && !silent) puts("Services CoderZ " SPVERSION "  2002-2005 Inter System");

	tmp = db_load_chans();
	if(!silent) printf("Base de donne Salon charge (%d)\n", tmp);
	tmp = db_load_users();
	if(!silent) printf("Base de donne User charge (%d)\n", tmp);
	if(tmp)
	{
		tmp = load_cmds();
		if(!silent) printf("Chargement des commandes IRC... OK (%d)\n", tmp);
		tmp = load_dnr();
		if(!silent) printf("Chargement des DNR mask... OK (%d)\n", tmp);
#ifdef USE_WELCOMESERV
		load_welcome();
#endif
#ifdef HAVE_VOTE
		load_votes();
#endif
	} /* first time */
	BuildCommandsTable(0);
	tmpl_load(); /* load templates for mails */
	help_load(NULL); /* load all languages */

	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP,	&sig_reload);
	signal(SIGINT, 	&sig_restart);
	signal(SIGTERM, &sig_die);

	if(background && (tmp = fork()) == -1)
	{
		Debug(W_TTY, "Impossible de se lancer en background.");
		exit(EXIT_FAILURE);
	}
	else if(background && tmp > 1) /* fork ok */
	{
		if(!restarted && !silent) puts("Lancement en background...");
		exit(0);
	}

	if(!getrlimit(RLIMIT_CORE, &rlim) && rlim.rlim_cur != RLIM_INFINITY)
	{
		Debug(W_TTY, "Core size limite  %ldk, changement en illimit.", rlim.rlim_cur);
		rlim.rlim_cur = RLIM_INFINITY;
		rlim.rlim_max = RLIM_INFINITY;
		setrlimit(RLIMIT_CORE, &rlim);
	}

#ifdef SQLLOG
	if(!sql_init(1))
	{
		Debug(W_TTY, "log:SQL connexion  %s:%d (user=%s base=%s) impossible...",
				bot.sql_host, bot.sql_port, bot.sql_user, bot.sql_db);
		exit(EXIT_FAILURE);
	}
#endif

	if(GetConf(CF_PREMIERE)) printf("Lorsque les services seront sur votre rseau IRC,\n"
		"tapez : /%s %s <username> <email> <email> <pass>\n", cs.nick, RealCmd("register"));

	FD_ZERO(&global_fd_set);
#ifdef WEB2CS
	w2c_initsocket();
#endif

#ifdef N2K
#	define NNICK "AA"
#else
#	define NNICK "A"
#endif

    sprintf(cs.num, "%s"NNICK"A", bot.servnum);
#ifdef USE_WELCOMESERV
    sprintf(ws.num, "%s"NNICK"B", bot.servnum);
#endif

	if((fd = fopen(SCODERZ_PID, "w")))
	{
		fprintf(fd, "%d\n", (int) getpid());
		fclose(fd);
	}
	else Debug(W_TTY, "Impossible d'crire le fichier PID. [%s]", strerror(errno));

	timer_add(PURGEDELAY, TIMER_PERIODIC, callback_check_accounts, NULL, NULL);
	timer_add(PURGEDELAY, TIMER_PERIODIC, callback_check_chans, NULL, NULL);
	timer_add(cf_write_delay, TIMER_PERIODIC, callback_write_dbs, NULL, NULL);

	while(running)
	{
		unsigned long int irc_ip = inet_addr(bot.ip);

		if(irc_ip == INADDR_NONE) putlog(LOG_ERREURS, "DNS lockup a chou pour [%s]", bot.ip);
		else if(init_bot(irc_ip) < 0) putlog(LOG_ERREURS, "Connexion choue.");
		else run_bot(bot.sock);

		if(running)
		{
			aChan *chan;
			putlog(LOG_ERREURS, "Dconnect du serveur.");

			purge_nickandserv();

			for(tmp = 0;tmp < CHANHASHSIZE;++tmp)
				for(chan = chan_tab[tmp];chan;chan = chan->next) DelCJoined(chan);
			sleep(WAIT_CONNECT);
		}
	}

	if(!GetConf(CF_PREMIERE))
	{
		db_write_users();
		db_write_chans();
		write_cmds();
	}
	remove(SCODERZ_PID);
	CleanUp();
	putlog(LOG_PARSES, "Fermeture normale du programme");
	if(GetConf(CF_RESTART)) execlp(argv[0], argv[0], "-h", NULL); /* restarting.. */
	return 0;
}
