/* src/memoserv.c - Diverses commandes sur le module memoserv
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: memoserv.c,v 1.43 2005/04/20 23:19:20 romexzf Exp $
 */

#include "main.h"
#ifdef USE_MEMOSERV
#include "memoserv.h"
#include "fichiers.h"
#include "aide.h"
#include "hash.h"
#include "outils.h"
#include "cs_cmds.h"
#include "add_info.h"
#include "del_info.h"

void show_notes(aNick *nick)
{
	aMemo *memo = nick->user->memohead;
	int i = 0;

	for(;memo;memo = memo->next) if(!MemoRead(memo)) ++i;

	if(i) csntc(nick, GetReply(nick, L_HAVENEWMEMO), i, PLUR(i), cs.nick, RealCmd("memos"), PLUR(i));
}

int memo_del(anUser *user, int *todel, int i)
{
	aMemo *memo = user->memohead, *mnext, *mlast = NULL;
	int p = 0, found = 0, tmp = 0;

	for(;memo;memo = mnext)
	{
		mnext = memo->next;
		++p;
		if(!i) free(memo), ++found;
		else if((tmp = item_isinlist(todel, i, p)) != -1) /* need to be deleted */
		{
			if(mlast) mlast->next = mnext;
			else user->memohead = mnext;
			todel[tmp] = -todel[tmp]; /* mark as deleted */
			free(memo);
			if(i == ++found) break; /* found all memos in list, drop! */
		}
		else mlast = memo;
	}
	if(!i && found) user->memohead = NULL; /* all */
	return found;
}

int memos(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aMemo *memo = nick->user->memohead, *mnext, *mlast = NULL;
	int i = 0, p = 0, found = 0;
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "LIRE") || !strcasecmp(cmd, "READ"))
	{	/* default (no arg) is NEW. */
		if(parc < 2 || !strcasecmp(parv[2], "NEW")) i = -2;
		else if(!strcasecmp(parv[2], "ALL")) i = -1;
		else if(is_num(parv[2])) i = atoi(parv[2]);
		else return csntc(nick, "Syntaxe : %s LIRE <ALL|NEW|N DU MEMO>", parv[0]);

		for(;memo;memo = mnext)
		{
			mnext = memo->next;
			if(++p == i || i == -1 || (i == -2 && !MemoRead(memo)))
			{
				csntc(nick, "%c#%d De\2 %s\2 [%s] : %s", MemoRead(memo)	? ' ' : '*',
					p, memo->de, get_time(nick, memo->date), memo->message);
				found = 1;
				if(memo->flag & MEMO_AUTOEXPIRE)
				{
					if(mlast) mlast->next = mnext;
					else nick->user->memohead = mnext;
					free(memo);
					continue; /* do not update mlast */
				}
				if(!MemoRead(memo)) memo->flag |= MEMO_READ;
			}
			mlast = memo;
		}

		if(found)
		{
			if(i == -1) csntc(nick, GetReply(nick, L_MEMOTOTAL), p, PLUR(p), cs.nick, parv[0]);
			else if(i == -2) csntc(nick, GetReply(nick, L_MEMONEWTOTAL), p, PLURX(p), PLUR(p));
			else csntc(nick, GetReply(nick, L_MEMOEND), i, cs.nick, parv[0], i);
		}
		else if(i == -1) csntc(nick, GetReply(nick, L_NOMEMOFOUND));
		else if(i == -2) csntc(nick, GetReply(nick, L_MEMONEWNOTFOUND));
		else csntc(nick, GetReply(nick, L_MEMONOTFOUND), i);
	}
	else if(!strcasecmp(cmd, "SUPPR") || !strcasecmp(cmd, "ERASE")
		|| !strcasecmp(cmd, "SUPPRIMER") || !strcasecmp(cmd, "DEL"))
	{
		int todel[10];

		if(parc < 2) return csntc(nick, "Syntaxe : %s SUPPR <ALL|N DU MEMO>", parv[0]);

		if(strcasecmp(parv[2], "all") && !(i = item_parselist(parv[2], todel, ASIZE(todel))))
			return csntc(nick, "Syntaxe : %s SUPPR <ALL|n,list>", parv[0]);

		found = memo_del(nick->user, todel, i);

		if(!i) csntc(nick, found ? GetReply(nick, L_MEMOSDEL) : GetReply(nick, L_NOMEMOFOUND), found);

		for(p = 0;p < i;++p) /* report results */
			if(todel[p] < 0) csntc(nick, GetReply(nick, L_MEMODEL), -todel[p]);
			else csntc(nick, GetReply(nick, L_MEMONOTFOUND), todel[p]);
	}
	else if(!strcasecmp(cmd, "SEND") || !strcasecmp(cmd, "ENVOYER"))
	{
		anUser *user;
		aNick *who;
		char *tmp;

		if(parc < 3)
			return csntc(nick, "Syntaxe : %s SEND <username> <message>", parv[0]);

		if(!(user = ParseNickOrUser(nick, parv[2]))) return 0;

		if(user == nick->user)
			return csntc(nick, GetReply(nick, L_CANTSENDYOURSELF));

		if(UNoMemo(user) && !IsAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_XNOMEMO), user->nick);

		tmp = parv2msg(parc, parv, 3, MEMOLEN + 3);
		if(*tmp == ':') tmp++;

		if(strlen(tmp) > MEMOLEN)
			return csntc(nick, GetReply(nick, L_MEMOLEN), MEMOLEN);

		if((who = user->n) && !IsAway(who))
			csntc(who, GetReply(who, L_MEMOFROM), nick->nick, nick->user->nick, tmp);

		for(memo = user->memohead;memo;memo = memo->next)
			if(!strcasecmp(memo->de, nick->user->nick)) ++i;

		if(i >= MAXMEMOS && !IsAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_MAXMEMO), user->nick, MAXMEMOS);

		add_memo(user, nick->user->nick, CurrentTS, tmp, (who && !IsAway(who)) ? MEMO_READ : 0);
		csntc(nick, GetReply(nick, L_MEMOSENT), user->nick);
		csntc(nick, "\2Texte:\2 %s", tmp);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), cmd);
	return 1;
}
#endif
