/* src/serveur.c - Traitement des messages IRC
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: serveur.c,v 1.213 2005/05/13 20:55:10 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "serveur.h"
#include "add_info.h"
#include "admin_manage.h"
#include "hash.h"
#include "flood.h"
#include "cs_cmds.h"
#include "config.h"
#include "del_info.h"
#include "debug.h"
#include "aide.h"
#include "timers.h"
#include <sys/time.h>
#ifdef USE_MEMOSERV
#include "memoserv.h"
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif

aServer *mainhub = NULL;
static int burst = 0;

#ifndef N2K
/*
 * m_account SS|SSCCC AC SSCCC account|""
 */
int m_account(int parc, char **parv)
{
	aNick *nick = NULL;
	anUser *user;

	if(parc < 3 || !(nick = num2nickinfo(parv[1])))
		return Debug(W_DESYNCH|W_PROTO|W_WARN, "Erreur dans un TOKEN AC de %s: arg=%d nick=%s",
					parv[0], parc, nick ? nick->nick : "NULL");

	if(!(user = getuserinfo(parv[2])) || user->n)
	{
		aServer *links = num2servinfo(parv[0]);
		cswallops("ACCOUNT %s (%s) pour %s depuis %s", user ? "Desynch" : "inconnu",
			parv[2], nick->nick, links->serv);
		cs_account(nick, NULL); /* desacounter :) */
		return 0;
	}

	nick->user = user;
	nick->user->n = nick;
	nick->user->lastseen = CurrentTS;
#ifdef USE_NICKSERV
	if(IsProtected(user) && !strcasecmp(user->nick, nick->nick)) kill_remove(nick); /* arret du kill */
#endif
	return 0;
}

/*
 * m_setident SCC SETIDENT ident
 */
int m_setident(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc > 1 && who) Strncpy(who->ident, parv[1], USERLEN);
	return 0;
}

/*
 * m_sethost SCC SETHOST host.com
 */
int m_sethost(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2 || !who || !*parv[1] || strchr(parv[1], ':') || !strchr(parv[1], '.')) return 0;
#ifdef HAVE_CRYPTHOST
	Strncpy(who->crypt, parv[1], HOSTLEN);
#else
	Strncpy(who->host, parv[1], HOSTLEN);
#endif
	return 0;
}

#else/* N2K*/

/*
 * m_clearmode SSCCC CM # mode_cleared
 */
int m_clearmode(int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);
	int clear = 0;

	if(strchr(parv[2], 'o')) clear |= J_OP;
	if(strchr(parv[2], 'v')) clear |= J_VOICE;

	if(chan && !IsSuspend(chan))
	{
		aLink *j;
		char mode[30] = "-";

		Strncpy(mode + 1, parv[2], sizeof mode -2);
		string2scmode(&chan->mode, mode, strchr(parv[2], 'k'), NULL);
		/* string2scmode n'enleve le +k que si une key est fournie, ce *hack* joue bien le jeu */
		if(!clear) return 0; /* aucun changement de +o/+v */
		for(j = chan->members;j;j = j->next)
			 if(!(j->value.j->nick->flag & (N_SERVICE | N_GOD))) j->value.j->status &= ~clear;
	}
	else
	{
		register aNick *nick;
		aJoin *join;
		int i = 0;

		if(!clear) return 0; /* aucun changement de +o/+v */
		for(;i < NICKHASHSIZE;++i)
			for(nick = nick_tab[i];nick;nick = nick->next)
				if(!(nick->flag & (N_SERVICE|N_GOD)) && (join = getjoininfo(nick, parv[1])))
					join->status &= ~clear;
	}
	return 0;
}
#endif

/*
 * m_mode SSCCC M #|SSCCC +|-m
 */
int m_mode(int parc, char **parv)
{
	aNick *nick = GetInfobyPrefix(parv[0]), *nptr = NULL;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: MODE de %s: #arg=%d", parv[0], parc);
	if(*parv[1] == '#')
	{
		aChan *ch = getchaninfo(parv[1]);
		char *modes = parv[2], *param = NULL, *key = NULL, *limit = NULL;
		anAccess *a = NULL, *a2 = NULL;
		aJoin *join = NULL;
		aLink *j = NULL;
		int what = 1, i = 2, l = 0, obv = 1;

		if(ch && IsSuspend(ch)) ch = NULL;
		while (*modes)
		{
			switch (*modes)
			{
				case '+':
					what = 1;
					break;
				case '-':
					what = 0;
					break;
				case 'o':
					param = parv[++i];
					if(!(nptr = num2nickinfo(param)))
					{
						Debug(W_DESYNCH|W_WARN, "m_mode: aNick non trouv pour '%s %co %s'", parv[1], what ? '+' : '-', param);
						break;
					}

					if(ch) /* salon reg */
					{
						if(!what) /* -o */
						{
							if(nptr->user && nick /* la victime a un accs ou est protge*/
							&& nick != nptr && (IsAdmin(nptr->user) || (a = GetAccessIbyUserI(nptr->user, ch))))
							{
								if(!nick->user || (nptr->user->level > nick->user->level ||/* l'OP n'est pas auth ou lvl infrieur*/
								(a && AProtect(a) && !ASuspend(a) && (a2 = GetAccessIbyUserI(nick->user, ch))
								&& !ASuspend(a2) && a2->level <= a->level)))/* ou accs infrieur/suspend*/
								{
									csmode(ch, MODE_OBV, "+o $", nptr->numeric);
									break;
								}
							}
							else if(!strcmp(cs.num, param)) /* deop du CServ */
							{ 	/* hack avec la Uline au cas o.. (ircu avec mode non dbrid etc.) */
								putserv("%s "TOKEN_MODE" %s +o %s", bot.servnum, ch->chan, cs.num);
								break;
							}
						}
						else /* +o */
						{ /* noops ou strictop */
							if(!IsOperOrService(nptr) && (CNoOps(ch)
							|| (CStrictOp(ch) && (!nptr->user || !(a = GetAccessIbyUserI(nptr->user, ch)) || ASuspend(a)))))
							{
								csmode(ch, MODE_OBV, "-o $", param);
								break;
							}
						}
					} /* mise  jour du status */

					if((join = ch ? GetJoinIbyC(nptr, ch) : getjoininfo(nptr, parv[1])))
					{
						if(what) DoOp(join);
						else DeOp(join);
					}
					break;

				case 'v': /* mise  jour du status */
					if((nptr = num2nickinfo(parv[++i]))
						&& (join = ch ? GetJoinIbyC(nptr, ch) : getjoininfo(nptr, parv[1])))
					{
						if(what) DoVoice(join);
						else DeVoice(join);
					}
					break;

				case 'b':
					param = parv[++i]; /* grab parameter (mask) */
					/* salon non reg, ou -b, ou service, on ignore, si ban perm, il sera enforc */
					if(!what || !ch || nick->flag & N_SERVICE) break;

					if(!CNoBans(ch))
					{
						if(!nick) break;
						l = nick->user ? ChanLevelbyUserI(nick->user, ch) : 0;
						if(l >= ch->banlevel)/* peut  priori bannir */
						{
							for(j = ch->members;j;j = j->next)
							{	/* recherche de personnes protges couvertes */
								nptr = j->value.j->nick;
								if(!nptr->user || match(param, GetNUHbyNick(nptr, 1))) continue;
								if((a = GetAccessIbyUserI(nptr->user, ch)) && AProtect(a) && !ASuspend(a)) break;
								else a = NULL;
							}
						}
					}

					if(CNoBans(ch) || ((ch->banlevel > l || a) && !IsAnAdmin(nick->user)))
						csmode(ch, MODE_OBV, "-b $", param);
					break;

				default: /* other modes than OBV! (Need Parse!) */
					if(*modes == 'l' && what) limit = parv[++i];
					else if(*modes == 'k') key = parv[++i];
					obv = 0;
			} /* switch */
			modes++;
		} /* while */

		if(ch && !obv) /* Only if registered AND some modes other than OBV */
		{ /* ChModesLevel check (cancel modes change if needed, a bit ugly but work) */
			if(nick && ch->cml && (!nick->user || (!IsAdmin(nick->user)
				&& (!(a = GetAccessIbyUserI(nick->user, ch)) || ASuspend(a) || a->level < ch->cml))))
			{
				char new[20] = {0}, mbuf[KEYLEN + 20] = {0}, last = 0;
				int n = 0, para = 0, m = 0;

				for(modes = parv[2]; *modes; modes++)
				{
					if(*modes == 'o' || *modes == 'v' || *modes == 'b') continue;
					else if(*modes == '+') new[n++] = last = '-';
					else if(*modes == '-') new[n++] = last = '+';
					else if(*modes == 'k') para |= 1;
					else if(*modes == 'l') para |= 2;
					else new[n++] = *modes;
				}
				if(para & 2) /* limit */
				{	/* sanity check: just avoid doubling mode */
					if((new[n] = HasMode(ch, C_MLIMIT) ? '+' : '-') != last) last = new[n++];
					new[n++] = 'l'; /* don't append limit if we're removing it */
					if(HasMode(ch, C_MLIMIT)) m = sprintf(mbuf, "%d", ch->mode.limit);
				}
				if(para & 1) /* key */
				{	/* same sanity check, but don't avoid -+|+- */
					if((new[n] = HasMode(ch, C_MKEY) ? '+' : '-') != last) n++;
					new[n] = 'k';
					if(m) mbuf[m++] = ' '; /* append the key */
					strcpy(mbuf + m, HasMode(ch, C_MKEY) ? ch->mode.key : key);
				} /* then, flush mode */
				putserv("%s "TOKEN_MODE" %s %s %s", cs.num, ch->chan, new, mbuf);
			}
			else string2scmode(&ch->mode, parv[2], key, limit); /* parse */
		}
	} /* if(ch) */
	else if(nick) nick->flag = parse_umode(nick->flag, parv[2]);
	return 0;
}

/*
 * m_part ABAAA L # [:raison]
 */
int m_part(int parc, char **parv)
{
	char *p, *ptr = NULL;
	aNick *nick = NULL;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: PART de %s: #arg=%d", parv[0], parc);

	if(!(nick = GetInfobyPrefix(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "PART %s de %s: aNick non trouv!", parv[1], parv[0]);

	for(p = Strtok(&ptr, parv[1], ',');p;p = Strtok(&ptr, NULL, ',')) del_join(nick, p);
	return 0;
}

/*
 * m_kick SSCCC K # SSCCC :raison
 */
int m_kick(int parc, char **parv)
{
	aNick *k, *v;
	aChan *ch;
	aJoin *j = NULL;
	anAccess *a1 = NULL, *a2 = NULL;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: KICK de %s: #arg=%d", parv[0], parc);

	if(!(v = num2nickinfo(parv[2])))
		return Debug(W_DESYNCH|W_WARN, "KICK de %s sur %s par %s: cible introuvable?!", parv[0], parv[2], parv[1]);

	del_join(v, parv[1]);

	if(!(ch = getchaninfo(parv[1])) || IsSuspend(ch)) return 0;

	if(!strcmp(cs.num, parv[2])) /* on ne sait jamais */
	{
		csjoin(ch, 1);
		return Debug(W_DESYNCH|W_WARN, "Kick de %s !?", ch->chan);
	}

	if(v->user && (k = GetInfobyPrefix(parv[0])) && k != v /* gnrique */
		&& !(k->flag & (N_SERVICE|N_GOD))
		&& ((IsAdmin(v->user) && (!k->user || k->user->level < v->user->level)) /* v admin > */
			|| ((a1 = GetAccessIbyUserI(v->user, ch)) && AProtect(a1) && !ASuspend(a1)/* OU v a un accs valide*/
				 && (!k->user /* et k non logu */
					 || (!IsAdmin(k->user) /* si auth mais PAS admin */
				 		&& (!(a2 = GetAccessIbyUserI(k->user, ch)) || ASuspend(a2)/* OU n'a pas d'accs valide*/
							|| a2->level <= a1->level)))))) /* OU v protect ET k <= v */
	{
		csmode(ch, MODE_OBV, "-o $", k->numeric);/* pas le droit de kick > dop */
		if((j = GetJoinIbyC(k, ch))) DeOp(j);
	}
	return 0;
}

/*
 * m_join SSCCC J # TS
 */
int m_join(int parc, char **parv)
{
	aNick *nick = NULL;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: JOIN de %s: arg=%d", parv[0], parc);

	if(!(nick = GetInfobyPrefix(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "JOIN %s de %s: aNick non trouv!", parv[1], parv[0]);

	if(*parv[1] == '0') /* partall !!*/
	{
		aJoin *join = nick->joinhead, *jt;
		for(;join;join = jt)
		{
			jt = join->next;
			del_join(nick, join->channel);
		}
	}
	else add_join(nick, parv[1], 0, CurrentTS, getchaninfo(parv[1]));
	return 0;
}

/*
 * m_create SSCCC C # TS
 */
int m_create(int parc, char **parv)
{
	char *p, *ptr = NULL;
	aNick *nick;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: CREATE de %s: arg=%d", parv[0], parc);
	if(!(nick = num2nickinfo(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "CREATE %s de %s: aNick non trouv!", parv[1], parv[0]);

	for(p = Strtok(&ptr, parv[1], ',');p;p = Strtok(&ptr, NULL, ','))
		add_join(nick, p, J_OP, atol(parv[2]), getchaninfo(p));
	return 0;
}

/*
 * m_nick SS N nick (hop) TS ident host.com [+mode [args ...]] IPBASE64 num realname
 */
int m_nick(int parc, char **parv)
{
	char *nick = parv[1], *ac = NULL;
	aNick *who = NULL;
	anUser *user;

	if(parc > 7) /* Nouveau nick */
	{
		aServer *serv = num2servinfo(parv[0]);
						/*  nick, ident	, host	 , base64		,	num	*/
		who = add_nickinfo(nick, parv[4], parv[5], parv[parc-3], parv[parc-2],
						/* aServ*,	real-name ,	timestamp	,	umodes */
							serv, parv[parc-1], atol(parv[3]), *parv[6] == '+' ? parv[6] : NULL);
		if(!who) return 0; /* sauvetage temporaire.. */

		if(*parv[6] == '+')
		{
			int i = 7;
			if(who->flag & N_REG) ac = parv[i++];
			/*if(who->flag & N_SPOOF) spoof = parv[i++];*/
		}

		if(ac)
		{
			if(!(user = getuserinfo(ac)) || user->n)
			{
				cswallops("ACCOUNT %s (%s) pour %s depuis %s", ac,
					user ? "Desynch" : "inconnu", who->nick, serv->serv);
				cs_account(who, NULL);
				return 0;
			}
			who->user = user;
			who->user->n = who;
			who->user->lastseen = CurrentTS;
			if(IsAdmin(user)) adm_active_add(who);
		}

#ifdef USE_WELCOMESERV /* Envoi du welcome si il est actif et non nul & non burst */
		if(GetConf(CF_WELCOME) && !burst && !(serv->flag & ST_BURST)) choose_welcome(who->numeric);
#endif

	}
	else if(parc <= 3) /* Changement de pseudo */
	{
		if(!(who = num2nickinfo(parv[0])))
			return Debug(W_DESYNCH|W_WARN, "NICK %s > %s: aNick non trouv!", parv[0], nick);

#ifdef USE_NICKSERV
		if(NHasKill(who)) kill_remove(who);
#endif
		switch_nick(who, nick);
	}
	else return Debug(W_PROTO|W_WARN, "#arg invalide: NICK %s %s %s: arg=%d", parv[0], parv[1], parv[2], parc);

#ifdef USE_NICKSERV	/* le mec est auth sur ce nick OU nick pris n'est pas reg */
	if((who->user && !strcasecmp(who->user->nick, nick)) || !(user = getuserinfo(nick))) return 0;

	csntc(who, GetReply(who, L_TOOKREGNICK), nick, cs.nick, RealCmd("login"), nick);

	if(IsProtected(user))
	{
		if(UPKill(user)) csntc(who, GetReply(who, L_NICKKILLED), kill_interval);
		else csntc(who, GetReply(who, L_NICKCHANGED), kill_interval);
		add_killinfo(who, UPKill(user) ? TIMER_KREGNICK : TIMER_CHNICK);
	}
#endif
	return 0;
}

/*
 * m_burst SS B # TS [mode key lim] users %ban
 */
int m_burst(int parc, char **parv)
{
	aNick *nick = NULL;
	aLink *l = NULL;
	aChan *c = NULL;
	char *modes = NULL, *chan = parv[1], *users = NULL;
	char *flags = NULL, *num = NULL, *key = NULL, *limit = NULL;
	int status = 0, i = 4;
	long il = atol(parv[2]);

	if(parc < 4)
		return parc < 3 ? Debug(W_PROTO|W_WARN, "#arg invalide: BURST de %s: arg=%d", parv[0], parc) : 0;
		/* le parc == 3 n'est pas hors protocole -Progs
		 * > en effet, si la liste d'user est vide c'est qu'il y a des zombies -Cesar */
	if(*parv[3] == '+')
	{  /* channel has mode(s) */
		modes = parv[3];
		if(strchr(modes, 'l')) limit = parv[i++];
		if(strchr(modes, 'k')) key = parv[i++];
		users = parv[i];
	}
	else if(*parv[3] != '%') users = parv[3];
	else return 0; /* bans only */

	if(!burst)
	{
		if((c = getchaninfo(chan)) && (!c->chanstamp || il < c->chanstamp))
		{ /* notre salon est plus rcent: deopall/clearmodes */
			c->mode.modes = 0;
			c->mode.limit = 0;
			*c->mode.key = 0;
			putserv("%s " TOKEN_MODE " %s +o %s", bot.servnum, c->chan, cs.num);
			c->chanstamp = il;
			/* le pb ici c'est qu'on ne peut dterminer un user qui s'est fait deop UNIQUEMENT sur les
			chans regs, donc si un mec tait op sur un chan non reg, que le timestamp le fasse se deop,
			et qu'il veut reg le chan, il pourra sans etre op (bien qu'il l'tait tout de mme avant)
			Toute faon  part a on s'en fou vu que les chans non reg on en a rien  battre */
			for(l = c->members;l;l = l->next) l->value.j->status &= ~(J_OP | J_VOICE);
		}
		/* burst d'un salon aprs EB -> ajout de ses modes (sauf si limite >=  l'actuelle) */
		if(c) string2scmode(&c->mode, modes, key,
			limit && HasMode(c, C_MLIMIT) && atoi(limit) >= c->mode.limit ? NULL : limit);
	}
	else
	{/* envoi d'un B d'un salon reg par uplink, B-back avec defmodes, ajout des deux sries de modes */
		if((c = getchaninfo(chan)) && !IsSuspend(c) && !CJoined(c))
		{
			putserv("%s "TOKEN_BURST" %s %ld +%s %s:o", bot.servnum, c->chan, il, GetCModes(c->defmodes), cs.num);
			c->mode.modes = c->defmodes.modes;/* c'est ma 1re connexion, donc aucun mode mmoris */
			c->mode.limit = c->defmodes.limit;
			string2scmode(&c->mode, modes, key ? key : c->defmodes.key, /* ajout des modes de l'uplink */
				limit && c->mode.limit && atoi(limit) >= c->mode.limit ? NULL : limit);

			if(CLockTopic(c) && *c->deftopic) cstopic(c, c->deftopic);
			if(CFLimit(c)) floating_limit_update_timer(c);
			SetCJoined(c);
			c->chanstamp = il;
		}
		else if(!c && !strcasecmp(bot.pchan, chan)) /* salon de log */
				num2nickinfo(cs.num)->floodtime = il; /* small hack to clean up */
	}

	for(num = Strtok(&key, users, ',');num;num = Strtok(&key, NULL, ','))
	{
		if(*(flags = &num[2*NUMSERV+1]) == ':') /* SSCCC[:[o|v]] recherche directe sans while */
		{
			status = 0;
			*flags++ = '\0'; /* efface le ':' pour n'avoir que la num */
			if(*flags == 'o') status = J_OP, flags++;/* recherche directe sans while */
			if(*flags == 'v') status |= J_VOICE;
		}
		if((nick = num2nickinfo(num))) add_join(nick, chan, status | J_BURST, il, c);
		else Debug(W_DESYNCH|W_WARN, "BURST %s sur %s de %s: aNick non trouv!", num, parv[1], parv[0]);
	}

	return 0;
}

/*
 * m_quit SSCCC Q :raison
 */
int m_quit(int parc, char **parv)
{
	del_nickinfo(parv[0], "QUIT");
	return 0;
}

/*
 * m_kill SSCCC D SSCC :path (raison)
 */
int m_kill(int parc, char **parv)
{
	aNick *nick;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: KILL de %s: arg=%d", parv[0], parc);
	/* Ghost 5 (Echo envoy par le serveur de la victime lorsque Z envoie un KILL) */
	if(!(nick = num2nickinfo(parv[1])) && parv[2][strlen(cs.nick)] == ' '
		&& !strncmp(parv[2], cs.nick, strlen(cs.nick))) return 0;
	else if(!nick) return Debug(W_DESYNCH|W_WARN, "KILL de %s sur %s: aNick non trouv!", parv[0], parv[1]);

	del_nickinfo(parv[1], "KILL");
	return 0;
}

/*
 * m_away SSCCC A [msg]
 */
int m_away(int parc, char **parv)
{
    aNick *nick = GetInfobyPrefix(parv[0]);

	if(!nick) return Debug(W_DESYNCH|W_WARN, "AWAY de %s: aNick non trouv!", parv[0]);
	if(parc > 1 && *parv[1]) nick->flag |= N_AWAY;
	else
	{
		nick->flag &= ~N_AWAY;
#ifdef USE_MEMOSERV
		if(nick->user) show_notes(nick);
#endif
	}
	return 0;
}

/*
 * m_whois
 */
int m_whois(int parc, char **parv)
{
	aNick *n;
	int remote = 0;

	if(!strcasecmp(parv[2], cs.nick)) n = num2nickinfo(cs.num);
#ifdef USE_WELCOMESERV
	else if(!strcasecmp(parv[2], ws.nick)) n = num2nickinfo(ws.num);
#endif
	else n = getnickbynick(parv[2]), remote = 1;

	if(!n) return 0;

	putserv("%s 311 %s %s %s %s * :%s", bot.servnum, parv[0], n->nick, n->ident,
#ifdef HAVE_CRYPTHOST
		remote ? (n->user && n->flag & N_CRYPT) ? "hidden" : n->crypt : n->host, n->name);
#else
		(n->user && n->flag & N_CRYPT) ? "hidden" : n->host, n->name);
#endif
	putserv("%s 312 %s %s %s :%s", bot.servnum, parv[0], parv[2],
		n->serveur->serv, remote ? n->name : bot.name);
	if(IsOper(n)) putserv("%s 313 %s %s :est un IRC Oprateur %s", bot.servnum, parv[0],
						parv[2], remote ?  "tyrannique et pervers" : "- Service");
	if(!remote) putserv("%s 317 %s %s %ld %ld :Secondes d'idle, secondes d'uptime",
					bot.servnum, parv[0], parv[2], CurrentTS - bot.uptime, bot.uptime);
	putserv("%s 318 %s %s :Fin de la commande /WHOIS", bot.servnum, parv[0], parv[2]);
	return 0;
}

int m_topic(int parc, char **parv)
{
	aChan *chan;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: TOPIC de %s: arg=%d", parv[0], parc);
	if(!(chan = getchaninfo(parv[1]))) return 0; /* chan non reg */

	if(CLockTopic(chan) && !burst) /* topic was protected, enforce previous topic if any */
		cstopic(chan, *chan->topic ? chan->topic : chan->deftopic); /* or restore DefTopic */
	else Strncpy(chan->topic, parv[parc-1], TOPICLEN); /* update DB */
	return 0;
}

/*
 * m_eob SS EB SS
 */
int m_eob(int parc, char **parv)
{
	aServer *serv = num2servinfo(parv[0]);

	if (!serv || !(serv->flag & ST_BURST))
		return Debug(W_DESYNCH|W_WARN, "EOB de %s: serveur inconnu/dj synchro?", parv[0]);

	serv->flag &= ~ST_BURST;
	serv->flag |= ST_ONLINE;
	if(serv == mainhub) /* mon uplink a fini son burst, j'envoie la fin du mien! */
	{
		aChan *chan;
		int i = 0;

#ifdef USE_WELCOMESERV /* burst > 0 indique que le salon de log est vide, creons le l */
		putserv("%s "TOKEN_BURST" %s %ld +mintrs %s:ov,%s", bot.servnum, bot.pchan,
			num2nickinfo(cs.num)->floodtime, cs.num, ws.num);
#else
		putserv("%s "TOKEN_BURST" %s %ld +mintrs %s:ov", bot.servnum, bot.pchan,
			num2nickinfo(cs.num)->floodtime, cs.num);
#endif
		for(;i < CHANHASHSIZE;++i) for(chan = chan_tab[i];chan;chan = chan->next)
		{/* burst des salons enregistrs qui n'existent pas sur l'uplink */
			if(!CJoined(chan) && HasDMode(chan, C_MINV|C_MKEY) && !IsSuspend(chan))
			{
				putserv("%s "TOKEN_BURST" %s %ld +%s %s:o", bot.servnum,
					chan->chan, CurrentTS, GetCModes(chan->defmodes), cs.num);
				if(*chan->deftopic) cstopic(chan, chan->deftopic);
				SetCJoined(chan);
				if(CFLimit(chan)) floating_limit_update_timer(chan);
				chan->chanstamp = CurrentTS;
				/* le salon est vide donc appliquons QUE mes defmodes */
				memcpy(&chan->mode, &chan->defmodes, sizeof chan->defmodes);
			}/* end if */
		}/* end foreach chan */

		putserv("%s " TOKEN_EOB, bot.servnum);
		putserv("%s " TOKEN_EOBACK, bot.servnum);

		if(bot.dataQ)
		{
			struct timeval tv;
			double tt;
			gettimeofday(&tv, NULL);/*un peu de stats sur le burst pour *fun* */
			tt = (tv.tv_usec - burst) / 1000000.0;
			cswallops("Synchronisation de %lu bytes en %.6f s (Taux %.6f Ko/s)",
						bot.dataQ, tt, (bot.dataQ / 1024.0) / tt);
		}
		check_accounts();
		burst = 0;
	}
	return 0;
}

int m_server(int parc, char **parv)
{
	if(parc < 7) return Debug(W_PROTO|W_WARN, "#arg invalide: SERVER de %s: arg=%d", parv[0], parc);

	return add_server(parv[1], parv[6], parv[2], parv[5], parv[0]);
}

static int do_squit(int servnum)
{
	int i = 0;

	for(;i < MAXNUM;++i)
		if(serv_tab[i] && serv_tab[i]->hub == serv_tab[servnum])/* i est un leaf de servnum */
			do_squit(i);/* squit rcursif de ses leaf */

	for(i = 0;i <= serv_tab[servnum]->maxusers;++i)
	{
		if(num_tab[servnum][i]) /* purge des clients de 'i' */
		{
			del_nickinfo(num_tab[servnum][i]->numeric, "SQUIT");
			num_tab[servnum][i] = NULL;
		}
	}

	free(num_tab[servnum]);
	num_tab[servnum] = NULL;
	free(serv_tab[servnum]);
	serv_tab[servnum] = NULL;
	return 1;
}

int m_squit(int parc, char **parv)
{
	int i = 0;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: SQUIT de %s: arg=%d", parv[0], parc);

	for(;i < MAXNUM;++i)
		if(serv_tab[i] && !strcasecmp(serv_tab[i]->serv, parv[1]))
			return do_squit(i);

	return Debug(W_DESYNCH|W_WARN, "Squit d'un serveur inconnu %s!", parv[1]);
}

int m_pass(int parc, char **parv)
{
	struct timeval tv;

	gettimeofday(&tv, NULL);
	burst = tv.tv_usec; /* TS en s pour les stats */

	putlog(LOG_PARSES, "Connect au serveur %s", bot.ip);
	bot.lasttime = tv.tv_sec;
	bot.lastbytes = bot.dataQ;
	return 0;
}

int m_ping(int parc, char **parv)
{
	struct timeval tv;
	char *theirttm;

	if(parc < 4)
	{
		putserv("%s " TOKEN_PONG " %s", bot.servnum, parv[1]);
		return 0;
	}
	if((theirttm = strchr(parv[3], '.'))) theirttm++;
	gettimeofday(&tv, NULL);
/* AsLL */
	putserv("%s " TOKEN_PONG " %s %s %s %ld %ld.%ld", bot.servnum, bot.servnum,
			parv[0], parv[3], ((tv.tv_sec - atoi(parv[3])) * 1000 + (tv.tv_usec - atoi(theirttm)) / 1000),
			tv.tv_sec, tv.tv_usec);/* mon TS seconde.micro */
    return 0;
}

int m_error(int parc, char **parv)
{
	return Debug(W_MAX|W_TTY, "Erreur reue du Hub (%s)", parv[1]);
}

static int exec_cmd(aHashCmd *cmd, unsigned int secure, aNick *nick, int parc, char **parv)
{
	char *tmp3 = NULL, buff[512];
	aChan *chan = NULL;
	anAccess *acces = NULL;
#ifdef HAVE_TRACK
	struct track *track = NULL;
#endif

	if(isignore(nick) || checkflood(nick)) return 0;

#ifdef HAVE_SECURE
	if(SecureCmd(cmd) && !secure) return csntc(nick, GetReply(nick, L_SECURECMD),
		cmd->name, cs.nick,	cmd->name, parv2msg(parc, parv, 1, 400));
#endif

	cmd->used++;
	if(cmd->flag & CMD_DISABLE)
		return csntc(nick, GetReply(nick, L_CMDDISABLE));

	if(parc < cmd->args || (!parc && ChanCmd(cmd)))	return syntax_cmd(nick, cmd);

	if(ChanCmd(cmd))
	{
		if(*parv[1] != '#' || !(chan = getchaninfo(parv[1])))
			return csntc(nick, GetReply(nick, L_NOSUCHCHAN), parv[1]);

		if(IsSuspend(chan) && cmd->level)
			return csntc(nick, "%s est suspendu : %s - %s", parv[1],
					chan->suspend->raison, chan->suspend->from);
	}
	if(cmd->level)
	{
		if(!nick->user || !nick->user->n) return csntc(nick, "%s", pasdeperm);

		if(AdmCmd(cmd) && nick->user->level < cmd->level)
			return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

		if(ChanCmd(cmd) && !IsAdmin(nick->user))
		{
			if(!(acces = GetAccessIbyUserI(nick->user, chan)))
				return csntc(nick, GetReply(nick, L_YOUNOACCESSON), parv[1]);

			if(cmd->level > acces->level)
				return csntc(nick, GetReply(nick, L_ACCESSTOOLOW), parv[1], acces->level, cmd->level);

			if(ASuspend(acces))
				return csntc(nick, GetReply(nick, L_ACCESSSUSPENDED), parv[1]);
		}
	}

	if(!ChanCmd(cmd) && parc && (SecureCmd(cmd) || (parc >= 2
		&& ((!strcasecmp(cmd->corename, "memos") && !strcasecmp(parv[1], "send"))
		|| (!strcasecmp(cmd->corename, "set") && !strcasecmp(parv[1], "pass"))
		|| (!strcasecmp(cmd->corename, "user") && !strcasecmp(parv[1], "newpass"))))))
		tmp3 = parv[1], tmp3[400] = 0; /* truncate the buffer otherwise it could overflow 'buff' */
	else if((ChanCmd(cmd) || !Secure2Cmd(cmd)) && parc)
		tmp3 = parv2msg(parc, parv, 1, 400);

	if(nick->user) fastfmt(buff, "\2$\2 $ par $@$", cmd->name, tmp3, nick->nick, nick->user->nick);
	else fastfmt(buff, "\2$\2 $ par $", cmd->name, tmp3, nick->nick);

#ifdef HAVE_TRACK
	if(nick->user && (track = istrack(nick->user))) csntc(track->tracker, "[\2Track\2] %s", buff);
#endif
	putchan(buff);

#ifdef SQLLOG
	if(cmd->level)
	{
		if(ChanCmd(cmd))
		{
			if(tmp3) tmp3[254] = 0;
			sql_query(SQL_QINSERTC, "('%s', '%s', %T, '%s', '%s')", nick->user->nick,
				chan->chan, CurrentTS, cmd->corename, tmp3);
		}
		else sql_query(SQL_QINSERTU, "('%s', %T, '%s', '%s')", nick->user->nick, CurrentTS,
				cmd->corename, NONE(tmp3));
	}
#else
	putlog(LOG_CMDS, "%s", buff);
#endif
	cmd->func(nick, chan, parc, parv);/* passage du aChan * en plus, pour conomie */
	return 1;
}

/*
 * m_privmsg SSCCC P SSCCC|# :msg
 * pparv[0] = nom de la commande
 * pparv[pparc] = arguments de la commande
 * ATTENTION: pparc est le nb d'arg  partir de *parv[1]*
 */
int m_privmsg(int parc, char **parv)
{
	char *tmp, *pparv[250], *ptr = NULL;
	unsigned int pparc = 0, securise = 0;
	aHashCmd *cmd;
	aNick *nptr = GetInfobyPrefix(parv[0]);

	if(!nptr) return 0;/* la source du message n'est pas un user connu (serveur/desynch)*/

	if(*parv[1] != '#') /* Message priv */
	{
		pparv[1] = NULL;

		if((tmp = strchr(parv[1], '@'))) /* P nick@server */
		{
			*tmp = 0;
			securise = 1;
		}

		if(!*parv[2] || (strcasecmp(parv[1], cs.nick) && strcmp(parv[1], cs.num))) return 0;

		pparv[0] = (tmp = Strtok(&ptr, parv[2], ' '));
		if(!tmp) return 0;

		if(!(cmd = FindCommand(tmp)))
		{
			if(*tmp != '\1') csntc(nptr, GetReply(nptr, L_NOSUCHCMD), tmp);
			return 0;
		}

		while((tmp = Strtok(&ptr, NULL, ' '))) pparv[++pparc] = tmp;

		exec_cmd(cmd, securise, nptr, pparc, pparv);
	}
	else /* Message envoy sur un salon, recherche d'une commande (quel flood..) */
	{
		if(parv[2] && *parv[2] == bot.cara && *++parv[2] && *parv[2] != ' ')
		{
			pparv[0] = (tmp = Strtok(&ptr, parv[2], ' '));

			if(!tmp || !(cmd = FindCommand(tmp))) return 0; /* pas d'arg ou commande inconnue */

			tmp = Strtok(&ptr, NULL, ' '); /* on traite le premier arg aprs la commande */
			if(ChanCmd(cmd) && (!tmp || *tmp != '#')) pparv[++pparc] = parv[1]; /* si ce n'est pas un salon */
			if(tmp) pparv[++pparc] = tmp;									/* on prend le salon o elle est tape */

			while((tmp = Strtok(&ptr, NULL, ' '))) pparv[++pparc] = tmp;

			exec_cmd(cmd, 0, nptr, pparc, pparv);
		}
	}
	return 0;
}
