/* src/sql_log.c - sql log
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: sql_log.c,v 1.20 2005/04/22 22:29:44 romexzf Exp $
 */

#include "main.h"
#ifdef SQLLOG
#include "sql_log.h"
#include "debug.h"
#include "cs_cmds.h"
#include "timers.h"
#include <mysql.h>
#include <errmsg.h>

static char sql_buf[2][2048];
static unsigned int sql_count[2];
static time_t SQL_LastFlush[2];

#define SQL_WAIT_FLUSH 1200 /* flush sql */
#define SQLDequeue() (type == SQL_QRAW || sql_count[type] > sizeof sql_buf[type] - 800)

static int sql_send(const char *buf)
{
 	if(mysql_query(bot.sql_id, buf))
 	{	/* erreur lors de la requete */
		if(mysql_errno(bot.sql_id) == CR_SERVER_LOST) /* ok ce n'est qu'une dco */
		{
			if(!sql_init(0)) /* essayons une reconnexion */
				Debug(W_WARN, "DB SQL inaccessible -- nouvel essai lors du prochain query.");
			else
			{
				cswallops("DB SQL perdue -- reconnexion OK -- envoi du query.");
				mysql_query(bot.sql_id, buf);
				return 1;
			}
		}
		else Debug(W_MAX|W_WARN, "SQL Server reports: %s", mysql_error(bot.sql_id));
		Debug(W_WARN|W_MAX, "sql_query failed (%s)", buf);
		return -1;
	}
	return 1;
}

int sql_flush(enum SQL_QType type)
{
	SQL_LastFlush[type] = CurrentTS;
	if(sql_count[type]) sql_send(sql_buf[type]), sql_count[type] = 0;
	return 0;
}

int sql_query(enum SQL_QType type, const char *pattern, ...)
{
	static const char table_dec[] = "0123456789";
	static char raw[1100], t;
 	register char *p;
 	va_list vl;

	if(type != SQL_QRAW)
	{
		if(!sql_count[type]) /* empty query */
		{
			strcpy(sql_buf[type], type == SQL_QINSERTU ?
				 "INSERT INTO "SQLUSER" VALUES " : "INSERT INTO "SQLCHAN" VALUES ");
			sql_count[type] = (SQL_QINSERTU ? sizeof("INSERT INTO "SQLUSER" VALUES ") :
				 sizeof("INSERT INTO "SQLCHAN" VALUES ")) -1;
		}
		else sql_buf[type][sql_count[type]++] = ',';
		p = &sql_buf[type][sql_count[type]];
	}
	else p = raw;

	va_start(vl, pattern);
	while((t = *pattern++)) /* %sa (t = %, *pattern = s) */
 	{
 		if(t == '%')
 		{
 			t = *pattern++; /* on drop le formateur (t = s, *pattern = a) */
 			if(t == 's')
 			{
 				register char *tmps = va_arg(vl, char *);
 				if(tmps)
 					while(*tmps) /* copie de la string */
 					{
 						if(*tmps == '\\' || *tmps == '\'' || *tmps == '"' || *tmps == '%')
 							*p++ = '\\'; /* escape it */
 						*p++ = *tmps++;
 					}
 				continue;
 			}
 			if(t == 'c')
 			{
 				*p++ = (char) va_arg(vl, int);
 				continue;
 			}
 			if(t == 'T')
 			{
 				unsigned long tmpi = va_arg(vl, unsigned long);
 				int pos = 31;
 				char bufi[32];

 				if(tmpi == 0)
 				{
 					*p++ = '0';
 					continue;
 				}
 				while(tmpi)	/* on converti une int en base 10 en string */
 				{			/* criture dans l'ordre inverse 51 > '   1' > '  51'  */
 					bufi[pos--] = table_dec[tmpi % 10];
 					tmpi /= 10;
 				}
 				while(pos < 31) *p++ = bufi[++pos];
 				continue;
			}
 			if(t != '%')
 			{	/* on sous traite (depuis le dbut..)  vsnprintf */
 				p += vsprintf(p, pattern - 2, vl); /* on remet le %format */
 				break;
 			}
 		}
 		*p++ = t;
	}
 	*p = '\0';
	va_end(vl);

	if(type == SQL_QRAW) return sql_send(raw);
	else
	{	/* update current length via pointer arithemtic */
		sql_count[type] += (p - &sql_buf[type][sql_count[type]]);
		if(SQLDequeue()) sql_flush(type);
	}
	return 1;
}

static int callback_flush_sql(Timer *timer)
{
	if(CurrentTS - SQL_LastFlush[SQL_QINSERTU] >= SQL_WAIT_FLUSH)
		sql_flush(SQL_QINSERTU);
	if(CurrentTS - SQL_LastFlush[SQL_QINSERTC] >= SQL_WAIT_FLUSH)
		sql_flush(SQL_QINSERTC);

	timer_enqueue(timer);
	return 0;
}

int sql_init(int init)
{
	MYSQL_RES *result = NULL;
	int culog = 1, cclog = 1;

 	bot.sql_id = mysql_init(NULL);
 	if(!mysql_real_connect(bot.sql_id, bot.sql_host, bot.sql_user, bot.sql_pass,
 		bot.sql_db, bot.sql_port, NULL, 0))	return 0;

	if(!init) return 1; /* pas d'initialisation  faire */

 	mysql_query(bot.sql_id, "SHOW TABLES"); /* on vrifie si les tables existent */
 	if((result = mysql_store_result(bot.sql_id)))
 	{
 		MYSQL_ROW row;

 		while((row = mysql_fetch_row(result)))
 		{
 			mysql_field_seek(result, 0);
 			if(!strcmp(row[0], SQLUSER)) culog = 0;
 			else if(!strcmp(row[0], SQLCHAN)) cclog = 0;
 		}
 		mysql_free_result(result);
	}
	/* cration des tables .. */
 	if(culog && sql_query(SQL_QRAW, "CREATE TABLE "SQLUSER" (user CHAR(%d), TS INT(4), "
 		"cmd CHAR(%d), log CHAR(255))", NICKLEN, CMDLEN) < 0) return 0;
 	if(cclog && sql_query(SQL_QRAW, "CREATE TABLE "SQLCHAN" (user CHAR(%d), chan CHAR(%d), "
 		"TS INT(4), cmd CHAR(%d), log CHAR(255))", NICKLEN, REGCHANLEN, CMDLEN) < 0)
 			return 0;

 	timer_add(SQL_WAIT_FLUSH, TIMER_PERIODIC, callback_flush_sql, NULL, NULL);
	return 1;
}
#endif
