/* src/templace.c - Template
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: template.c,v 1.2 2005/01/03 19:34:55 romexzf Exp $
 */

#include "main.h"
#include "lang.h"
#include "debug.h"
#include "config.h"
#include <errno.h>

#define TMPLBUFSIZE 512

struct Template {
	char **buf;
	int count;
} tmpl_mail_register, tmpl_mail_oubli;

static int tmpl_realloc(struct Template *tpl, size_t size)
{
	void *temp;
	if(!size)
	{
		temp = realloc(tpl->buf, ++tpl->count * sizeof *tpl->buf);
		if(!temp) return Debug(W_MAX, "tmpl::realloc OOM!");
		tpl->buf = temp;
		tpl->buf[tpl->count-1] = NULL;
	}
	temp = realloc(tpl->buf[tpl->count-1], size ? size : TMPLBUFSIZE);
	if(!temp) return Debug(W_MAX, "tmpl::realloc OOM!");
	tpl->buf[tpl->count-1] = temp;
	return 1;
}

static int tmpl_load_addstr(struct Template *tpl, const char *str, int used, size_t size)
{
	if(!size) return 0;
	if(size + used >= TMPLBUFSIZE) tmpl_realloc(tpl, used + size);

	strcpy(tpl->buf[tpl->count -1] + used, str);
	return size + used >= TMPLBUFSIZE ? 0 : used + size;
}

static void tmpl_load_atmpl(struct Template *tpl, FILE *f)
{
	int i = 0, used = 0;
	char buf[512];

	if(tpl->count) /* clean up */
	{
		for(;i < tpl->count;++i) free(tpl->buf[i]);
		free(tpl->buf), tpl->count = 0, tpl->buf = NULL;
	}

	while(fgets(buf, sizeof buf, f))
	{
		if(!used) tmpl_realloc(tpl, 0);
		used = tmpl_load_addstr(tpl, buf, used, strlen(buf));
	}
}

int tmpl_load(void)
{
	FILE *f = fopen(LANG_PATH "/mailregister.tmpl", "r");

	if(!f) return 0;
	tmpl_load_atmpl(&tmpl_mail_register, f);
	fclose(f);

	if(!(f =  fopen(LANG_PATH "/mailoubli.tmpl", "r"))) return 0;
	tmpl_load_atmpl(&tmpl_mail_oubli, f);
	fclose(f);
	return 1;
}

int tmpl_mailsend(struct Template *tpl, const char *mail, const char *user,
	const char *pass, const char *host)
{
	FILE *fm;
	int i = 0, size = 0, sizes = 0;
	char buf[2 * TMPLBUFSIZE];

	if(!tpl->count) return Debug(W_WARN, "No template loaded for mailing!");
	if(!(fm = popen(mailprog, "w")))
		return Debug(W_WARN, "popen() failed for %s [%s]", mail, strerror(errno));

	for(;i < tpl->count;++i)
	{
		const char *p = tpl->buf[i];

#define AddStr(s) do { \
	sizes = strlen(s); \
	if(size + sizes >= sizeof buf -1) buf[size] = 0, fputs(buf, fm), size = 0; \
	strcpy(buf + size, s); \
	size += sizes; \
} while(0)

		while(*p)
			if(*p == '$')
			{
				if(*++p == 'u') AddStr(user);
				else if(*p == 'e') AddStr(mail);
				else if(*p == 'p') AddStr(pass);
				else if(*p == 'h') AddStr(host);
				else --p; /* go back */
				++p;
			}
			else
			{
				if(size >= sizeof buf -1) buf[size] = 0, fputs(buf, fm), size = 0;
				buf[size++] = *p++;
			}
	}
	buf[size] = 0;
	fputs(buf, fm); /* dump line */

	pclose(fm);
	return 1;
}
