/* src/timers.c - fonctions de controle appelles periodiquement
 * Copyright (C) 2002-2005 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: timers.c,v 1.59 2005/04/23 22:42:11 romexzf Exp $
 */

#include "main.h"
#include "config.h"
#include "outils.h"
#include "hash.h"
#include "del_info.h"
#include "fichiers.h"
#include "cs_cmds.h"
#include "debug.h"
#include "timers.h"

void check_accounts(void)
{
	anUser *u, *u2;
	int i = 0;

	for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u2)
	{
		u2 = u->next;
		if((u->lastseen + maxlastseen) < CurrentTS && !IsAdmin(u) && !UNopurge(u))
			cswallops("Purge de l'username %s (Lastseen %s)",
				u->nick, duration(CurrentTS - u->lastseen));
		else if(UFirst(u) && (u->lastseen + REGISTER_TIMEOUT) < CurrentTS)
			cswallops("Purge de l'username %s par register timeout", u->nick);
		else
		{
			if(u->n) u->lastseen = CurrentTS;
			if(u->cantregchan > 0 && u->cantregchan < CurrentTS) u->cantregchan = -1;
			continue; /* user is ok, go on without purging it. */
		}
		del_regnick(u, HF_SQLLOG, "Purge de l'username de l'owner");
	}
}

void check_chans(void)
{
	aChan *c, *next;
	int i = 0;
	char raison[300];

	for(;i < CHANHASHSIZE;++i) for(c = chan_tab[i];c;c = next)
	{
		next = c->next;

		if(!c->access || !c->owner)
			Debug(W_WARN, "chan_check: Channel %s has no %s -- Why ?!",
				c->chan, !c->owner ? "owner" : "access");
		else if(c->owner->lastseen > 1 && c->owner->lastseen + maxlastseen < CurrentTS
			&& !UNopurge(c->owner->user))
		{
			snprintf(raison, sizeof raison, "Pas de visite de l'owner depuis \2%s",
				get_time(NULL, c->owner->lastseen));
			del_chan(c, HF_SQLLOG, raison);
		}
		else if(!IsSuspend(c) && CJoined(c) && c->users == 1
		&& !HasMode(c, C_MKEY|C_MINV) && CurrentTS - c->lastact > chanmaxidle)
			cspart(c, "Idle");
	} /* for() */
}

#ifdef TDEBUG
#	define TDEBUGF(x) timer_debug x
void timer_debug(const char * fmt, ...)
{
	va_list vl;

	va_start(vl, fmt);
	fputs(get_time(NULL, CurrentTS), stdout); /* TimeStamp */
	fputc(' ', stdout);
	vprintf(fmt, vl);
	fputc('\n', stdout);
	va_end(vl);
}
#else
#	define TDEBUGF(x)
#endif

Timer *timer_add(time_t delay, enum TimerType type, int (*callback) (Timer *),
				void *data1, void *data2)
{
	Timer *timer = calloc(1, sizeof *timer);

	TDEBUGF(("Adding timer %d (delay=%ld) d1=[%s] d2=[%s]", type, delay, data1, data2));

	if(!timer)
	{
		Debug(W_MAX|W_WARN, "add_timer, malloc a chou pour un timer (%lu type %d)",
			delay, type);
		return NULL;
	}

	timer->delay = delay;
	timer->callback = callback;
	timer->type = type;
	timer->data1 = data1;
	timer->data2 = data2;

	return (timer = timer_enqueue(timer));
}

Timer *timer_enqueue(Timer *timer)
{
	Timer *tmp = Timers;

	switch(timer->type)
	{
		case TIMER_ABSOLU:
			timer->expire = timer->delay;
			break;
		case TIMER_RELATIF:
		case TIMER_PERIODIC: /* reenqueue it */
			timer->expire = CurrentTS + timer->delay;
			break;
	}

	if(timer->expire < CurrentTS)
	{
		Debug(W_DESYNCH|W_WARN, "timer_enqueue, ajout d'un Timer dj expir? (%lu < %lu)",
			(unsigned long) timer->expire, (unsigned long) CurrentTS);
		free(timer);
		return NULL;
	}

	if(!tmp || timer->expire <= tmp->expire)
	{
		timer->next = tmp;
		timer->last = NULL;
		if(tmp) tmp->last = timer;
		Timers = timer;
	}
	else
	{
		for(;tmp->next && timer->expire >= tmp->expire;tmp = tmp->next);

		timer->next = tmp;
		timer->last = tmp->last;
		if(tmp->last) tmp->last->next = timer;
		tmp->last = timer;
	}

	TDEBUGF(("Queueing Timer %p, (delay=%ld/%ld exp: %ld) ->last=%p ->next=%p",
		(void *) timer, timer->delay, timer->expire, timer->expire - CurrentTS,
		(void *) timer->last, (void *) timer->next));

	return timer;
}

void timer_free(Timer *timer)
{
	TDEBUGF(("Freeing Timer %p, ->last=%p ->next=%p", (void *) timer,
		(void *) timer->last, (void *) timer->next));
	free(timer);
}

void timer_dequeue(Timer *timer)
{
	TDEBUGF(("Dequeueing Timer %p, ->last=%p ->next=%p", (void *) timer,
		(void *) timer->last, (void *) timer->next));

	if(timer->next) timer->next->last = timer->last;
	if(timer->last) timer->last->next = timer->next;
	else Timers = timer->next;

	timer->next = timer->last = NULL; /* removed from list */
}

void timer_remove(Timer *timer)
{
	timer_dequeue(timer);
	timer_free(timer);
}

void timers_run(void)
{
	Timer *timer = Timers, *tmp;

	TDEBUGF(("Running Timers ! ! !"));

	for(;timer && timer->expire <= CurrentTS;timer = tmp)
	{
		tmp = timer->next;

		TDEBUGF(("Running Timer %p, ->last=%p ->next=%p (%ld)", (void *) timer,
			(void *) timer->last, (void *) timer->next, timer->expire));

		timer_dequeue(timer);
		if(timer->callback(timer)) timer_free(timer);
	}
}

int callback_ban(Timer *timer)
{
	aBan *ban = timer->data2;
	aChan *chan = timer->data1;

	ban->timer = NULL; /* prevent del_ban from clearing it too */

	csmode(chan, MODE_OBV, "-b $", ban->mask);
	del_ban(chan, ban);
	return 1;
}

#ifdef USE_NICKSERV
int callback_kill(Timer *timer)
{
	aKill *kill = timer->data1;

	kill->nick->timer = NULL;
	if(TIMER_CHNICK == kill->type)
	{	/* generates a new nick based on current one and appends */
		char buf[NICKLEN + 1];
		do { /* a number of at max 4 digits */
			snprintf(buf, sizeof buf, "%.20s%d", kill->nick->nick, rand() & 4095);
		} while(getnickbynick(buf));

		putserv("%s "TOKEN_SVSNICK" %s :%s", bot.servnum, kill->nick->numeric, buf);
	}
	else
	{
		putserv("%s "TOKEN_KILL" %s :%s (Pseudo enregistr/Registered Nick)",
			cs.num,	kill->nick->numeric, cs.nick);
		del_nickinfo(kill->nick->numeric, "timer kill");
	}
	kill_free(kill);
	return 1;
}
#endif

int callback_check_accounts(Timer *timer)
{
	check_accounts();
	timer_enqueue(timer);
	return 0;
}

int callback_check_chans(Timer *timer)
{
	check_chans();
	timer_enqueue(timer);
	return 0;
}

int callback_write_dbs(Timer *timer)
{
	db_write_chans();
	db_write_users();
	timer_enqueue(timer);
	return 0;
}

int callback_unsuspend(Timer *timer)
{
	struct suspendinfo **suspend = timer->data1;

	(*suspend)->expire = -1;
	if((*suspend)->data)
	{
		aChan *chan = (*suspend)->data;
		if(!CJoined(chan)) csjoin(chan, 0);
		putlog(LOG_CHANS, "SUSPEND OFF (Auto) %s par %s", chan->chan, chan->suspend->from);
	}
	return 1;
}

int callback_fl_update(Timer *timer)
{
	aChan *c = timer->data1;

	if(CurrentTS - bot.lasttime > 30) /* ugly! */
	{
		bot.lasttime = CurrentTS;	/* mise  jour des infos  */
		bot.lastbytes = bot.dataQ;	/* pour les stats traffic */
	}

	if(CFLimit(c) && CJoined(c)) /* floating limit */
	{
		int newlimit = c->users + c->limit_inc, dif = newlimit - c->mode.limit;
		if(dif < 0) dif = -dif;
		/* only updates the limit if it is worst (dif >= grace) */
		if(c->mode.limit != newlimit && dif >= c->limit_min)
		{
			c->mode.limit = newlimit;
			if(!HasMode(c, C_MLIMIT)) c->mode.modes |= C_MLIMIT;
			putserv("%s "TOKEN_MODE" %s +l %d", cs.num, c->chan, newlimit);
		}
	}
	else /* not on chan or FL not active => remove timer */
	{
		c->timer = NULL;
		return 1;
	}
	timer_enqueue(timer);
	return 0;
}


