/* src/track.c - Tracker un user
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: track.c,v 1.22 2005/05/05 21:49:18 romexzf Exp $
 */

#include "main.h"
#ifdef HAVE_TRACK
#include "track.h"
#include "cs_cmds.h"
#include "hash.h"
#include "outils.h"
#include "debug.h"

struct track *trackhead = NULL;

struct track *istrack(anUser *user)
{
	register struct track *track = trackhead;
	for(;track && track->tracked != user;track = track->next);
	return track;
}

static int add_track(anUser *nick, aNick *from, time_t expire)
{
	struct track *track = malloc(sizeof *track);

	if(!track)
		return Debug(W_WARN|W_MAX, "add_track, malloc a chou pour le track %s (%s)",
			nick->nick, from->nick);
	track->tracked = nick;
	track->tracker = from;
	track->expire = expire;
	track->from = CurrentTS;
	track->next = trackhead;
	trackhead = track;
	return 1;
}

void del_track(anUser *nick)
{
	struct track *tmp = trackhead, *last = NULL;

	for(;tmp && tmp->tracked != nick;last = tmp, tmp = tmp->next);
	if(tmp)
	{
		if(last) last->next = tmp->next;
		else trackhead = tmp->next;
		free(tmp);
	}
}

int cmd_track(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	struct track *track = trackhead;

	if(!strcasecmp(parv[1], "list"))
	{
		unsigned int i = (parc < 2) ? 1 : 0;

		if(!track) return csntc(nick, "Aucun track actuellement.");

		csntc(nick, "\2Liste des Traqus:\2");
		for(;track;track = track->next)
			if(i || !match(parv[2], track->tracked->nick))
				csntc(nick, "> \2%s\2 (Pour \2%s\2, depuis %s, expire %s)",
				track->tracked->nick, track->tracker->nick, duration(CurrentTS - track->from),
				!track->expire ? "Jamais" : duration(CurrentTS - track->expire));
	}

	else if(!strcasecmp(parv[1], "add"))
	{
		anUser *user = NULL;
		time_t timeout = 0;

		if(parc < 2) return csntc(nick, "Syntaxe: %s ADD <username|%%nick> [expiration]", parv[0]);

		if(!(user = ParseNickOrUser(nick, parv[2]))) return 0;

		if((track = istrack(user)))
			return csntc(nick, "\2%s\2 est dj track par %s (depuis %s)",
				track->tracked->nick, track->tracker->nick, duration(CurrentTS - track->from));

		if(nick->user == user) return csntc(nick, "Pourquoi se traquer soi-mme!?");

		if(IsAdmin(user) && nick->user->level < MAXADMLVL)
			return csntc(nick, "Veuillez ne pas tracker un Admin.");

		if(parc > 2 && *parv[3] == '%' && (timeout = convert_duration(++parv[3])) < 0)
			return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

		add_track(user, nick, timeout);

		csntc(nick, "\2%s\2 a t ajout a vos tracks. (Expire dans %s)", user->nick,
			timeout ? duration(timeout) : "Jamais");
		if(user->n) csntc(nick, "%s est connect via %s", user->nick, GetNUHbyNick(user->n, 0));
	}

	else if(!strcasecmp(parv[1], "del"))
	{
		anUser *user = NULL;

		if(parc < 2) return csntc(nick, "Syntaxe: %s DEL <username>", parv[0]);

		if(!(user = getuserinfo(parv[2])))
			return csntc(nick,  GetReply(nick, L_NOSUCHUSER), parv[2]);

		if(!(track = istrack(user)))
			return csntc(nick, "\2%s\2 n'est pas track.", parv[2]);

		if(track->tracker != nick)
			return csntc(nick, "\2%s\2 est actuellement track par %s.",
						track->tracked->nick, track->tracker->nick);

		del_track(track->tracked);
		csntc(nick, "%s a t supprim de vos tracks.", parv[2]);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), parv[1]);
	return 1;
}
#endif
