/* src/vote.c - Effectuer un vote
 * Copyright (C) 2002-2005 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: vote.c,v 1.23 2005/04/24 16:26:07 romexzf Exp $
 */

#include "main.h"
#ifdef HAVE_VOTE
#include "vote.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "hash.h"
#include "outils.h"

struct votes vote[MAXVOTEPROP + 1];
struct Vote Vote = {NULL, 0, 0, 0};

static int *sort_vote(int *tmp)
{
	int i = 0, isgood = 0;
	static int intbuf[MAXVOTEPROP];

	for(;i< Vote.nbprop;++i) intbuf[i] = i;
	while(!isgood)
	{
		isgood = 1;
		for(i = 0;i < Vote.nbprop;++i)
		{
			if(tmp[i] < tmp[i+1])
			{
				int j = tmp[i+1];
				tmp[i+1] = tmp[i];
				tmp[i] = j;
				j = intbuf[i+1];
				intbuf[i+1] = intbuf[i];
				intbuf[i] = j;
				isgood = 0;
			}
		}
	}
	return intbuf;
}

void show_vote(aNick *nick)
{
	int i = 0;
	csntc(nick, "\2Sujet:\2 %s", vote[Vote.nbprop].prop);
	csntc(nick, "\2Propositions:\2");
	for(;i < Vote.nbprop;++i) csntc(nick, "%d. %s", i + 1, vote[i].prop);
}

/*
 * vote_results
 */
int vote_results(aNick *nick, aChan *c, int parc, char **parv)
{
	float f = 0;
	int i = 0, tmp[MAXVOTEPROP];

	if(Vote.actif)
		return csntc(nick, "Vous ne pouvez pas voir les rsultats, le vote est encore en cours.");

	if(!Vote.nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

	if(nick->user->level < Vote.level)
		return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

	csntc(nick, "Resultats du vote\2 %s\2", vote[Vote.nbprop].prop);

	for(;i < Vote.nbprop;++i)
	{
		f += vote[i].voies;
		tmp[i] = vote[i].voies;
	}

	if(!Vote.result) Vote.result = sort_vote(tmp);
	for(i = 0;i < Vote.nbprop;++i)
		csntc(nick, "%d. %.2f%% [%d] \2%s", i+1, (vote[Vote.result[i]].voies/f) * 100,
			vote[Vote.result[i]].voies, vote[Vote.result[i]].prop);
	return 1;
}

/*
 * voter parv[1] = n de la proposition
 */
int voter(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int nb_prop = 0;

	if(nick->user->level < Vote.level)
		return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

	if(!Vote.actif)
	{
		if(!Vote.nbprop) return csntc(nick, "Le vote n'est pas actif");
		else return csntc(nick, "Le vote est clos. Pour voir les rsultats, tapez /%s %s",
						cs.nick, RealCmd("results"));
	}

	if(!parc)
	{
		csntc(nick, "Un vote est actuellement en cours.");
		show_vote(nick);
		if(nick->user && !nick->user->vote)
			csntc(nick, "Pour voter utilisez: \2/%s %s <n de la proposition>\2", cs.nick, parv[0]);
		return 1;
	}

	if(nick->user->vote == -1)
		return csntc(nick, "Vous avez enregistr votre username aprs l'ouverture du vote."
			"Vous ne pourrez voter qu'au prochain vote");

	if(nick->user->vote) return csntc(nick, "Vous avez dj vot.");

	if(!is_num(parv[1])) return csntc(nick, "Le n de la proposition doit tre un nombre.");

	if((nb_prop = atoi(parv[1])) < 1 || nb_prop > Vote.nbprop)
		return csntc(nick, "Les propositions vont de 1  %d", Vote.nbprop);

	nick->user->vote = nb_prop--;
	vote[nb_prop].voies++;
	csntc(nick, "Votre vote pour \2%s\2 a bien t pris en compte.", vote[nb_prop].prop);
	write_votes(); /* to avoid multiple votes in case of crash */
	return 1;
}

/*
 * vote NEW <nombre de prop> <level> <sujet>
 * 		PROPOSITION <proposition>
 * 		CLORE
 * 		RESULTS
 */

int do_vote(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "NEW"))
	{
		anUser *u;
		char *msg = NULL;
		int i = 0;

		if(parc < 4)
			return csntc(nick, "Syntaxe: %s NEW <nombre de propositions> <level> :<sujet>", parv[0]);

		if(Vote.actif) return csntc(nick, "Le vote est dj actif");

		if(!is_num(parv[2]))
			return csntc(nick, "Le nombre de propositions doit tre une valeur numrique");

		if((Vote.nbprop = atoi(parv[2])) < 2 || Vote.nbprop > MAXVOTEPROP)
			return csntc(nick, "Il peut y avoir de 2  %d propositions possibles.", MAXVOTEPROP);

		if(!is_num(parv[3]) || (Vote.level = atoi(parv[3])) > MAXADMLVL || Vote.level < 1)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		msg = parv2msg(parc, parv, 4, MEMOLEN + 3);
		if(*msg == ':') msg++;
		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur du sujet du vote est limite  %d caractres", MEMOLEN);

		memset(vote, 0, sizeof vote);
		Strncpy(vote[Vote.nbprop].prop, msg, MEMOLEN);
		for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next) u->vote = 0;

		csntc(nick, "Le vote est en cours de cration au level %d.", Vote.level);
		csntc(nick, "\2Sujet:\2 %s", vote[Vote.nbprop].prop);
		csntc(nick, "Vous devez maintenant dfinir les propositions avec: "
				"\2/%s %s PROPOSITION :<nom>\2 jusqu' que vous ayez donn les %d propositions",
				cs.nick, parv[0], Vote.nbprop);
		Vote.result = NULL;
	}
	else if(!strcasecmp(cmd, "PROPOSITION"))
	{
		int i = 0;
		char *msg = NULL;

		if(parc < 2) return csntc(nick, "Syntaxe: %s PROPOSITION :<proposition>", parv[0]);

		if(Vote.actif) return csntc(nick, "Le vote est dj actif");

		if(!Vote.nbprop) return csntc(nick, "Aucun vote n'est en cours d'enregistrement");

		for(;*vote[i].prop && i < Vote.nbprop;++i);
		if(i >= Vote.nbprop)
			return csntc(nick, "Vous avez bien donn toutes les propositions pour le vote");

		msg = parv2msg(parc, parv, 2, MEMOLEN + 3);
		if(*msg == ':') msg++;
		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur d'une proposition est limite  %d caractres", MEMOLEN);

		Strncpy(vote[i].prop, msg, MEMOLEN);
		vote[i].voies = 0;

		if(i == Vote.nbprop-1)
		{
			Vote.actif = 1;
			csntc(nick, "Le vote est maintenant \2ACTIF\2.");
			show_vote(nick);
			write_votes();
		}
		else csntc(nick, "Vous avez donn %d des %d propositions.", i + 1, Vote.nbprop);
	}
	else if(!strcasecmp(cmd, "CLORE"))
	{
		if(!Vote.actif) return csntc(nick, "Il n'y a pas de vote actif.");

		Vote.actif = 0;
		csntc(nick, "Le vote est clos, les users peuvent maintenant voir les rsultats.");
		write_votes();
	}
	else if(!strcasecmp(cmd, "RESULTS"))
	{
		int i = 0, tmp[MAXVOTEPROP], usr = 0, *tmpp;
		anUser *u;
		float f = 0;

		if(!Vote.nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

		csntc(nick, "Rsultats:");

		for(;i < Vote.nbprop;++i)
		{
			f += vote[i].voies;
			tmp[i] = vote[i].voies;
		}

		tmpp = sort_vote(tmp);

		for(i = 0;i < Vote.nbprop;++i) csntc(nick, "%d. %.2f%% [%d] \2%s", i + 1,
			(vote[tmpp[i]].voies / f) * 100, vote[tmpp[i]].voies, vote[tmpp[i]].prop);

		for(i = 0;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next)
			if(u->level >= Vote.level) ++usr;

		csntc(nick, "%d des %d users ont vot (%.2f%%)", (int)f, usr, (f / usr) * 100);
	}
	else csntc(nick, "La sous commande %s n'existe pas", cmd);
	return 1;
}

void write_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "w");
   int i = 0;

   if(!fp) return;

   fprintf(fp, "VOTE %d %d %d :%s\n", Vote.actif, Vote.nbprop, Vote.level, vote[Vote.nbprop].prop);
   for(;i < Vote.nbprop;++i) fprintf(fp, "PROP %d :%s\n", vote[i].voies, vote[i].prop);
   fclose(fp);
   return;
}

void load_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "r");
   char buf[300], *ar[5];
   int i = 0;

   if(!fp) return;
   while(fgets(buf, sizeof buf, fp))
   {
	   int items = split_buf(buf, ar, ASIZE(ar));
	   strip_newline(ar[items - 1]);
	   if(!strcmp(buf, "VOTE") && items > 4)
	   {
		   Vote.actif = strtol(ar[1], NULL, 10);
		   Vote.nbprop = strtol(ar[2], NULL, 10);
		   Vote.level = strtol(ar[3], NULL, 10);
		   Strncpy(vote[Vote.nbprop].prop, ar[4], MEMOLEN);
	   }
	   else if(!strcmp(buf, "PROP") && items > 2)
	   {
		   Strncpy(vote[i].prop, ar[2], MEMOLEN);
		   vote[i++].voies = strtol(ar[1], NULL, 10);
	   }
   }
   fclose(fp);
}

#endif
