/* src/web2cs-commands.c - Commandes du Web2CS
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: web2cs-commands.c,v 1.52 2004/02/07 16:21:48 romexzf Exp $
 */

#include "main.h"
#ifdef TESTBOT
#include <time.h>
#include <sys/time.h>
#include "web2cs.h"

extern const char *network;
extern const char *mailprog;
extern const char w2c_pass[];

struct w2c_commands w2c_cmds[] = {
 /* cmd,            function,             niveau, needarg, 0 */
 {  "LOGIN",		w2c_login,		0,		2, 0 },
 {  "REGISTER",		w2c_reg_user,		0,		3, 0 },
 {  "MEMOS",		w2c_memos,		1,		2, 0 },
 {  "MODUSER",		w2c_moduser,		1,		3, 1 },
 {  "CHANOPT",		w2c_chanopt,		400,		2, 1 },
 {  "CHANINFO",		w2c_chaninfo,		1,		1, 1 },
 {  "ACCESS",		w2c_access,		1,		1, 1 },
 {  "SET",		w2c_set,		1,		1, 0 },
 {  "USER",		w2c_user,		1,		2, 0 },
 {  "CHAN",		w2c_chan,		1,		2, 0 },
 {  "MYACCESS",		w2c_myaccess,		1,		0, 0 },
 {  "WHOISON",		w2c_whoison,		2,		1, 1 },
 {  "PASS",		w2c_passwd,		0,		1, 0 },
 {  "",			NULL,			0,		0, 0 }
};

int w2c_passwd(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	if(source->flag & W2C_ALLOWED) return 0;
	if(strcasecmp(parv[1], w2c_pass))
		return w2c_exit_client(source, "Non autoris");
	source->flag |= W2C_ALLOWED;
	return w2c_sendrpl(source, RPL_100);
}

int w2c_memos(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	aMemo *memo,*mt = NULL;
	int i = 0, p = 0, found = 0;
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "READ"))
	{
		if(parc<2)
			return w2c_sendrpl(source, RPL_102);

		if(is_num(parv[2])) i = atoi(parv[2]);
		for(memo=source->user->memohead;memo;memo=memo->next)
		{
			p++;
			if(p==i || !i)
			{
				w2c_sendrpl(source, RPL_290, memo->read, memo->de, memo->date, memo->message);
				found = memo->read = 1;
			}
		}
		if(found) return w2c_sendrpl(source, RPL_292, i ? 1 : p);
		else return w2c_sendrpl(source, RPL_291);
	}
	else if(!strcasecmp(cmd, "DEL"))
	{
		if(parc<2)
			return w2c_sendrpl(source, RPL_102);
		if(is_num(parv[2])) i = atoi(parv[2]);
		for(p = 1, memo = source->user->memohead;memo;p++, mt = memo, memo = memo->next)
		{
			if(!i)
			{
				free(mt);
				found++;
			}
			else if(p == i)
			{
				if(mt) mt->next = memo->next;
				else source->user->memohead = memo->next;
				free(memo);
				found = 1;
				if(p == i) break;
			}
		}
		if(!i && found)
		{
			free(mt);
			source->user->memohead = NULL;
		}
		write_anuser(source->user);
		if(found)
			return w2c_sendrpl(source, RPL_292, i ? 1 : p);
		else
			return w2c_sendrpl(source, RPL_291);
	}
	else if(!strcasecmp(cmd, "SEND"))
	{
		anUser *user;
		aNick *who;
		char message[MEMOLEN + 1];

		if(parc < 3) return w2c_sendrpl(source, RPL_102);
		if(!(user = getuserinfo(parv[2]))) return w2c_sendrpl(source, RPL_105, parv[2]);
		if(!strcasecmp(user->nick, source->user->nick))
			return w2c_sendrpl(source, RPL_101);
		if(UNoMemo(user) && !IsAnAdmin(source->user))
			return w2c_sendrpl(source, RPL_293);

		parv2msgn(parc, parv, 3, message, MEMOLEN);

		if((who = user->n) && !(who->flag & N_AWAY)) csntc(who, "\2MEMO DE W2C@%s :\2 %s", source->user->nick, message);

		for(memo = user->memohead;memo;memo = memo->next) if(!strcasecmp(memo->de, source->user->nick)) i++;
		if(i >= MAXMEMOS && !IsAnAdmin(source->user))
			return w2c_sendrpl(source, RPL_294, MAXMEMOS);

		add_memo(user, source->user->nick, time(NULL), message[0] == ':' ? message+1 : message, !!(who && !(who->flag & N_AWAY)));
		write_anuser(user);
	}
	else
		return w2c_sendrpl(source, RPL_101);
	return w2c_sendrpl(source, RPL_100);
}

int w2c_chanopt(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[2];
	if(!strcasecmp(cmd, "setwelcome"))
	{
		if(CSetWelcome(chaninfo)) DelCSetWelcome(chaninfo);
		else SetCSetWelcome(chaninfo);
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "setwelcome", CSetWelcome(chaninfo) ? 1 : 0);
	}
	if(!strcasecmp(cmd, "defmodes"))
	{
		struct cmode tmp = {0};
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(!string2scmode(&tmp, parv[3], parc > 3 ? parv[3] : NULL, parc > 4 ? parv[4] : NULL))
				return w2c_sendrpl(source, RPL_101);
		tmp.modes &= ~C_MOPERONLY;

		memcpy(&chaninfo->defmodes, &tmp, sizeof tmp);

		csmode(chaninfo, 0, "-%s", GetCModes(chaninfo->mode));
		csmode(chaninfo, 0, "+%s", GetCModes(chaninfo->defmodes));

		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "defmodes", GetCModes(chaninfo->defmodes));
	}
	if(!strcasecmp(cmd, "welcome"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(!strcasecmp(parv[3], "none"))
		{
			*chaninfo->welcome = '\0';
			DelCSetWelcome(chaninfo);
			write_achan(chaninfo);
			return w2c_sendrpl(source, RPL_281, "welcome", "none");
		}
		parv2msgn(parc, parv, 3, chaninfo->welcome, TOPICLEN);
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "welcome", chaninfo->welcome);
	}
	if(!strcasecmp(cmd, "deftopic"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(CLockTopic(chaninfo)) return w2c_sendrpl(source, RPL_282);

		parv2msgn(parc, parv, 3, chaninfo->deftopic, TOPICLEN);
		cstopic(chaninfo, chaninfo->deftopic);
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "deftopic", chaninfo->deftopic);
	}
	if(!strcasecmp(cmd, "motd"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(!strcasecmp(parv[3], "none"))
		{
			if(chaninfo->motd)
			{
				free(chaninfo->motd);
				chaninfo->motd = NULL;
			}
			write_achan(chaninfo);
			return w2c_sendrpl(source, RPL_281, "motd", "none");;
		}
		str_dup(&chaninfo->motd, parv2msg(parc, parv, 3, TOPICLEN));
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "motd", chaninfo->motd);
	}
	if(!strcasecmp(cmd, "description"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		parv2msgn(parc, parv, 3, chaninfo->description, DESCRIPTIONLEN);
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "description", chaninfo->description);
	}
	if(!strcasecmp(cmd, "chanurl"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);
		if(!strchr(parv[3], '.')) return w2c_sendrpl(source, RPL_101);

		Strncpy(chaninfo->url, parv[3], sizeof chaninfo->url);
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_281, "chanurl", chaninfo->url);
	}
	if(!strcasecmp(cmd, "banlevel"))
	{
		int l = ChanLevelbyUserI(source->user, chaninfo), n;
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(!is_num(parv[3])) return w2c_sendrpl(source, RPL_101);

		if(((n = atoi(parv[3])) > l || chaninfo->banlevel > l) && source->user->level < 2)
			return w2c_sendrpl(source, RPL_283);

		chaninfo->banlevel = n;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "banlevel", chaninfo->banlevel);
	}
	if(!strcasecmp(cmd, "bantype"))
	{
		int t;
		if(parc < 3) return w2c_sendrpl(source, RPL_102);
		if(!is_num(parv[3]) || ((t = atoi(parv[3])) > 4 || t < 1)) return w2c_sendrpl(source, RPL_101);
		chaninfo->bantype = t;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "bantype", chaninfo->bantype);
	}
	if(!strcasecmp(cmd, "strictop"))
	{
		chaninfo->flag ^= C_STRICTOP;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "strictop", !!CStrictOp(chaninfo));
	}
	if(!strcasecmp(cmd, "locktopic"))
	{
		chaninfo->flag ^= C_LOCKTOPIC;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "locktopic", !!CLockTopic(chaninfo));
	}
	if(!strcasecmp(cmd, "nobans"))
	{
		chaninfo->flag ^= C_NOBANS;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "nobans", !!CNoBans(chaninfo));
	}
	if(!strcasecmp(cmd, "noops"))
	{
		chaninfo->flag ^= C_NOOPS;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "noops", !!CNoOps(chaninfo));
	}
	if(!strcasecmp(cmd, "autoinvite"))
	{
		chaninfo->flag ^= C_AUTOINVITE;
		write_achan(chaninfo);
		return w2c_sendrpl(source, RPL_280, "autoinvite", !!CAutoInvite(chaninfo));
	}
	return w2c_sendrpl(source, RPL_104, cmd);
}

int w2c_reg_user(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	anUser *user;
	char passwd[20], mail[101], username[NICKLEN + 1]
#ifdef USE_MEMOSERV
,memo[MEMOLEN]
#endif
;
	time_t current = time(NULL);

	if(source->user) return w2c_sendrpl(source, RPL_101);

	Strncpy(username, parv[1], NICKLEN);
	Strncpy(mail, parv[2], 100);

	if(!IsValidNick(username) || source->user || !IsValidMail(mail) || strlen(parv[3]) < 6 || strlen(parv[3]) > 15)
		return w2c_sendrpl(source, RPL_101);
	if(IsBadNick(username) || (user = getuserinfo(username))) return w2c_sendrpl(source, RPL_250);
	strncpy(passwd, mkpass(parv[3]), 20);

	if(GetUserIbyMail(mail)) return w2c_sendrpl(source, RPL_251);

	user = add_regnick(username, passwd, current, 1, U_DEFAULT, mail, -1);

#ifdef USE_MEMOSERV
	snprintf(memo, MEMOLEN, "Bienvenue sur les services CoderZ.  Vous pouvez avoir la liste de leurs commandes en tapant \037/%s %s\037", cs.nick, RealCmd("showcommands"));
	add_memo(user, "Services", current, memo, 0);
#endif
	source->user = user;
#ifdef HAVE_VOTE
	user->vote = -1;
#endif
	user->lastseend = current;
	append_file(USERS_FILE, user->nick);
	write_anuser(user);
	return w2c_sendrpl(source, RPL_100);
}

int w2c_show_accessn(anAccess *access, anUser *user, struct w2c_clientinfo *source)
{
	w2c_sendrpl(source, RPL_230, access->c->chan, user->nick, access->level, user->n ? "Oui" : "Non");
	w2c_sendrpl(source, RPL_231, GetAccessOptions(access));
	w2c_sendrpl(source, RPL_232, access->lastseen);

	if(access->info) w2c_sendrpl(source, RPL_233, access->info);
	return w2c_sendrpl(source, RPL_234, user->nick);
}

int w2c_whoison(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	int size = 0;
	char buf[400] = {0}, tmp[NICKLEN + 16];
	aLink *lp = chaninfo->members;

	for(;lp;lp=lp->next)
	{
		int tmps = size, p;
		p = snprintf(tmp, sizeof tmp, " %s%s", GetChanPrefix(lp->value.j->nick, lp->value.j, 0), lp->value.j->nick->nick);
		size += p > 0 ? p : sizeof tmp;
		if(size >= 400)
		{
			strcat(buf, "...");
			w2c_sendrpl(source, RPL_222, buf);
			strcpy(buf, tmp);
			size = strlen(tmp);
		}
		else strcpy(&buf[tmps], tmp);
	}
	return w2c_sendrpl(source, RPL_222, buf);
}

/* parv[1] = login, parv[2] = pass; */
int w2c_login(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	anUser *user;

	if(source->user) return w2c_sendrpl(source, RPL_212, source->user->nick);
	if(!(user = getuserinfo(parv[1])) || !checkpass(parv[2], user)) return w2c_sendrpl(source, RPL_210);

	source->user = user;
	return w2c_sendrpl(source, RPL_211);
}

int w2c_chaninfo(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
  char options[300] = {0}, buf[10];

  if(chaninfo->mode.modes & (C_MPRIVATE | C_MSECRET) && (!source->user || !GetAccessIbyUserI(source->user, chaninfo))
		&& !IsAnAdmin(source->user)) w2c_sendrpl(source, RPL_107);

  w2c_sendrpl(source, RPL_221, "owner", chaninfo->owner ? chaninfo->owner->user->nick : "Aucun");
  w2c_sendrpl(source, RPL_221, "description", chaninfo->description);
  w2c_sendrpl(source, RPL_221, "defmodes", GetCModes(chaninfo->defmodes));
  w2c_sendrpl(source, RPL_221, "deftopic", chaninfo->deftopic);
  if(IsAnAdmin(source->user)) w2c_sendrpl(source, RPL_221, "motd", chaninfo->motd);
  if(IsAnAdmin(source->user) && CWantUnreg(chaninfo)) w2c_sendrpl(source, RPL_221, "unreg", "1");
  w2c_sendrpl(source, RPL_221, "url", chaninfo->url);

  if(CLockTopic(chaninfo)) strcat(options, " locktopic");
  if(CNoOps(chaninfo)) strcat(options, " noops");
  if(CNoBans(chaninfo)) strcat(options, " nobans");
  if(CStrictOp(chaninfo)) strcat(options, " strictop");
  if(CSetWelcome(chaninfo)) strcat(options, " welcome");
  if(CAutoInvite(chaninfo)) strcat(options, " autoinvite");
  if(IsSuspend(chaninfo)) strcat(options, " suspendu");
  if(CFLimit(chaninfo)) sprintf(options + strlen(options), " auto-limit(%d,%d)", chaninfo->limit_inc, chaninfo->limit_min);

  w2c_sendrpl(source, RPL_221, "options", options[0] != '\0' ? options : " Aucunes");
  snprintf(buf, 10, "%d", chaninfo->banlevel);
  w2c_sendrpl(source, RPL_221, "banlevel", buf);

  w2c_sendrpl(source, RPL_221, "bantype", GetBanType(chaninfo));
  if(CSetWelcome(chaninfo)) w2c_sendrpl(source, RPL_221, "welcome", chaninfo->welcome);
  if(IsSuspend(chaninfo)) w2c_sendrpl(source, RPL_221, "suspend", chaninfo->suspend->raison);
  w2c_sendrpl(source, RPL_221, "topic", chaninfo->topic);
  w2c_sendrpl(source, RPL_221, "modes", GetCModes(chaninfo->mode));
  if(*chaninfo->mode.key && IsAnAdmin(source->user)) w2c_sendrpl(source, RPL_221, "key", chaninfo->mode.key);
  if(IsAnAdmin(source->user)) w2c_whoison(source, chaninfo, parc, parv);
  return w2c_sendrpl(source, RPL_100);
}

int w2c_myaccess(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	char level[2] = {0}, chans[512] = {0}, b[REGCHANLEN + 20];
	int size = 0;
	anAccess *a;

	if(!source->user) return -1;

	if(parc == 2 && (!strcasecmp(parv[1], "ACCEPT") || !strcasecmp(parv[1], "REFUSE")))
	{
		if(!(source->user->flag & U_POKACCESS)) return w2c_sendrpl(source, RPL_310);
		for(a = source->user->accesshead;a && a->c != chaninfo;a = a->next);
		if(!a || !(a->flag & A_WAITACCESS)) return w2c_sendrpl(source, RPL_311);

		if(!strcasecmp(parv[1], "ACCEPT")) a->flag &= ~A_WAITACCESS;
		else if(!strcasecmp(parv[1], "REFUSE")) del_access(source->user, a->c);
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
	sprintf(level, "%d", source->user->level);

	w2c_sendrpl(source, RPL_240, source->user->nick);
	w2c_sendrpl(source, RPL_241, "mail", source->user->mail);
	w2c_sendrpl(source, RPL_241, "level", level);
	w2c_sendrpl(source, RPL_241, "options", GetUserOptions(source->user));

	for(a = source->user->accesshead;a;a=a->next)
	{
		snprintf(b, sizeof b, "%s %d ", a->c->chan, a->level);
		size += strlen(b);
		if(size >= 450)
		{
			w2c_sendrpl(source, RPL_242, chans);
			chans[0] = '\0';
			strcpy(chans, b);
			size = strlen(b);
		}
		else strcat(chans, b);
	}
	if(chans[0] != '\0') w2c_sendrpl(source, RPL_242, chans);
	return w2c_sendrpl(source, RPL_100);
}

int w2c_access(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	register aLink *lp;
	char *nick2 = parv[2], comp = '\0';
	int i = 0, autoop, protect, suspend, autovoice, l = 0, wild = 0;

	if(*nick2 == '>' || *nick2 == '<' || *nick2 == '=')
	{
		comp = *nick2++;
		if(!*nick2 || !is_num(nick2)) return w2c_sendrpl(source, RPL_101);
		l = atoi(nick2);
	}
	else wild = (strchr(nick2, '*') || strchr(nick2, '?'));
	autoop = getoption("-autoop", parv, parc, 2, -1);
	autovoice = getoption("-autovoice", parv, parc, 2, -1);
	protect = getoption("-protect", parv, parc, 2, -1);
	suspend = getoption("-suspend", parv, parc, 2, -1);

	for(lp = chaninfo->access;lp;lp=lp->next)
	{
		if(!AWait(lp->value.a)	&& (
			 (comp != '\0' && ((comp == '>' && lp->value.a->level > l) /* recherche */
			                 || (comp == '<' && lp->value.a->level < l)	/* via level */
			                 || (comp == '=' && l == lp->value.a->level)))
			  || (*nick2 == '*' && !*(nick2+1)) /* recherche sur '*' */
			  || (wild /* ou sur un vrai mask */
			  		&& !match(nick2, lp->value.a->user->nick))
			  || !strcasecmp(nick2, lp->value.a->user->nick)) /* ou access exact */
			&& (!autoop || AOp(lp->value.a)) /* ET match des options Aop/Avoice/Suspend etc */
			&& (!autovoice || AVoice(lp->value.a))
			&& (!protect || AProtect(lp->value.a))
			&& (!suspend || ASuspend(lp->value.a)))
		{
			w2c_show_accessn(lp->value.a, lp->value.a->user, source);
			i++;
		}
	}
	return w2c_sendrpl(source, RPL_235, i);
}

int w2c_moduser(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
   anUser *user;
   anAccess *access, *acces;
   const char *cmd = parv[2];

   if(!(user = getuserinfo(parv[3]))) return w2c_sendrpl(source, RPL_105, parv[3]);

   if(!(access = GetAccessIbyUserI(user, chaninfo))) return w2c_sendrpl(source, RPL_270, parv[1], user->nick);

   if(!strcasecmp(cmd, "protect"))
   {
	   access->flag ^= A_PROTECT;
	   write_anuser(user);
	   return w2c_sendrpl(source, RPL_271, "protect", AProtect(access) ? 1 : 0);
   }
   if(!strcasecmp(cmd, "autoop"))
   {
   	   if(!strcasecmp(source->user->nick, parv[3]) && access->level < 100)
		return w2c_sendrpl(source, RPL_107, "moduser", "op");
	   access->flag ^= A_OP;
	   write_anuser(user);
   	   return w2c_sendrpl(source, RPL_271, "autoop", AOp(access) ? 1 : 0);
   }
   if(!strcasecmp(cmd, "autovoice"))
   {
   	   if(!strcasecmp(source->user->nick, parv[3]) && access->level < 50)
		return w2c_sendrpl(source, RPL_107, "moduser", "voice");
	   access->flag ^= A_VOICE;
	   write_anuser(user);
   	   return w2c_sendrpl(source, RPL_271, "autovoice", AVoice(access) ? 1 : 0);
   }
   if(!strcasecmp(cmd, "suspend"))
   {
	   access->flag ^= A_SUSPEND;
	   write_anuser(user);
   	   return w2c_sendrpl(source, RPL_271, "suspend", ASuspend(access) ? 1 : 0);
   }
   if(!strcasecmp(cmd, "level"))
   {
	   int l;
	   if(parc < 4 || !is_num(parv[4]) || ((l = atoi(parv[4])) >= OWNERLEVEL || access->level == OWNERLEVEL || l < 5 || l > 499))	return w2c_sendrpl(source, RPL_101);
	   if(!IsAnAdmin(source->user))/*si c un admin pas de verif pr dep/dest*/
	   {
		   acces = GetAccessIbyUserI(source->user, chaninfo); /*on recup les infos de l'executant*/
		   if(l >= acces->level || acces->level <= access->level) return w2c_sendrpl(source, RPL_101);
	   }
	   access->level = l;
	   write_anuser(user);
	   return w2c_sendrpl(source, RPL_271, "level", access->level);
   }
   if(!strcasecmp(cmd, "info"))
   {
	   if(parc < 4) return w2c_sendrpl(source, RPL_102);
	   if(!strcasecmp(parv[4], "none"))
	   {
		   if(access->info)
		   {
			   free(access->info);
			   access->info = NULL;
		   }

	   }
	   else str_dup(&access->info, parv2msg(parc, parv, 4, 199));
	   write_anuser(user);
	   return w2c_sendrpl(source, RPL_272, NONE(access->info));
   }
   return w2c_sendrpl(source, RPL_104, "MODUSER", cmd);
}


int w2c_set(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv)
{
	const char *arg = parv[1];

	if(parc < 2) return w2c_sendrpl(source, RPL_102);
	if(!strcasecmp(arg, "MAIL"))
	{
		if(strlen(parv[2]) > MAILLEN || !IsValidMail(parv[2]) || !strcasecmp(source->user->mail, parv[2])) return w2c_sendrpl(source, RPL_101);

		if(GetUserIbyMail(parv[2])) return w2c_sendrpl(source, RPL_251);

		strcpy(source->user->mail, parv[2]);
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
	else if(!strcasecmp(arg, "virtualhost"))
	{
		if(!GetConf(CF_XMODE)) return 0;

		if(!strcasecmp(parv[2], "ON")) SetUWantX(source->user);
		else if(!strcasecmp(parv[2], "OFF")) DelUWantX(source->user);
		else return w2c_sendrpl(source, RPL_101);
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
#ifdef USE_NICKSERV
	else if(!strcasecmp(arg, "protect"))
	{
		int protect;
		if(parc < 2 || !is_num(parv[2]) || (protect = atoi(parv[2])) < 0 || protect < 0 || protect > 3)
			return w2c_sendrpl(source, RPL_101);

		if(protect == 1)
		{
			SetUPNick(source->user);
			if(UPKill(source->user)) DelUPKill(source->user);
		}
		else if(protect == 2)
		{
			SetUPKill(source->user);
			if(UPNick(source->user)) DelUPNick(source->user);
		}
		else
		{
			if(UPNick(source->user)) DelUPNick(source->user);
			if(UPKill(source->user)) DelUPKill(source->user);
		}
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
#endif
#ifdef USE_MEMOSERV
	else if(!strcasecmp(arg, "nomemo"))
	{
		if(!strcasecmp(parv[2], "ON")) SetUNoMemo(source->user);
		else if(!strcasecmp(parv[2], "OFF")) DelUNoMemo(source->user);
		else return w2c_sendrpl(source, RPL_101);

		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
#endif
	else if(!strcasecmp(arg, "pass"))
	{
		char p[15];

		strncpy(p,parv[2],15);
		if(strlen(p) < 6) return w2c_sendrpl(source, RPL_101);

		strcpy(source->user->passwd, mkpass(p));
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
	/*else if(!strcasecmp(arg, "username"))
	{
		anUser *u;

		if(strlen(parv[2]) >= NICKLEN || !IsValidNick(parv[2])) return w2c_sendrpl(source, RPL_101);
		if(IsBadNick(parv[2]) || (u=getuserinfo(parv[2]))) return w2c_sendrpl(source, RPL_250);

		File_UserDel(source->user->nick);
		if(source->user->n) cs_account(source->user->n, parv[2]);
		strncpy(source->user->nick, parv[2], NICKLEN);
		write_anuser(source->user);
		write_userlist();
		return w2c_sendrpl(source, RPL_100);
	}*/
	else if(!strcasecmp(arg, "access"))
	{
		int i;
		anAccess *a, *at;
		if((i = atoi(parv[2])) < 0 || i > 2) return w2c_sendrpl(source, RPL_101);

		switch(i)
		{
			case 0:
			source->user->flag &= ~(U_POKACCESS | U_PACCEPT);
			source->user->flag |= U_PREJECT;
			for(a = source->user->accesshead;a;a = at)
			{
				at = a->next;
				del_access(source->user, a->c);
			}
			break;
			case 1:
			source->user->flag &= ~(U_PACCEPT | U_PREJECT);
			source->user->flag |= U_POKACCESS;
			break;
			case 2:
			source->user->flag &= ~(U_PREJECT | U_POKACCESS);
			source->user->flag |= U_PACCEPT;
			for(a = source->user->accesshead;a;a = a->next) a->flag &= ~A_WAITACCESS;
			break;
		}
		write_anuser(source->user);
		return w2c_sendrpl(source, RPL_100);
	}
	return w2c_sendrpl(source, RPL_104, parv[2]);
}

int w2c_user(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv) {

	const char *arg = parv[1], *user = parv[2];
	anUser *u = NULL;
	anAccess *a, *a2;
	aNick *n;

	/*if(!strcasecmp(arg, "suspend"))
	{
		if(!strcasecmp(source->user->nick, user)) return w2c_sendrpl(source, RPL_101);
		if(!(u=getuserinfo(user))) return w2c_sendrpl(source, RPL_106, user);
		if(IsAnAdmin(u) && u->level >= source->user->level) return w2c_sendrpl(source, RPL_107);

		u->suspendu = (u->suspendu > 0) ? (-1 * time(NULL)) : time(NULL);
		write_anuser(u);
		return w2c_sendrpl(source, RPL_100);
	}*/

	if(!strcasecmp(arg, "mail"))
	{
		if(parc < 3 || !IsValidMail(parv[3])) return w2c_sendrpl(source, RPL_101);
		if(!(u=getuserinfo(user))) return w2c_sendrpl(source, RPL_106, user);

		Strncpy(u->mail, parv[3], MAILLEN);
		write_anuser(u);
		return w2c_sendrpl(source, RPL_100);
	}

	if(!strcasecmp(arg, "pass"))
	{
		char p[15];

		if(parc < 3) return w2c_sendrpl(source, RPL_102);
		strncpy(p,parv[3],15);

		if(strlen(p) < 6) return w2c_sendrpl(source, RPL_101);
		if(!(u=getuserinfo(user))) return w2c_sendrpl(source, RPL_106, user);
		if(IsAnAdmin(u) && u->level >= source->user->level && strcasecmp(u->nick,source->user->nick))
			return w2c_sendrpl(source, RPL_107);

		strcpy(u->passwd, mkpass(p));
		write_anuser(u);
		return w2c_sendrpl(source, RPL_100);
	}

	if(!strcasecmp(arg, "username"))
	{
		if(parc < 3) return w2c_sendrpl(source, RPL_102);

		if(strlen(parv[3]) >= NICKLEN || !IsValidNick(parv[3])) return w2c_sendrpl(source, RPL_101);
		if(IsBadNick(parv[3]) || (u=getuserinfo(parv[3]))) return w2c_sendrpl(source, RPL_250);
		if(!(u=getuserinfo(user))) return w2c_sendrpl(source, RPL_106, user);
		if(IsAnAdmin(u) && u->level >= source->user->level && strcasecmp(u->nick,source->user->nick))
			return w2c_sendrpl(source, RPL_107);

		File_UserDel(source->user->nick);
		if(source->user->n) cs_account(source->user->n, parv[3]);
		strncpy(source->user->nick, parv[3], NICKLEN);
		write_anuser(source->user);
		write_userlist();

		return w2c_sendrpl(source, RPL_100);
	}

	if(!strcasecmp(arg, "del"))
	{
		if(!strcasecmp(source->user->nick, user)) return w2c_sendrpl(source, RPL_101);
		if(!(u=getuserinfo(user))) return w2c_sendrpl(source, RPL_106, user);
		if(IsAnAdmin(u) && u->level >= source->user->level) return w2c_sendrpl(source, RPL_107);

		if((n = u->n))
		{
			char raison[200] = {0};
			if(parc > 2) parv2msgn(parc, parv, 3, raison, 200);

			csntc(n, "Votre Username vient d'tre supprim par l'Admin %s %s", source->user->nick, raison);
			cs_account(n, NULL);
		}

		for(a=u->accesshead;a;a=a2)
		{
			a2 = a->next;
			if(a->level == OWNERLEVEL)
			{
				char tmp[400];
				snprintf(tmp, sizeof tmp, "Salon %s desenregistr par %s (\2Suppression de l'username de l'owner\2)",
					a->c->chan, source->user->nick);
				del_chan(a->c, tmp);
			}
		}
		del_regnick(u);
		write_userlist();

		return w2c_sendrpl(source, RPL_100);
	}

	return w2c_sendrpl(source, RPL_104, parv[2]);
}

int w2c_chan(struct w2c_clientinfo *source, aChan *chaninfo, int parc, char **parv) { return 0; }
#endif

