/* src/welcome.c - Diverses commandes sur le module welcome
 * Copyright (C) 2002-2004 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: welcome.c,v 1.23 2004/12/24 17:49:22 romexzf Exp $
 */

#include "main.h"
#ifdef USE_WELCOMESERV
#include "cs_cmds.h"
#include "debug.h"
#include "config.h"
#include "outils.h"
#include "welcome.h"

struct welcomeinfo *welcomehead = NULL;
static int WelcomeCount = 0;

/*
 * set_welcome on/off
 */

int set_welcome(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	switch_option(nick, parc ? parv[1] : NULL, "welcome", "serveur", &ConfFlag, CF_WELCOME);
	write_welcome();
	return 1;
}

/* addwelcome <level> <message> */

int add_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *w;
	int level;

	if(!is_num(parv[1]) || (level = atoi(parv[1])) < 1 || level > 100)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if((w = add_welcometolist(level, parv2msg(parc, parv, 2, 250))))
	{
		csntc(nick, "Welcome bien ajout au level %d", level);
		csntc(nick, "\2[Message de bienvenue #%d]\2 - %s", w->id, w->msg);
		write_welcome();
	}
	return 1;
}

int del_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo **tmpp = &welcomehead, *tmp;
	int id;

	if(!is_num(parv[1])) return csntc(nick, "L'id doit tre un nombre.");

	for(id = atoi(parv[1]); (tmp = *tmpp); tmpp = &tmp->next)
		if(id == tmp->id)
		{
			*tmpp = tmp->next;
			free(tmp);
			id = -1; /* mark it as found */
			break;
		}

	if(id >= 0)	return csntc(nick, "Aucun message ayant pour id %d", id);

	for(--WelcomeCount, tmp = welcomehead; tmp; tmp = tmp->next)
		if(tmp->id >= WelcomeCount) --tmp->id;

	csntc(nick, "Le message #%s a bien t supprim (%d messages au total)", parv[1], WelcomeCount);
	write_welcome();
	return 1;
}

int list_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *w = welcomehead;

	if(!w) return csntc(nick, "Aucun message de bienvenue.");

	csntc(nick, "\2#Id Level Message");
	for(;w;w = w->next) csntc(nick, "#%d %d %s", w->id, w->level, w->msg);

	return 1;
}

void choose_welcome(const char *num)
{
	struct welcomeinfo *tmp = welcomehead, *last = tmp;
	int rnd;

	if(!tmp) return;

	for(rnd = rand() % 100;tmp; tmp = tmp->next)
		if(tmp->level >= rnd)
		{
			last = tmp;
			if((float) tmp->level/rnd > 2) break;
		}

	putserv("%s %s %s :\2[Message de bienvenue #%d]\2 - %s", ws.num,
		GetConf(CF_PRIVWELCOME) ? TOKEN_PRIVMSG : TOKEN_NOTICE, num, last->id, last->msg);
}

void write_welcome(void)
{
	FILE *fp = fopen(WELCOME_FILE, "w");
	struct welcomeinfo *w = welcomehead;

	if(!fp) return;

	fprintf(fp, "%d\n%s\n%s\n", GetConf(CF_WELCOME), user_motd, admin_motd);
	for(;w;w = w->next) fprintf(fp, "%d :%s\n", w->level, w->msg);
   	fclose(fp);
}

struct welcomeinfo *add_welcometolist(int level, const char *msg)
{
	struct welcomeinfo *w = welcomehead, *new = malloc(sizeof *w);

   	if(!new)
   	{
	   	Debug(W_MAX, "add_loadwelcome, malloc a chou pour le welcomeinfo %s", msg);
	   	return NULL;
   	}

	Strncpy(new->msg, msg, 250);
	new->level = level;
	new->id = ++WelcomeCount;

	if(1 == WelcomeCount || level >= w->level)
	{
		new->next = w;
		welcomehead = new;
	}
	else {
		for(;w->next && level < w->next->level;w = w->next);
		new->next = w->next;
		w->next = new;
	}

	return new;
}

void load_welcome(void)
{
	char buf[420], *tmp;
	int t = 1;
	FILE *fp = fopen(WELCOME_FILE, "r");

	if(!fp) return;

	while(fgets(buf, sizeof buf, fp))
   	{
		strip_newline(buf);
		if(!strncmp(buf, "VERSION", 7)) continue;
		else if(t == 1) { if(atoi(buf)) ConfFlag |= CF_WELCOME; }
		else if(t == 2) Strncpy(user_motd, buf, 390);
		else if(t == 3) Strncpy(admin_motd, buf, 390);
		else if((tmp = strchr(buf, ' ')))
		{
			*tmp++ = 0;
			add_welcometolist(atoi(buf), ++tmp);
		}
		t++;
	}
	fclose(fp);
}

#endif
